@@ -1,20 +1,15 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 #ifndef ZABBIX_MODULE_H
@@ -64,6 +59,7 @@ typedef struct
 	zbx_uint64_t			lastlogsize;
 	int				mtime;
 	zbx_request_parameter_type_t	*types;
+	int				timeout;
 }
 AGENT_REQUEST;
 
@@ -85,8 +81,11 @@ zbx_log_t;
 #define AR_LOG		0x10
 #define AR_MESSAGE	0x20
 #define AR_META		0x40
+#define AR_BIN		0x80
 
-/* agent return structure */
+/* Agent return structure.                                       */
+/* Need to preserve the compatibility with previous versions,    */
+/* so new fields must be added at the end of the struct.         */
 typedef struct
 {
 	zbx_uint64_t	lastlogsize;	/* meta information */
@@ -98,6 +97,7 @@ typedef struct
 	zbx_log_t	*log;
 	int		type;		/* flags: see AR_* above */
 	int		mtime;		/* meta information */
+	char		*bin;
 }
 AGENT_RESULT;
 
@@ -108,7 +108,10 @@ typedef struct
 	int		(*function)(AGENT_REQUEST *request, AGENT_RESULT *result);
 	char		*test_param;	/* item test parameters; user parameter items keep command here */
 }
-ZBX_METRIC;
+zbx_metric_t;
+
+/* for backward-compatibility */
+#define ZBX_METRIC	zbx_metric_t
 
 /* SET RESULT */
 
@@ -152,6 +155,129 @@ ZBX_METRIC;
 	(res)->msg = (char *)(val)		\
 )
 
+/* CHECK RESULT */
+
+#define ZBX_ISSET_UI64(res)	((res)->type & AR_UINT64)
+#define ZBX_ISSET_DBL(res)	((res)->type & AR_DOUBLE)
+#define ZBX_ISSET_STR(res)	((res)->type & AR_STRING)
+#define ZBX_ISSET_TEXT(res)	((res)->type & AR_TEXT)
+#define ZBX_ISSET_LOG(res)	((res)->type & AR_LOG)
+#define ZBX_ISSET_MSG(res)	((res)->type & AR_MESSAGE)
+#define ZBX_ISSET_META(res)	((res)->type & AR_META)
+#define ZBX_ISSET_BIN(res)	((res)->type & AR_BIN)
+
+#define ZBX_ISSET_VALUE(res)	((res)->type & (AR_UINT64 | AR_DOUBLE | AR_STRING | AR_TEXT | AR_LOG))
+
+/* UNSET RESULT */
+
+#define ZBX_UNSET_UI64_RESULT(res)					\
+									\
+do									\
+{									\
+	(res)->type &= ~AR_UINT64;					\
+	(res)->ui64 = (zbx_uint64_t)0;					\
+}									\
+while (0)
+
+#define ZBX_UNSET_DBL_RESULT(res)					\
+									\
+do									\
+{									\
+	(res)->type &= ~AR_DOUBLE;					\
+	(res)->dbl = (double)0;						\
+}									\
+while (0)
+
+#define ZBX_UNSET_STR_RESULT(res)					\
+									\
+do									\
+{									\
+	if ((res)->type & AR_STRING)					\
+	{								\
+		zbx_free((res)->str);					\
+		(res)->type &= ~AR_STRING;				\
+	}								\
+}									\
+while (0)
+
+#define ZBX_UNSET_TEXT_RESULT(res)					\
+									\
+do									\
+{									\
+	if ((res)->type & AR_TEXT)					\
+	{								\
+		zbx_free((res)->text);					\
+		(res)->type &= ~AR_TEXT;				\
+	}								\
+}									\
+while (0)
+
+#define ZBX_UNSET_LOG_RESULT(res)					\
+									\
+do									\
+{									\
+	if ((res)->type & AR_LOG)					\
+	{								\
+		zbx_free((res)->log->source);				\
+		zbx_free((res)->log->value);				\
+		zbx_free((res)->log);					\
+		(res)->type &= ~AR_LOG;					\
+	}								\
+}									\
+while (0)
+
+#define ZBX_UNSET_MSG_RESULT(res)					\
+									\
+do									\
+{									\
+	if ((res)->type & AR_MESSAGE)					\
+	{								\
+		zbx_free((res)->msg);					\
+		(res)->type &= ~AR_MESSAGE;				\
+	}								\
+}									\
+while (0)
+
+#define ZBX_UNSET_BIN_RESULT(res)					\
+									\
+do									\
+{									\
+	if ((res)->type & AR_BIN)					\
+	{								\
+		zbx_free((res)->bin);					\
+		(res)->type &= ~AR_BIN	;				\
+	}								\
+}									\
+while (0)
+
+/* AR_META is always excluded */
+#define ZBX_UNSET_RESULT_EXCLUDING(res, exc_type)				\
+										\
+do										\
+{										\
+	if (!(exc_type & AR_UINT64))	ZBX_UNSET_UI64_RESULT(res);		\
+	if (!(exc_type & AR_DOUBLE))	ZBX_UNSET_DBL_RESULT(res);		\
+	if (!(exc_type & AR_STRING))	ZBX_UNSET_STR_RESULT(res);		\
+	if (!(exc_type & AR_TEXT))	ZBX_UNSET_TEXT_RESULT(res);		\
+	if (!(exc_type & AR_LOG))	ZBX_UNSET_LOG_RESULT(res);		\
+	if (!(exc_type & AR_MESSAGE))	ZBX_UNSET_MSG_RESULT(res);		\
+	if (!(exc_type & AR_BIN))	ZBX_UNSET_BIN_RESULT(res);		\
+}										\
+while (0)
+
+#define	zbx_init_agent_result(result)	{memset((result), 0, sizeof(AGENT_RESULT));}
+
+#define	zbx_free_agent_result(result)		\
+{						\
+	ZBX_UNSET_UI64_RESULT((result));	\
+	ZBX_UNSET_DBL_RESULT((result));		\
+	ZBX_UNSET_STR_RESULT((result));		\
+	ZBX_UNSET_TEXT_RESULT((result));	\
+	ZBX_UNSET_LOG_RESULT((result));		\
+	ZBX_UNSET_MSG_RESULT((result));		\
+	ZBX_UNSET_BIN_RESULT(result);		\
+}
+
 #define SYSINFO_RET_OK		0
 #define SYSINFO_RET_FAIL	1
 
