@@ -1,55 +1,33 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 #ifndef ZABBIX_ZBXDB_H
 #define ZABBIX_ZBXDB_H
 
-#include "common.h"
+#include "zbxcommon.h"
 #include "zbxjson.h"
 
 #define ZBX_DB_OK	0
 #define ZBX_DB_FAIL	-1
 #define ZBX_DB_DOWN	-2
 
-#define ZBX_DB_WAIT_DOWN	10
-
-#define ZBX_MAX_SQL_SIZE	262144	/* 256KB */
-#ifndef ZBX_MAX_OVERFLOW_SQL_SIZE
-#	ifdef HAVE_ORACLE
-		/* Do not use "overflowing" (multi-statement) queries for Oracle. */
-		/* Zabbix benefits from cursor_sharing=force Oracle parameter */
-		/* which doesn't apply to PL/SQL blocks. */
-#		define ZBX_MAX_OVERFLOW_SQL_SIZE	0
-#	else
-#		define ZBX_MAX_OVERFLOW_SQL_SIZE	ZBX_MAX_SQL_SIZE
-#	endif
-#elif 0 != ZBX_MAX_OVERFLOW_SQL_SIZE && \
-		(1024 > ZBX_MAX_OVERFLOW_SQL_SIZE || ZBX_MAX_OVERFLOW_SQL_SIZE > ZBX_MAX_SQL_SIZE)
-#error ZBX_MAX_OVERFLOW_SQL_SIZE is out of range
-#endif
-
 #define ZBX_DB_TLS_CONNECT_REQUIRED_TXT		"required"
 #define ZBX_DB_TLS_CONNECT_VERIFY_CA_TXT	"verify_ca"
 #define ZBX_DB_TLS_CONNECT_VERIFY_FULL_TXT	"verify_full"
 
-typedef char	**DB_ROW;
-typedef struct zbx_db_result	*DB_RESULT;
+typedef char	**zbx_db_row_t;
+typedef struct zbx_db_result	*zbx_db_result_t;
 
 /* database field value */
 typedef union
@@ -61,6 +39,24 @@ typedef union
 }
 zbx_db_value_t;
 
+typedef struct
+{
+	char	*config_dbhost;
+	char	*config_dbname;
+	char	*config_dbschema;
+	char	*config_dbuser;
+	char	*config_dbpassword;
+	char	*config_dbsocket;
+	char	*config_db_tls_connect;
+	char	*config_db_tls_cert_file;
+	char	*config_db_tls_key_file;
+	char	*config_db_tls_ca_file;
+	char	*config_db_tls_cipher;
+	char	*config_db_tls_cipher_13;
+	int	config_dbport;
+}
+zbx_config_dbhigh_t;
+
 #ifdef HAVE_SQLITE3
 	/* we have to put double % here for sprintf */
 #	define ZBX_SQL_MOD(x, y) #x "%%" #y
@@ -74,39 +70,40 @@ zbx_db_value_t;
 #	define ZBX_FOR_UPDATE	" for update"
 #endif
 
-#ifdef HAVE_MULTIROW_INSERT
-#	define ZBX_ROW_DL	","
-#else
-#	define ZBX_ROW_DL	";\n"
-#endif
+int	zbx_db_init_basic(const char *dbname, const char *const dbschema, int log_slow_queries, char **error);
+void	zbx_db_deinit_basic(void);
 
-int	zbx_db_init(const char *dbname, const char *const dbschema, char **error);
-void	zbx_db_deinit(void);
+void	zbx_db_init_autoincrement_options_basic(void);
 
-void	zbx_db_init_autoincrement_options(void);
+int	zbx_db_connect_basic(const zbx_config_dbhigh_t *cfg);
+void	zbx_db_close_basic(void);
 
-int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *dbschema, char *dbsocket, int port,
-			char *tls_connect, char *cert, char *key, char *ca, char *cipher, char *cipher_13);
-void	zbx_db_close(void);
-
-int	zbx_db_begin(void);
-int	zbx_db_commit(void);
-int	zbx_db_rollback(void);
+int	zbx_db_begin_basic(void);
+int	zbx_db_commit_basic(void);
+int	zbx_db_rollback_basic(void);
 int	zbx_db_txn_level(void);
 int	zbx_db_txn_error(void);
 int	zbx_db_txn_end_error(void);
 const char	*zbx_db_last_strerr(void);
+
+typedef enum
+{
+	ERR_Z3001 = 3001,
+	ERR_Z3002,
+	ERR_Z3003,
+	ERR_Z3004,
+	ERR_Z3005,
+	ERR_Z3006,
+	ERR_Z3007,
+	ERR_Z3008,
+	ERR_Z3009
+}
+zbx_err_codes_t;
+
 zbx_err_codes_t	zbx_db_last_errcode(void);
 
 #ifdef HAVE_POSTGRESQL
 int	zbx_tsdb_get_version(void);
-char	*zbx_tsdb_get_license(void);
-#define ZBX_DB_TSDB_V1	(20000 > zbx_tsdb_get_version())
-int	zbx_tsdb_table_has_compressed_chunks(const char *table_names);
-#define ZBX_TSDB1_HISTORY_TABLES "'history_uint'::regclass,'history_log'::regclass,'history_str'::regclass,'history_text'::regclass,'history'::regclass"
-#define ZBX_TSDB2_HISTORY_TABLES "'history_uint','history_log','history_str','history_text','history'"
-#define ZBX_TSDB1_TRENDS_TABLES "'trends'::regclass,'trends_uint'::regclass"
-#define ZBX_TSDB2_TRENDS_TABLES "'trends','trends_uint'"
 #endif
 
 #ifdef HAVE_ORACLE
@@ -127,19 +124,27 @@ typedef struct
 }
 zbx_db_bind_context_t;
 
-int		zbx_db_statement_prepare(const char *sql);
+int		zbx_db_statement_prepare_basic(const char *sql);
 int		zbx_db_bind_parameter_dyn(zbx_db_bind_context_t *context, int position, unsigned char type,
 				zbx_db_value_t **rows, int rows_num);
 void		zbx_db_clean_bind_context(zbx_db_bind_context_t *context);
 int		zbx_db_statement_execute(int iters);
 #endif
-int		zbx_db_vexecute(const char *fmt, va_list args);
-DB_RESULT	zbx_db_vselect(const char *fmt, va_list args);
-DB_RESULT	zbx_db_select_n(const char *query, int n);
 
-DB_ROW		zbx_db_fetch(DB_RESULT result);
-void		DBfree_result(DB_RESULT result);
-int		zbx_db_is_null(const char *field);
+#if defined (HAVE_MYSQL)
+void	zbx_mysql_escape_bin(const char *src, char *dst, size_t size);
+#elif defined(HAVE_POSTGRESQL)
+void	zbx_postgresql_escape_bin(const char *src, char **dst, size_t size);
+#endif
+
+int		zbx_db_vexecute(const char *fmt, va_list args);
+zbx_db_result_t	zbx_db_vselect(const char *fmt, va_list args);
+zbx_db_result_t	zbx_db_select_n_basic(const char *query, int n);
+
+int		zbx_db_get_row_num(zbx_db_result_t result);
+zbx_db_row_t		zbx_db_fetch_basic(zbx_db_result_t result);
+void		zbx_db_free_result(zbx_db_result_t result);
+int		zbx_db_is_null_basic(const char *field);
 
 typedef enum
 {
@@ -147,10 +152,10 @@ typedef enum
 	ESCAPE_SEQUENCE_ON
 }
 zbx_escape_sequence_t;
-char		*zbx_db_dyn_escape_string(const char *src, size_t max_bytes, size_t max_chars,
+char		*zbx_db_dyn_escape_string_basic(const char *src, size_t max_bytes, size_t max_chars,
 		zbx_escape_sequence_t flag);
 #define ZBX_SQL_LIKE_ESCAPE_CHAR '!'
-char		*zbx_db_dyn_escape_like_pattern(const char *src);
+char		*zbx_db_dyn_escape_like_pattern_basic(const char *src);
 
 int		zbx_db_strlen_n(const char *text_loc, size_t maxlen);
 
@@ -189,7 +194,7 @@ typedef enum
 {	/* db extension error codes shared with FRONTEND */
 	ZBX_EXT_ERR_UNDEFINED = 0,
 	ZBX_EXT_SUCCEED = 1,
-	/*ZBX_TIMESCALEDB_POSTGRES_TOO_OLD, obsoleted since Zabbix 7.0 */
+	/* ZBX_TIMESCALEDB_POSTGRES_TOO_OLD, obsoleted since Zabbix 7.0 */
 	ZBX_TIMESCALEDB_VERSION_FAILED_TO_RETRIEVE = 3,
 	ZBX_TIMESCALEDB_VERSION_LOWER_THAN_MINIMUM,
 	ZBX_TIMESCALEDB_VERSION_NOT_SUPPORTED,
@@ -198,8 +203,6 @@ typedef enum
 }
 zbx_db_ext_err_code_t;
 
-zbx_uint32_t	zbx_dbms_version_get(void);
-
 struct zbx_db_version_info_t
 {
 	/* information about database server */
@@ -236,7 +239,6 @@ struct zbx_db_version_info_t
 
 	zbx_db_version_status_t	ext_flag;
 
-	char			*ext_lic;
 	zbx_db_ext_err_code_t	ext_err_code;
 
 	int			history_compressed_chunks;
@@ -246,19 +248,12 @@ struct zbx_db_version_info_t
 #endif
 };
 
-#if defined(HAVE_POSTGRESQL)
-void	zbx_tsdb_update_dbversion_info(struct zbx_db_version_info_t *db_version_info);
-#endif
-
 void	zbx_dbms_version_info_extract(struct zbx_db_version_info_t *version_info);
 #ifdef HAVE_POSTGRESQL
 void	zbx_tsdb_info_extract(struct zbx_db_version_info_t *version_info);
 void	zbx_tsdb_set_compression_availability(int compression_availabile);
 int	zbx_tsdb_get_compression_availability(void);
-#endif
-
-#ifdef HAVE_MYSQL
-int	zbx_dbms_mariadb_used(void);
+void	zbx_tsdb_extract_compressed_chunk_flags(struct zbx_db_version_info_t *version_info);
 #endif
 
 int	zbx_db_version_check(const char *database, zbx_uint32_t current_version, zbx_uint32_t min_version,
@@ -267,10 +262,16 @@ void	zbx_db_version_json_create(struct zbx_json *json, struct zbx_db_version_inf
 
 #if defined(HAVE_MYSQL)
 #	define ZBX_DB_TIMESTAMP()	"unix_timestamp()"
+#	define ZBX_DB_CHAR_LENGTH(str)	"char_length(" #str ")"
 #elif defined(HAVE_POSTGRESQL)
 #	define ZBX_DB_TIMESTAMP()	"cast(extract(epoch from now()) as int)"
-#else /* HAVE_ORACLE */
+#	define ZBX_DB_CHAR_LENGTH(str)	"char_length(" #str ")"
+#elif defined(HAVE_ORACLE)
 #	define ZBX_DB_TIMESTAMP()	"(cast(sys_extract_utc(systimestamp) as date) - date'1970-01-01') * 86400"
+#	define ZBX_DB_CHAR_LENGTH(str)	"length(" #str ")"
+#else
+#	define ZBX_DB_TIMESTAMP()	"cast(strftime('%s', 'now') as integer)"
+#	define ZBX_DB_CHAR_LENGTH(str)	"length(" #str ")"
 #endif
 
 #endif
