@@ -1,41 +1,46 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 package main
 
 import (
 	"errors"
-	"flag"
 	"fmt"
 	"os"
 	"path/filepath"
 	"sync"
 	"time"
 
-	"git.zabbix.com/ap/plugin-support/log"
 	"golang.org/x/sys/windows/svc"
 	"golang.org/x/sys/windows/svc/eventlog"
 	"golang.org/x/sys/windows/svc/mgr"
-	"zabbix.com/internal/agent"
-	"zabbix.com/internal/agent/keyaccess"
-	"zabbix.com/internal/agent/scheduler"
-	"zabbix.com/internal/monitor"
+	"golang.zabbix.com/agent2/internal/agent"
+	"golang.zabbix.com/agent2/internal/agent/keyaccess"
+	"golang.zabbix.com/agent2/internal/agent/scheduler"
+	"golang.zabbix.com/agent2/internal/monitor"
+	"golang.zabbix.com/sdk/errs"
+	"golang.zabbix.com/sdk/log"
+	"golang.zabbix.com/sdk/zbxflag"
+)
+
+const usageMessageExampleConfPath = `C:\zabbix\zabbix_agent2.conf`
+
+const (
+	startTypeAutomatic = "automatic"
+	startTypeDelayed   = "delayed"
+	startTypeManual    = "manual"
+	startTypeDisabled  = "disabled"
 )
 
 var (
@@ -46,6 +51,7 @@ var (
 	svcStartFlag         bool
 	svcStopFlag          bool
 	svcMultipleAgentFlag bool
+	svcStartType         string
 
 	winServiceRun bool
 
@@ -59,45 +65,77 @@ var (
 	stopChan      = make(chan bool)
 )
 
-func loadOSDependentFlags() {
-	const (
-		svcInstallDefault     = false
-		svcInstallDescription = "Install Zabbix agent 2 as service"
-	)
-	flag.BoolVar(&svcInstallFlag, "install", svcInstallDefault, svcInstallDescription)
-	flag.BoolVar(&svcInstallFlag, "i", svcInstallDefault, svcInstallDescription+" (shorthand)")
+func osDependentFlags() zbxflag.Flags {
+	return zbxflag.Flags{
+		&zbxflag.BoolFlag{
+			Flag: zbxflag.Flag{
+				Name:      "multiple-agents",
+				Shorthand: "m",
+				Description: "For -i -d -s -x functions service name will " +
+					"include Hostname parameter specified in configuration file",
+			},
+			Default: false,
+			Dest:    &svcMultipleAgentFlag,
+		},
 
-	const (
-		svcUninstallDefault     = false
-		svcUninstallDescription = "Uninstall Zabbix agent 2 from service"
-	)
-	flag.BoolVar(&svcUninstallFlag, "uninstall", svcUninstallDefault, svcUninstallDescription)
-	flag.BoolVar(&svcUninstallFlag, "d", svcUninstallDefault, svcUninstallDescription+" (shorthand)")
-
-	const (
-		svcStartDefault     = false
-		svcStartDescription = "Start Zabbix agent 2 service"
-	)
-	flag.BoolVar(&svcStartFlag, "start", svcStartDefault, svcStartDescription)
-	flag.BoolVar(&svcStartFlag, "s", svcStartDefault, svcStartDescription+" (shorthand)")
-
-	const (
-		svcStopDefault     = false
-		svcStopDescription = "Stop Zabbix agent 2 service"
-	)
-	flag.BoolVar(&svcStopFlag, "stop", svcStopDefault, svcStopDescription)
-	flag.BoolVar(&svcStopFlag, "x", svcStopDefault, svcStopDescription+" (shorthand)")
-
-	const (
-		svcMultipleDefault     = false
-		svcMultipleDescription = "For -i -d -s -x functions service name will\ninclude Hostname parameter specified in\nconfiguration file"
-	)
-	flag.BoolVar(&svcMultipleAgentFlag, "multiple-agents", svcMultipleDefault, svcMultipleDescription)
-	flag.BoolVar(&svcMultipleAgentFlag, "m", svcMultipleDefault, svcMultipleDescription+" (shorthand)")
+		&zbxflag.BoolFlag{
+			Flag: zbxflag.Flag{
+				Name:        "install",
+				Shorthand:   "i",
+				Description: "Install Zabbix agent 2 as service",
+			},
+			Default: false,
+			Dest:    &svcInstallFlag,
+		},
+		&zbxflag.BoolFlag{
+			Flag: zbxflag.Flag{
+				Name:        "uninstall",
+				Shorthand:   "d",
+				Description: "Uninstall Zabbix agent 2 from service",
+			},
+			Default: false,
+			Dest:    &svcUninstallFlag,
+		},
+		&zbxflag.BoolFlag{
+			Flag: zbxflag.Flag{
+				Name:        "start",
+				Shorthand:   "s",
+				Description: "Start Zabbix agent 2 service",
+			},
+			Default: false,
+			Dest:    &svcStartFlag,
+		},
+		&zbxflag.BoolFlag{
+			Flag: zbxflag.Flag{
+				Name:        "stop",
+				Shorthand:   "x",
+				Description: "Stop Zabbix agent 2 service",
+			},
+			Default: false,
+			Dest:    &svcStopFlag,
+		},
+		&zbxflag.StringFlag{
+			Flag: zbxflag.Flag{
+				Name:      "startup-type",
+				Shorthand: "S",
+				Description: fmt.Sprintf(
+					"Set startup type of the Zabbix Windows agent service to be installed."+
+						" Allowed values: %s (default), %s, %s, %s",
+					startTypeAutomatic,
+					startTypeDelayed,
+					startTypeManual,
+					startTypeDisabled,
+				),
+			},
+			Default: "",
+			Dest:    &svcStartType,
+		},
+	}
 }
 
 func isWinLauncher() bool {
-	if svcInstallFlag || svcUninstallFlag || svcStartFlag || svcStopFlag || svcMultipleAgentFlag {
+	if svcInstallFlag || svcUninstallFlag || svcStartFlag || svcStopFlag || svcMultipleAgentFlag ||
+		svcStartType != "" {
 		return true
 	}
 	return false
@@ -153,23 +191,47 @@ func eventLogErr(err error) error {
 	return nil
 }
 
-func validateExclusiveFlags() error {
-	defaultFlagSet := argTest || argPrint || argVerbose
-	serviceFlagsSet := []bool{svcInstallFlag, svcUninstallFlag, svcStartFlag, svcStopFlag}
-	var count int
-	for _, serserviceFlagSet := range serviceFlagsSet {
-		if serserviceFlagSet {
+func validateMultipleAgentFlag() bool {
+	if svcMultipleAgentFlag &&
+		!(svcInstallFlag || svcUninstallFlag || svcStartFlag || svcStopFlag || svcStartType != "") &&
+		!winServiceRun {
+		return false
+	}
+
+	return true
+}
+
+func validateExclusiveFlags(args *Arguments) error {
+	var (
+		exclusiveFlagsSet = []bool{
+			svcInstallFlag,
+			svcUninstallFlag,
+			svcStartFlag,
+			svcStopFlag,
+			args.print,
+			args.test != "",
+			args.runtimeCommand != "",
+			args.testConfig,
+		}
+		count int
+	)
+
+	if args.verbose && !(args.test != "" || args.print) {
+		return errors.New("option -v, --verbose can only be specified with -t or -p")
+	}
+
+	for _, exclusiveFlagSet := range exclusiveFlagsSet {
+		if exclusiveFlagSet {
 			count++
 		}
-		if count >= 2 || (serserviceFlagSet && defaultFlagSet) {
-			return errors.New("mutually exclusive options used, use help '-help'('-h'), for additional information")
+		if count >= 2 { //nolint:gomnd
+			return errors.New("mutually exclusive options used, see -h, --help for more information")
 		}
 	}
 
-	if svcMultipleAgentFlag && count == 0 && !winServiceRun {
+	if !validateMultipleAgentFlag() {
 		return errors.New(
-			"multiple agents '-multiple-agents'('-m'), flag has to be used with another windows service flag, use help '-help'('-h'), for additional information",
-		)
+			"option -m, --multiple-agents can only be used with one of the service options, see -h, --help for more information")
 	}
 
 	return nil
@@ -217,7 +279,7 @@ func handleWindowsService(confPath string) error {
 		serviceName = fmt.Sprintf("%s [%s]", serviceName, agent.FirstHostname)
 	}
 
-	if svcInstallFlag || svcUninstallFlag || svcStartFlag || svcStopFlag {
+	if svcInstallFlag || svcUninstallFlag || svcStartFlag || svcStopFlag || svcStartType != "" {
 		absPath, err := filepath.Abs(confPath)
 		if err != nil {
 			return err
@@ -261,6 +323,11 @@ func resolveWindowsService(confPath string) error {
 			return fmt.Errorf("failed to stop %s service: %s", serviceName, err)
 		}
 		msg = fmt.Sprintf("'%s' stopped successfully", serviceName)
+	case svcStartType != "":
+		if err := svcStartupTypeSet(); err != nil {
+			return errs.Wrapf(err, "failed to set service '%s' startup type", serviceName)
+		}
+		msg = fmt.Sprintf("service '%s' startup type configured successfully", serviceName)
 	}
 
 	msg = fmt.Sprintf("zabbix_agent2 [%d]: %s\n", os.Getpid(), msg)
@@ -294,10 +361,34 @@ func getAgentPath() (p string, err error) {
 	return
 }
 
+func svcStartTypeFlagParse() (uint32, bool, error) {
+	var startType uint32
+	var delayedAutoStart bool
+	var err error
+
+	switch svcStartType {
+	case "":
+		startType = mgr.StartAutomatic
+	case startTypeAutomatic:
+		startType = mgr.StartAutomatic
+	case startTypeDelayed:
+		delayedAutoStart = true
+		startType = mgr.StartAutomatic
+	case startTypeManual:
+		startType = mgr.StartManual
+	case startTypeDisabled:
+		startType = mgr.StartDisabled
+	default:
+		err = fmt.Errorf("unknown service start type: '%s'", svcStartType)
+	}
+
+	return startType, delayedAutoStart, err
+}
+
 func svcInstall(conf string) error {
 	exepath, err := getAgentPath()
 	if err != nil {
-		return fmt.Errorf("failed to get Zabbix Agent 2 exeutable path: %s", err.Error())
+		return fmt.Errorf("failed to get Zabbix Agent 2 executable path: %s", err.Error())
 	}
 
 	m, err := mgr.Connect()
@@ -312,10 +403,25 @@ func svcInstall(conf string) error {
 		return errors.New("service already exists")
 	}
 
-	s, err = m.CreateService(serviceName, exepath, mgr.Config{
-		StartType: mgr.StartAutomatic, DisplayName: serviceName,
-		Description: "Provides system monitoring", BinaryPathName: fmt.Sprintf("%s -c %s -f=false", exepath, conf),
-	}, "-c", conf, "-f=false")
+	startType, delayedAutoStart, err := svcStartTypeFlagParse()
+
+	if err != nil {
+		return errs.Wrap(err, "failed to get new startup type")
+	}
+
+	s, err = m.CreateService(
+		serviceName,
+		exepath,
+		mgr.Config{
+			StartType:        startType,
+			DisplayName:      serviceName,
+			Description:      "Provides system monitoring",
+			BinaryPathName:   fmt.Sprintf("%s -c %s -f=false", exepath, conf),
+			DelayedAutoStart: delayedAutoStart,
+		},
+		"-c", conf,
+		"-f=false",
+	)
 	if err != nil {
 		return fmt.Errorf("failed to create service: %s", err.Error())
 	}
@@ -409,6 +515,45 @@ func svcStop() error {
 	return nil
 }
 
+func svcStartupTypeSet() error {
+
+	m, err := mgr.Connect()
+
+	if err != nil {
+		return errs.Wrap(err, "failed to connect to service manager")
+	}
+
+	defer m.Disconnect()
+
+	s, err := m.OpenService(serviceName)
+
+	if err != nil {
+		return errs.Wrap(err, "failed to open service")
+	}
+
+	defer s.Close()
+
+	c, err := s.Config()
+
+	if err != nil {
+		return errs.Wrap(err, "failed to retrieve service config")
+	}
+
+	c.StartType, c.DelayedAutoStart, err = svcStartTypeFlagParse()
+
+	if err != nil {
+		return errs.Wrap(err, "failed to get new startup type")
+	}
+
+	err = s.UpdateConfig(c)
+
+	if err != nil {
+		return errs.Wrap(err, "failed to update service config")
+	}
+
+	return nil
+}
+
 func confirmService() {
 	if winServiceRun {
 		startChan <- true
@@ -439,9 +584,7 @@ func runService() {
 type winService struct{}
 
 func (ws *winService) Execute(
-	args []string,
-	r <-chan svc.ChangeRequest,
-	changes chan<- svc.Status,
+	args []string, r <-chan svc.ChangeRequest, changes chan<- svc.Status,
 ) (ssec bool, errno uint32) {
 	changes <- svc.Status{State: svc.StartPending}
 	select {
