@@ -1,20 +1,15 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 package scheduler
@@ -25,19 +20,23 @@ import (
 	"fmt"
 	"math"
 	"sort"
+	"strconv"
+	"strings"
 	"time"
 
-	"git.zabbix.com/ap/plugin-support/conf"
-	"git.zabbix.com/ap/plugin-support/log"
-	"git.zabbix.com/ap/plugin-support/plugin"
-	"git.zabbix.com/ap/plugin-support/plugin/comms"
-	"zabbix.com/internal/agent"
-	"zabbix.com/internal/agent/alias"
-	"zabbix.com/internal/agent/keyaccess"
-	"zabbix.com/internal/monitor"
-	"zabbix.com/pkg/glexpr"
-	"zabbix.com/pkg/itemutil"
-	"zabbix.com/plugins/external"
+	"golang.zabbix.com/agent2/internal/agent"
+	"golang.zabbix.com/agent2/internal/agent/alias"
+	"golang.zabbix.com/agent2/internal/agent/keyaccess"
+	"golang.zabbix.com/agent2/internal/agent/resultcache"
+	"golang.zabbix.com/agent2/internal/monitor"
+	"golang.zabbix.com/agent2/pkg/glexpr"
+	"golang.zabbix.com/agent2/pkg/itemutil"
+	"golang.zabbix.com/agent2/plugins/external"
+	"golang.zabbix.com/sdk/conf"
+	"golang.zabbix.com/sdk/errs"
+	"golang.zabbix.com/sdk/log"
+	"golang.zabbix.com/sdk/plugin"
+	"golang.zabbix.com/sdk/plugin/comms"
 )
 
 const (
@@ -47,6 +46,15 @@ const (
 	shutdownInactive = -1
 )
 
+type Request struct {
+	Itemid      uint64  `json:"itemid"`
+	Key         string  `json:"key"`
+	Delay       string  `json:"delay"`
+	LastLogsize *uint64 `json:"lastlogsize"`
+	Mtime       *int    `json:"mtime"`
+	Timeout     any     `json:"timeout"`
+}
+
 // Manager implements Scheduler interface and manages plugin interface usage.
 type Manager struct {
 	input       chan interface{}
@@ -63,10 +71,19 @@ type Manager struct {
 // updateRequest contains list of metrics monitored by a client and additional client configuration data.
 type updateRequest struct {
 	clientID                   uint64
-	sink                       plugin.ResultWriter
+	sink                       resultcache.Writer
 	firstActiveChecksRefreshed bool
-	requests                   []*plugin.Request
+	requests                   []*Request
 	expressions                []*glexpr.Expression
+	now                        time.Time
+}
+
+// commandRequest contains list of remote commands
+type commandRequest struct {
+	clientID uint64
+	sink     resultcache.Writer
+	commands []*agent.RemoteCommand
+	now      time.Time
 }
 
 // queryRequest contains status/debug query request.
@@ -81,18 +98,34 @@ type queryRequestUserParams struct {
 }
 
 type Scheduler interface {
-	UpdateTasks(clientID uint64, writer plugin.ResultWriter, firstActiveChecksRefreshed bool,
-		expressions []*glexpr.Expression, requests []*plugin.Request)
+	UpdateTasks(
+		clientID uint64,
+		writer resultcache.Writer, firstActiveChecksRefreshed bool,
+		expressions []*glexpr.Expression,
+		requests []*Request,
+		now time.Time,
+	)
+	UpdateCommands(
+		clientID uint64,
+		writer resultcache.Writer,
+		commands []*agent.RemoteCommand,
+		now time.Time,
+	)
 	FinishTask(task performer)
 	PerformTask(
 		key string,
 		timeout time.Duration,
 		clientID uint64,
-	) (result string, err error)
+	) (result *string, err error)
 	Query(command string) (status string)
 	QueryUserParams() (status string)
 }
 
+type systemOptions struct {
+	ForceActiveChecksOnStart *int `conf:"optional"`
+	Capacity                 int  `conf:"optional"`
+}
+
 // cleanupClient performs deactivation of plugins the client is not using anymore.
 // It's called after client update and once per hour for the client associated to
 // single passive checks.
@@ -153,28 +186,72 @@ func (m *Manager) cleanupClient(c *client, now time.Time) {
 	}
 }
 
-// processUpdateRequest processes client update request. It's being used for multiple requests
-// (active checks on a server) and also for direct requets (single passive and internal checks).
-func (m *Manager) processUpdateRequest(update *updateRequest, now time.Time) {
-	log.Debugf("[%d] processing update request (%d requests)", update.clientID, len(update.requests))
+func parseItemTimeout(s string) (seconds int, e error) {
+	const invalidTimeoutError = "Unsupported timeout value."
+	const maxTimeout = 600
+
+	if s == "" {
+		e = errors.New(invalidTimeoutError)
 
-	// immediately fail direct checks and ignore bulk requests when shutting down
-	if m.shutdownSeconds != shutdownInactive {
-		if update.clientID <= agent.MaxBuiltinClientID {
-			if len(update.requests) == 1 {
-				update.sink.Write(&plugin.Result{
-					Itemid: update.requests[0].Itemid,
-					Error:  errors.New("Cannot obtain item value during shutdown process."),
-					Ts:     now,
-				})
-			} else {
-				log.Warningf("[%d] direct checks can contain only single request while received %d requests",
-					update.clientID, len(update.requests))
-			}
-		}
 		return
 	}
 
+	if intVal, err := strconv.Atoi(s); err != nil {
+		var mult int
+
+		if strings.HasSuffix(s, "m") {
+			mult = 60
+		} else if strings.HasSuffix(s, "s") {
+			mult = 1
+		} else {
+			e = errors.New(invalidTimeoutError)
+
+			return
+		}
+
+		if val, err := strconv.Atoi(s[:len(s)-1]); err != nil {
+			e = errors.New(invalidTimeoutError)
+
+			return
+		} else {
+			seconds = val * mult
+		}
+	} else {
+		seconds = intVal
+	}
+
+	if seconds > maxTimeout {
+		e = errors.New(invalidTimeoutError)
+	}
+
+	return
+}
+
+func ParseItemTimeoutAny(timeoutIn any) (timeout int, err error) {
+	switch v := timeoutIn.(type) {
+	case nil:
+		timeout = agent.Options.Timeout
+	case float64:
+		timeout = int(v)
+	case int:
+		timeout = v
+	case string:
+		timeout, err = parseItemTimeout(v)
+	default:
+		err = fmt.Errorf("unexpected timeout %q of type %T", timeoutIn, timeoutIn)
+	}
+	if err == nil {
+		if timeout > 600 || timeout < 1 {
+			err = fmt.Errorf("Unsupported timeout value.")
+		}
+	}
+
+	return timeout, err
+}
+
+// processUpdateRequest processes client update request. It's being used for multiple requests
+// (active checks on a server) and also for direct requets (single passive and internal checks).
+func (m *Manager) processUpdateRequestRun(update *updateRequest) {
 	var c *client
 	var ok bool
 	if c, ok = m.clients[update.clientID]; !ok {
@@ -208,7 +285,12 @@ func (m *Manager) processUpdateRequest(update *updateRequest, now time.Time) {
 			if !ok {
 				err = fmt.Errorf("Unknown metric %s", key)
 			} else {
-				err = c.addRequest(p, r, update.sink, now, update.firstActiveChecksRefreshed)
+				var timeout int
+				timeout, err = ParseItemTimeoutAny(r.Timeout)
+
+				if err == nil {
+					err = c.addRequest(p, r, timeout, update.sink, update.now, update.firstActiveChecksRefreshed)
+				}
 			}
 		}
 
@@ -223,9 +305,15 @@ func (m *Manager) processUpdateRequest(update *updateRequest, now time.Time) {
 					tacc.task().deactivate()
 				}
 			}
-
-			update.sink.Write(&plugin.Result{Itemid: r.Itemid, Error: err, Ts: now})
-			log.Debugf("[%d] cannot monitor metric \"%s\": %s", update.clientID, r.Key, err.Error())
+			update.sink.Write(
+				&plugin.Result{Itemid: r.Itemid, Error: err, Ts: update.now},
+			)
+			log.Debugf(
+				"[%d] cannot monitor metric \"%s\": %s",
+				update.clientID,
+				r.Key,
+				err.Error(),
+			)
 
 			continue
 		}
@@ -237,7 +325,96 @@ func (m *Manager) processUpdateRequest(update *updateRequest, now time.Time) {
 		}
 	}
 
-	m.cleanupClient(c, now)
+	m.cleanupClient(c, update.now)
+}
+
+func (m *Manager) processUpdateRequestShutdown(update *updateRequest) {
+	if update.clientID <= agent.MaxBuiltinClientID {
+		if len(update.requests) == 1 {
+			update.sink.Write(&plugin.Result{
+				Itemid: update.requests[0].Itemid,
+				Error: errors.New(
+					"Cannot obtain item value during shutdown process.",
+				),
+				Ts: update.now,
+			})
+		} else {
+			log.Warningf("[%d] direct checks can contain only single request while received %d requests",
+				update.clientID, len(update.requests))
+		}
+	}
+}
+
+func (m *Manager) processUpdateRequest(update *updateRequest) {
+	log.Debugf(
+		"[%d] processing update request (%d requests)",
+		update.clientID,
+		len(update.requests),
+	)
+
+	// immediately fail direct checks and ignore bulk requests when shutting down
+	if m.shutdownSeconds != shutdownInactive {
+		m.processUpdateRequestShutdown(update)
+	} else {
+		m.processUpdateRequestRun(update)
+	}
+}
+
+func (m *Manager) processCommandRequest(update *commandRequest) {
+	log.Debugf(
+		"[%d] processing command request (%d commands)",
+		update.clientID,
+		len(update.commands),
+	)
+
+	var c *client
+	var ok bool
+	if c, ok = m.clients[update.clientID]; !ok {
+		log.Debugf("[%d] registering new client", update.clientID)
+		c = newClient(update.clientID, update.sink)
+		m.clients[update.clientID] = c
+	}
+
+	for _, rc := range update.commands {
+		var wait string
+		if rc.Wait == 1 {
+			wait = "wait"
+		} else {
+			wait = "nowait"
+		}
+		params := []string{rc.Command, wait}
+
+		if !keyaccess.CheckRules("system.run", params) {
+			log.Debugf("Remote command '%s' is not allowed", rc.Command)
+
+			update.sink.WriteCommand(
+				&resultcache.CommandResult{
+					ID:    rc.Id,
+					Error: errors.New("Unsupported item key."),
+				},
+			)
+			update.sink.Flush()
+
+			continue
+		}
+
+		if p, ok := m.plugins["system.run"]; ok {
+			c.addCommand(p, rc.Id, params, update.sink, update.now)
+
+			if !p.queued() {
+				heap.Push(&m.pluginQueue, p)
+			} else {
+				m.pluginQueue.Update(p)
+			}
+		} else {
+			log.Warningf("Remote commands cannot be executed, plugin \"system.run\" is unavailable")
+
+			update.sink.WriteCommand(&resultcache.CommandResult{ID: rc.Id, Error: errors.New("Unsupported item key.")})
+			update.sink.Flush()
+
+			break
+		}
+	}
 }
 
 // processQueue processes queued plugins/tasks
@@ -405,6 +582,8 @@ run:
 					for _, client := range m.clients {
 						if len(client.pluginsInfo) == 0 {
 							delete(m.clients, client.ID())
+						} else {
+							client.commandCleanup()
 						}
 					}
 					cleaned = now
@@ -420,7 +599,10 @@ run:
 			}
 			switch v := u.(type) {
 			case *updateRequest:
-				m.processUpdateRequest(v, time.Now())
+				m.processUpdateRequest(v)
+				m.processQueue(time.Now())
+			case *commandRequest:
+				m.processCommandRequest(v)
 				m.processQueue(time.Now())
 			case performer:
 				m.processFinishRequest(v)
@@ -490,14 +672,6 @@ run:
 	monitor.Unregister(monitor.Scheduler)
 }
 
-type pluginOptions struct {
-	Capacity int `conf:"optional"`
-	System   struct {
-		ForceActiveChecksOnStart *int `conf:"optional"`
-		Capacity                 int  `conf:"optional"`
-	} `conf:"optional"`
-}
-
 func (m *Manager) init() {
 	m.input = make(chan interface{}, 10)
 	m.pluginQueue = make(pluginHeap, 0, len(plugin.Metrics))
@@ -593,8 +767,13 @@ func (m *Manager) Stop() {
 	m.input <- nil
 }
 
-func (m *Manager) UpdateTasks(clientID uint64, writer plugin.ResultWriter, firstActiveChecksRefreshed bool,
-	expressions []*glexpr.Expression, requests []*plugin.Request,
+func (m *Manager) UpdateTasks(
+	clientID uint64,
+	writer resultcache.Writer,
+	firstActiveChecksRefreshed bool,
+	expressions []*glexpr.Expression,
+	requests []*Request,
+	now time.Time,
 ) {
 	m.input <- &updateRequest{
 		clientID:                   clientID,
@@ -602,6 +781,21 @@ func (m *Manager) UpdateTasks(clientID uint64, writer plugin.ResultWriter, first
 		requests:                   requests,
 		expressions:                expressions,
 		firstActiveChecksRefreshed: firstActiveChecksRefreshed,
+		now:                        now,
+	}
+}
+
+func (m *Manager) UpdateCommands(
+	clientID uint64,
+	writer resultcache.Writer,
+	commands []*agent.RemoteCommand,
+	now time.Time,
+) {
+	m.input <- &commandRequest{
+		clientID: clientID,
+		sink:     writer,
+		commands: commands,
+		now:      now,
 	}
 }
 
@@ -611,6 +805,10 @@ func (r resultWriter) Write(result *plugin.Result) {
 	r <- result
 }
 
+func (r resultWriter) WriteCommand(cr *resultcache.CommandResult) {
+	log.Errf("remote commands are not supported by single task requests")
+}
+
 func (r resultWriter) Flush() {
 }
 
@@ -626,30 +824,30 @@ func (m *Manager) PerformTask(
 	key string,
 	timeout time.Duration,
 	clientID uint64,
-) (result string, err error) {
+) (*string, error) {
 	var lastLogsize uint64
 	var mtime int
 
 	w := make(resultWriter, 1)
 
-	m.UpdateTasks(clientID, w, false, nil, []*plugin.Request{{Key: key, LastLogsize: &lastLogsize, Mtime: &mtime}})
+	m.UpdateTasks(
+		clientID,
+		w,
+		false,
+		nil,
+		[]*Request{
+			{
+				Key:         key,
+				LastLogsize: &lastLogsize,
+				Mtime:       &mtime,
+				Timeout:     int(timeout.Seconds()),
+			},
+		},
+		time.Now(),
+	)
 
-	select {
-	case r := <-w:
-		if r.Error == nil {
-			if r.Value != nil {
-				result = *r.Value
-			} else {
-				// single metric requests do not support empty values, return error instead
-				err = errors.New("No values have been gathered yet.")
-			}
-		} else {
-			err = r.Error
-		}
-	case <-time.After(timeout):
-		err = fmt.Errorf("Timeout occurred while gathering data.")
-	}
-	return
+	r := <-w
+	return r.Value, r.Error
 }
 
 func (m *Manager) FinishTask(task performer) {
@@ -688,13 +886,96 @@ func (m *Manager) configure(options *agent.AgentOptions) (err error) {
 	return
 }
 
-func NewManager(options *agent.AgentOptions) (mannager *Manager, err error) {
-	var m Manager
-	m.init()
-	if err = m.validatePlugins(options); err != nil {
-		return
+// NewManager crates a new manager instance.
+func NewManager(options *agent.AgentOptions) (*Manager, error) {
+	m := &Manager{
+		input:           make(chan any, 10),
+		pluginQueue:     make(pluginHeap, 0, len(plugin.Metrics)),
+		clients:         make(map[uint64]*client),
+		plugins:         make(map[string]*pluginAgent),
+		shutdownSeconds: shutdownInactive,
 	}
-	return &m, m.configure(options)
+
+	metrics := make([]*plugin.Metric, 0, len(plugin.Metrics))
+
+	for _, metric := range plugin.Metrics {
+		metrics = append(metrics, metric)
+	}
+
+	sort.Slice(
+		metrics,
+		func(i, j int) bool {
+			return metrics[i].Plugin.Name() < metrics[j].Plugin.Name()
+		},
+	)
+
+	pagent := &pluginAgent{}
+	for _, metric := range metrics {
+		if metric.Plugin != pagent.impl { //nolint:nestif
+			capacity, forceActiveChecksOnStart := getPluginOptions(
+				agent.Options.Plugins[metric.Plugin.Name()],
+				metric.Plugin.Name(),
+			)
+			if capacity > metric.Plugin.Capacity() {
+				log.Warningf(
+					"lowering the plugin %s capacity to %d as the configured capacity %d exceeds limits",
+					metric.Plugin.Name(),
+					metric.Plugin.Capacity(),
+					capacity,
+				)
+
+				capacity = metric.Plugin.Capacity()
+			}
+
+			pagent = &pluginAgent{
+				impl:                     metric.Plugin,
+				tasks:                    make(performerHeap, 0),
+				maxCapacity:              capacity,
+				usedCapacity:             0,
+				forceActiveChecksOnStart: forceActiveChecksOnStart,
+				index:                    -1,
+				refcount:                 0,
+				usrprm:                   metric.UsrPrm,
+			}
+
+			if metric.Plugin.IsExternal() {
+				ext, ok := metric.Plugin.(*external.Plugin)
+				if !ok {
+					return nil, errs.Errorf(
+						"unknown external plugin implementation for plugin - %q",
+						metric.Plugin.Name(),
+					)
+				}
+
+				log.Infof(
+					"using plugin '%s' (%s) providing following interfaces: %s",
+					metric.Plugin.Name(),
+					ext.Path,
+					getPluginInterfaceNames(metric.Plugin),
+				)
+			} else {
+				log.Infof(
+					"using plugin '%s' (built-in) providing following interfaces: %s",
+					metric.Plugin.Name(),
+					getPluginInterfaceNames(metric.Plugin),
+				)
+			}
+		}
+
+		m.plugins[metric.Key] = pagent
+	}
+
+	err := m.validatePlugins(options)
+	if err != nil {
+		return nil, errs.Wrap(err, "failed to validate plugins")
+	}
+
+	err = m.configure(options)
+	if err != nil {
+		return nil, errs.Wrap(err, "failed to configure manager")
+	}
+
+	return m, nil
 }
 
 func (m *Manager) addUserParamsPlugin(key string) {
@@ -729,74 +1010,67 @@ func peekTask(tasks performerHeap) performer {
 	return tasks[0]
 }
 
-func getPluginOptions(
-	optsRaw interface{},
-	name string,
-) (capacity, forceActiveChecksOnStart int) {
-	pluginCap, pluginSystemCap, pluginForceActiveChecksOnStart := getPluginOpts(
-		optsRaw,
-		name,
-	)
-
-	if pluginSystemCap > 0 {
-		if pluginCap > 0 {
-			log.Warningf(
-				"both Plugins.%s.Capacity and Plugins.%s.System.Capacity "+
-					"configuration parameters are set, using System.Capacity: %d",
-				name,
-				name,
-				pluginSystemCap,
-			)
-		}
-		capacity = pluginSystemCap
-	} else if pluginCap > 0 {
-		log.Warningf(
-			"plugin %s configuration parameter Plugins.%s.Capacity is deprecated, use Plugins.%s.System.Capacity instead",
-			name, name, name,
-		)
-		capacity = pluginCap
-	} else {
-		capacity = plugin.DefaultCapacity
+func getPluginOptions(optsRaw any, pluginName string) (int, int) {
+	var opt struct {
+		System systemOptions `conf:"optional"`
 	}
 
-	if nil != pluginForceActiveChecksOnStart {
-		if *pluginForceActiveChecksOnStart > 1 ||
-			*pluginForceActiveChecksOnStart < 0 {
-			log.Warningf(
-				"invalid Plugins.%s.System.ForceActiveChecksOnStart configuration parameter: %d",
-				name,
-				*pluginForceActiveChecksOnStart,
-			)
-			forceActiveChecksOnStart = agent.Options.ForceActiveChecksOnStart
-		} else {
-			forceActiveChecksOnStart = *pluginForceActiveChecksOnStart
-		}
-	} else {
-		forceActiveChecksOnStart = agent.Options.ForceActiveChecksOnStart
-	}
-
-	return
-}
-
-func getPluginOpts(
-	optsRaw interface{},
-	name string,
-) (pluginCap, pluginSystemCap int, forceActiveChecksOnStart *int) {
-	var opt pluginOptions
+	capacity := plugin.DefaultCapacity
 
 	if optsRaw == nil {
-		return
+		return capacity, agent.Options.ForceActiveChecksOnStart
 	}
 
-	if err := conf.Unmarshal(optsRaw, &opt, false); err != nil {
-		log.Warningf("invalid plugin %s configuration: %s", name, err)
+	err := conf.Unmarshal(optsRaw, &opt, false)
+	if err != nil {
+		log.Warningf("invalid plugin %s configuration: %s", pluginName, err)
 
-		return
+		return capacity, agent.Options.ForceActiveChecksOnStart
 	}
 
-	pluginCap = opt.Capacity
-	pluginSystemCap = opt.System.Capacity
-	forceActiveChecksOnStart = opt.System.ForceActiveChecksOnStart
+	if opt.System.Capacity > 0 {
+		capacity = opt.System.Capacity
+	}
 
-	return
+	if opt.System.ForceActiveChecksOnStart == nil {
+		return capacity, agent.Options.ForceActiveChecksOnStart
+	}
+
+	if *opt.System.ForceActiveChecksOnStart > 1 || *opt.System.ForceActiveChecksOnStart < 0 {
+		log.Warningf(
+			"invalid Plugins.%s.System.ForceActiveChecksOnStart configuration parameter: %d",
+			pluginName,
+			*opt.System.ForceActiveChecksOnStart,
+		)
+
+		return capacity, agent.Options.ForceActiveChecksOnStart
+	}
+
+	return capacity, *opt.System.ForceActiveChecksOnStart
+}
+
+func getPluginInterfaceNames(p plugin.Accessor) string {
+	interfaceNames := make([]string, 0, 5)
+
+	if _, ok := p.(plugin.Exporter); ok {
+		interfaceNames = append(interfaceNames, "exporter")
+	}
+
+	if _, ok := p.(plugin.Collector); ok {
+		interfaceNames = append(interfaceNames, "collector")
+	}
+
+	if _, ok := p.(plugin.Runner); ok {
+		interfaceNames = append(interfaceNames, "runner")
+	}
+
+	if _, ok := p.(plugin.Watcher); ok {
+		interfaceNames = append(interfaceNames, "watcher")
+	}
+
+	if _, ok := p.(plugin.Configurator); ok {
+		interfaceNames = append(interfaceNames, "configurator")
+	}
+
+	return strings.Join(interfaceNames, ", ")
 }
