@@ -1,20 +1,15 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 package scheduler
@@ -25,16 +20,18 @@ import (
 	"reflect"
 	"time"
 
-	"git.zabbix.com/ap/plugin-support/log"
-	"git.zabbix.com/ap/plugin-support/plugin"
-	"zabbix.com/internal/agent"
-	"zabbix.com/pkg/itemutil"
-	"zabbix.com/pkg/zbxlib"
+	"golang.zabbix.com/agent2/internal/agent"
+	"golang.zabbix.com/agent2/internal/agent/resultcache"
+	"golang.zabbix.com/agent2/pkg/itemutil"
+	"golang.zabbix.com/agent2/pkg/zbxlib"
+	"golang.zabbix.com/sdk/log"
+	"golang.zabbix.com/sdk/plugin"
 )
 
 // task priority within the same second is done by setting nanosecond component
 const (
 	priorityConfiguratorTaskNs = iota
+	priorityCommandTaskNs      = iota
 	priorityStarterTaskNs
 	priorityCollectorTaskNs
 	priorityWatcherTaskNs
@@ -109,9 +106,24 @@ func (t *collectorTask) perform(s Scheduler) {
 	log.Debugf("plugin %s: executing collector task", t.plugin.name())
 	go func() {
 		collector, _ := t.plugin.impl.(plugin.Collector)
-		if err := collector.Collect(); err != nil {
+		start := time.Now()
+		err := collector.Collect()
+
+		if err != nil {
 			log.Warningf("plugin '%s' collector failed: %s", t.plugin.impl.Name(), err.Error())
 		}
+
+		elapsedSeconds := time.Since(start).Seconds()
+
+		if elapsedSeconds > float64(collector.Period()) {
+			log.Warningf(
+				"plugin '%s': time spent in collector task %f s exceeds collecting interval %d s",
+				t.plugin.impl.Name(),
+				elapsedSeconds,
+				collector.Period(),
+			)
+		}
+
 		s.FinishTask(t)
 	}()
 }
@@ -151,11 +163,35 @@ type exporterTask struct {
 	output  plugin.ResultWriter
 }
 
+func invokeExport(a plugin.Accessor, key string, params []string, ctx plugin.ContextProvider) (any, error) {
+	exporter, _ := a.(plugin.Exporter)
+
+	if a.HandleTimeout() {
+		return exporter.Export(key, params, ctx)
+	}
+
+	var ret any
+	var err error
+	tc := make(chan bool)
+
+	go func() {
+		ret, err = exporter.Export(key, params, ctx)
+		tc <- true
+	}()
+
+	select {
+	case <-tc:
+	case <-time.After(time.Second * time.Duration(ctx.Timeout())):
+		err = fmt.Errorf("Timeout occurred while gathering data.")
+	}
+
+	return ret, err
+}
+
 func (t *exporterTask) perform(s Scheduler) {
 	// pass item key as parameter so it can be safely updated while task is being processed in its goroutine
 	go func(itemkey string) {
 		var result *plugin.Result
-		exporter, _ := t.plugin.impl.(plugin.Exporter)
 		now := time.Now()
 		var key string
 		var params []string
@@ -163,9 +199,10 @@ func (t *exporterTask) perform(s Scheduler) {
 
 		if key, params, err = itemutil.ParseKey(itemkey); err == nil {
 			var ret interface{}
-			log.Debugf("executing exporter task for itemid:%d key '%s'", t.item.itemid, itemkey)
 
-			if ret, err = exporter.Export(key, params, t); err == nil {
+			ret, err = invokeExport(t.plugin.impl, key, params, t)
+
+			if err == nil {
 				log.Debugf("executed exporter task for itemid:%d key '%s'", t.item.itemid, itemkey)
 				if ret != nil {
 					rt := reflect.TypeOf(ret)
@@ -244,6 +281,14 @@ func (t *exporterTask) GlobalRegexp() plugin.RegexpMatcher {
 	return t.client.GlobalRegexp()
 }
 
+func (t *exporterTask) Timeout() int {
+	return t.item.timeout
+}
+
+func (t *exporterTask) Delay() string {
+	return t.item.delay
+}
+
 // directExporterTask provides access to plugin Exporter interaface.
 // It's used for non-recurring exporter requests - single passive checks
 // and internal requests to obtain HostnameItem, HostMetadataItem,
@@ -261,40 +306,52 @@ type directExporterTask struct {
 func (t *directExporterTask) isRecurring() bool {
 	return !t.done
 }
+
+func (t *directExporterTask) invokeExport(key string, params []string) (any, error) {
+	ret, err := invokeExport(t.plugin.impl, key, params, t)
+
+	if err != nil {
+		log.Debugf("failed to execute direct exporter task for key '%s[%s]' error: '%s'", key, params, err.Error())
+
+		return nil, err
+	}
+
+	log.Debugf("executed direct exporter task for key '%s[%s]'", key, params)
+	if ret != nil {
+		rt := reflect.TypeOf(ret)
+		switch rt.Kind() {
+		case reflect.Slice, reflect.Array:
+			return nil, errors.New("Multiple return values are not supported for single passive checks")
+		default:
+			return ret, nil
+		}
+	}
+
+	return ret, nil
+}
+
 func (t *directExporterTask) perform(s Scheduler) {
 	// pass item key as parameter so it can be safely updated while task is being processed in its goroutine
 	go func(itemkey string) {
 		var result *plugin.Result
-		exporter, _ := t.plugin.impl.(plugin.Exporter)
 		now := time.Now()
 		var key string
 		var params []string
 		var err error
 
 		if now.After(t.expire) {
-			err = errors.New("No data available.")
+			err = errors.New("Timeout while waiting for item in queue.")
 			log.Debugf("direct exporter task expired for key '%s' error: '%s'", itemkey, err.Error())
 		} else {
 			if key, params, err = itemutil.ParseKey(itemkey); err == nil {
 				var ret interface{}
 				log.Debugf("executing direct exporter task for key '%s'", itemkey)
 
-				if ret, err = exporter.Export(key, params, t); err == nil {
-					log.Debugf("executed direct exporter task for key '%s'", itemkey)
-					if ret != nil {
-						rt := reflect.TypeOf(ret)
-						switch rt.Kind() {
-						case reflect.Slice, reflect.Array:
-							err = errors.New("Multiple return values are not supported for single passive checks")
-						default:
-							result = itemutil.ValueToResult(t.item.itemid, now, ret)
-							t.output.Write(result)
-							t.done = true
-						}
-					}
-				} else {
-					log.Debugf("failed to execute direct exporter task for key '%s' error: '%s'",
-						itemkey, err.Error())
+				ret, err = t.invokeExport(key, params)
+				if err == nil {
+					result = itemutil.ValueToResult(t.item.itemid, now, ret)
+					t.output.Write(result)
+					t.done = true
 				}
 			}
 		}
@@ -343,6 +400,14 @@ func (t *directExporterTask) GlobalRegexp() plugin.RegexpMatcher {
 	return t.client.GlobalRegexp()
 }
 
+func (t *directExporterTask) Timeout() int {
+	return t.item.timeout
+}
+
+func (t *directExporterTask) Delay() string {
+	return t.item.delay
+}
+
 // starterTask provides access to plugin Exporter interaface Start() method.
 type starterTask struct {
 	taskBase
@@ -400,15 +465,15 @@ func (t *stopperTask) isItemKeyEqual(itemkey string) bool {
 // stopperTask provides access to plugin Watcher interaface.
 type watcherTask struct {
 	taskBase
-	requests []*plugin.Request
-	client   ClientAccessor
+	items  []*plugin.Item
+	client ClientAccessor
 }
 
 func (t *watcherTask) perform(s Scheduler) {
 	log.Debugf("plugin %s: executing watcher task", t.plugin.name())
 	go func() {
 		watcher, _ := t.plugin.impl.(plugin.Watcher)
-		watcher.Watch(t.requests, t)
+		watcher.Watch(t.items, t)
 		s.FinishTask(t)
 	}()
 }
@@ -448,6 +513,14 @@ func (t *watcherTask) GlobalRegexp() plugin.RegexpMatcher {
 	return t.client.GlobalRegexp()
 }
 
+func (t *watcherTask) Timeout() int {
+	return 0
+}
+
+func (t *watcherTask) Delay() string {
+	return ""
+}
+
 // configuratorTask provides access to plugin Configurator interaface.
 type configuratorTask struct {
 	taskBase
@@ -475,3 +548,51 @@ func (t *configuratorTask) getWeight() int {
 func (t *configuratorTask) isItemKeyEqual(itemkey string) bool {
 	return false
 }
+
+// commandTask executes remote commands received with active requestes
+type commandTask struct {
+	taskBase
+	id     uint64
+	params []string
+	output resultcache.Writer
+}
+
+func (t *commandTask) isRecurring() bool {
+	return false
+}
+func (t *commandTask) perform(s Scheduler) {
+	// execute remote command
+	go func() {
+		e := t.plugin.impl.(plugin.Exporter)
+
+		var cr *resultcache.CommandResult
+
+		if ret, err := e.Export("system.run", t.params, nil); err == nil {
+			if ret != nil {
+				cr = &resultcache.CommandResult{
+					ID:     t.id,
+					Result: *itemutil.ValueToString(ret),
+				}
+			}
+		} else {
+			log.Debugf("failed to execute remote command '%s' error: '%s'",
+				t.params[0], err.Error())
+
+			cr = &resultcache.CommandResult{
+				ID:    t.id,
+				Error: err,
+			}
+		}
+
+		t.output.WriteCommand(cr)
+		t.output.Flush()
+
+		s.FinishTask(t)
+	}()
+}
+
+func (t *commandTask) reschedule(now time.Time) (err error) {
+	t.scheduled = time.Unix(now.Unix(), priorityCommandTaskNs)
+
+	return
+}
