@@ -1,20 +1,15 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 // Package version provides zabbix release version
@@ -22,19 +17,20 @@ package version
 
 import (
 	"fmt"
+	"runtime"
 	"strings"
 )
 
 const (
-	ZABBIX_REVDATE          = "13 December 2023"
-	ZABBIX_VERSION_MAJOR    = 6
+	ZABBIX_REVDATE          = "3 June 2024"
+	ZABBIX_VERSION_MAJOR    = 7
 	ZABBIX_VERSION_MINOR    = 0
-	ZABBIX_VERSION_PATCH    = 25
+	ZABBIX_VERSION_PATCH    = 0
 	ZABBIX_VERSION_RC       = ""
-	ZABBIX_VERSION_RC_NUM   = "2400"
-	ZABBIX_VERSION_REVISION = "1706b11e866"
-	copyrightMessage        = "Copyright (C) 2023 Zabbix SIA\n" +
-		"License GPLv2+: GNU GPL version 2 or later <https://www.gnu.org/licenses/>.\n" +
+	ZABBIX_VERSION_RC_NUM   = "1400"
+	ZABBIX_VERSION_REVISION = "49955f1fb5c"
+	copyrightMessage        = "Copyright (C) 2024 Zabbix SIA\n" +
+		"License AGPLv3: GNU Affero General Public License version 3 <https://www.gnu.org/licenses/>.\n" +
 		"This is free software: you are free to change and redistribute it according to\n" +
 		"the license. There is NO WARRANTY, to the extent permitted by law."
 )
@@ -84,6 +80,10 @@ func Long() string {
 	}
 	return ver
 }
+func LongNoRC() string {
+	var ver string = fmt.Sprintf("%d.%d.%d", Major(), Minor(), Patch())
+	return ver
+}
 
 func Short() string {
 	return fmt.Sprintf("%d.%d", Major(), Minor())
@@ -140,9 +140,15 @@ func TitleMessage() string {
 	return title
 }
 
+// Display shows program version.
+// Program version includes Zabbix revision and it's time and date, compilation time and date, Go compiler tree's
+// version string, copyright message, and additionalMessages provided by the caller function.
 func Display(additionalMessages []string) {
 	fmt.Printf("%s (Zabbix) %s\n", TitleMessage(), Long())
-	fmt.Printf("Revision %s %s, compilation time: %s %s\n", Revision(), RevDate(), CompileDate(), CompileTime())
+	fmt.Printf(
+		"Revision %s %s, compilation time: %s %s, built with: %s\n",
+		Revision(), RevDate(), CompileDate(), CompileTime(), runtime.Version(),
+	)
 
 	for _, msg := range additionalMessages {
 		fmt.Println(msg)
