@@ -1,36 +1,32 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 package zbxcmd
 
 import (
 	"bytes"
+	"errors"
 	"fmt"
 	"os/exec"
+	"path/filepath"
 	"strings"
 	"syscall"
 	"time"
 	"unsafe"
 
-	"git.zabbix.com/ap/plugin-support/log"
-
 	"golang.org/x/sys/windows"
+	"golang.zabbix.com/sdk/log"
 )
 
 type process struct {
@@ -38,10 +34,22 @@ type process struct {
 	Handle uintptr
 }
 
-var ntResumeProcess *syscall.Proc
+var (
+	ntResumeProcess *syscall.Proc
+	cmd_path        string
+)
 
 func execute(s string, timeout time.Duration, path string, strict bool) (out string, err error) {
-	cmd := exec.Command("cmd")
+	if cmd_path == "" {
+		cmd_exe := "cmd.exe"
+		if cmd_exe, err = exec.LookPath(cmd_exe); err != nil && !errors.Is(err, exec.ErrDot) {
+			return "", fmt.Errorf("Cannot find path to %s command: %s", cmd_exe, err)
+		}
+		if cmd_path, err = filepath.Abs(cmd_exe); err != nil {
+			return "", fmt.Errorf("Cannot find full path to %s command: %s", cmd_exe, err)
+		}
+	}
+	cmd := exec.Command(cmd_path)
 	cmd.Dir = path
 
 	var b bytes.Buffer
@@ -59,7 +67,7 @@ func execute(s string, timeout time.Duration, path string, strict bool) (out str
 		CmdLine:       fmt.Sprintf(`/C "%s"`, s),
 	}
 	if err = cmd.Start(); err != nil {
-		return "", fmt.Errorf("Cannot execute command: %s", err)
+		return "", fmt.Errorf("Cannot execute command (%s, path: %s): %s", s, path, err)
 	}
 
 	processHandle := windows.Handle((*process)(unsafe.Pointer(cmd.Process)).Handle)
@@ -107,14 +115,23 @@ func execute(s string, timeout time.Duration, path string, strict bool) (out str
 	return strings.TrimRight(b.String(), " \t\r\n"), nil
 }
 
-func ExecuteBackground(s string) error {
-	cmd := exec.Command("cmd")
+func ExecuteBackground(s string) (err error) {
+	if cmd_path == "" {
+		cmd_exe := "cmd.exe"
+		if cmd_exe, err = exec.LookPath(cmd_exe); err != nil && !errors.Is(err, exec.ErrDot) {
+			return fmt.Errorf("Cannot find path to %s command: %s", cmd_exe, err)
+		}
+		if cmd_path, err = filepath.Abs(cmd_exe); err != nil {
+			return fmt.Errorf("Cannot find full path to %s command: %s", cmd_exe, err)
+		}
+	}
+	cmd := exec.Command(cmd_path)
 	cmd.SysProcAttr = &windows.SysProcAttr{
 		CmdLine: fmt.Sprintf(`/C %s`, s),
 	}
 
-	if err := cmd.Start(); err != nil {
-		return fmt.Errorf("Cannot execute command: %s", err)
+	if err = cmd.Start(); err != nil {
+		return fmt.Errorf("Cannot execute command (%s): %s", s, err)
 	}
 
 	go func() { _ = cmd.Wait() }()
