@@ -2,22 +2,17 @@
 // +build linux
 
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 package proc
@@ -28,6 +23,7 @@ package proc
 import "C"
 
 import (
+	"encoding/json"
 	"errors"
 	"fmt"
 	"math"
@@ -37,9 +33,10 @@ import (
 	"sync"
 	"time"
 
-	"git.zabbix.com/ap/plugin-support/log"
-	"git.zabbix.com/ap/plugin-support/plugin"
-	"zabbix.com/pkg/procfs"
+	"golang.zabbix.com/agent2/pkg/procfs"
+	"golang.zabbix.com/sdk/errs"
+	"golang.zabbix.com/sdk/log"
+	"golang.zabbix.com/sdk/plugin"
 )
 
 const (
@@ -53,7 +50,7 @@ type Plugin struct {
 	queries map[procQuery]*cpuUtilStats
 	mutex   sync.Mutex
 	scanid  uint64
-	stats   map[int64]*cpuUtil
+	stats   map[int64]*procStat
 }
 
 type PluginExport struct {
@@ -61,7 +58,7 @@ type PluginExport struct {
 }
 
 var impl Plugin = Plugin{
-	stats:   make(map[int64]*cpuUtil),
+	stats:   make(map[int64]*procStat),
 	queries: make(map[procQuery]*cpuUtilStats),
 }
 
@@ -69,6 +66,23 @@ var implExport PluginExport = PluginExport{}
 
 type historyIndex int
 
+func init() {
+	err := plugin.RegisterMetrics(&impl, "Proc", "proc.cpu.util", "Process CPU utilization percentage.")
+	if err != nil {
+		panic(errs.Wrap(err, "failed to register metrics"))
+	}
+
+	err = plugin.RegisterMetrics(
+		&implExport, "ProcExporter",
+		"proc.mem", "Process memory utilization values.",
+		"proc.num", "The number of processes.",
+		"proc.get", "List of OS processes with statistics.",
+	)
+	if err != nil {
+		panic(errs.Wrap(err, "failed to register metrics"))
+	}
+}
+
 func (h historyIndex) inc() historyIndex {
 	h++
 	if h == maxHistory {
@@ -142,11 +156,84 @@ type procInfo struct {
 	state   string
 }
 
-type cpuUtil struct {
-	utime   uint64
-	stime   uint64
-	started uint64
-	err     error
+type procStatus struct {
+	Pid           uint64  `json:"pid"`
+	Ppid          int64   `json:"ppid"`
+	Tgid          int64   `json:"-"`
+	Name          string  `json:"name"`
+	ThreadName    string  `json:"-"`
+	Cmdline       string  `json:"cmdline"`
+	User          string  `json:"user"`
+	Group         string  `json:"group"`
+	UserID        int64   `json:"uid"`
+	GroupID       int64   `json:"gid"`
+	Vsize         int64   `json:"vsize"`
+	Pmem          float64 `json:"pmem"`
+	Rss           int64   `json:"rss"`
+	Data          int64   `json:"data"`
+	Exe           int64   `json:"exe"`
+	Hwm           int64   `json:"hwm"`
+	Lck           int64   `json:"lck"`
+	Lib           int64   `json:"lib"`
+	Peak          int64   `json:"peak"`
+	Pin           int64   `json:"pin"`
+	Pte           int64   `json:"pte"`
+	Size          int64   `json:"size"`
+	Stk           int64   `json:"stk"`
+	Swap          int64   `json:"swap"`
+	CpuTimeUser   float64 `json:"cputime_user"`
+	CpuTimeSystem float64 `json:"cputime_system"`
+	State         string  `json:"state"`
+	CtxSwitches   int64   `json:"ctx_switches"`
+	Threads       int64   `json:"threads"`
+	PageFaults    int64   `json:"page_faults"`
+}
+
+type procSummary struct {
+	Name          string  `json:"name"`
+	Processes     int     `json:"processes"`
+	Vsize         int64   `json:"vsize"`
+	Pmem          float64 `json:"pmem"`
+	Rss           int64   `json:"rss"`
+	Data          int64   `json:"data"`
+	Exe           int64   `json:"exe"`
+	Lck           int64   `json:"lck"`
+	Lib           int64   `json:"lib"`
+	Pin           int64   `json:"pin"`
+	Pte           int64   `json:"pte"`
+	Size          int64   `json:"size"`
+	Stk           int64   `json:"stk"`
+	Swap          int64   `json:"swap"`
+	CpuTimeUser   float64 `json:"cputime_user"`
+	CpuTimeSystem float64 `json:"cputime_system"`
+	CtxSwitches   int64   `json:"ctx_switches"`
+	Threads       int64   `json:"threads"`
+	PageFaults    int64   `json:"page_faults"`
+}
+
+type thread struct {
+	Pid           int64   `json:"pid"`
+	Ppid          int64   `json:"ppid"`
+	Name          string  `json:"name"`
+	User          string  `json:"user"`
+	Group         string  `json:"group"`
+	UserID        int64   `json:"uid"`
+	GroupID       int64   `json:"gid"`
+	Tid           uint64  `json:"tid"`
+	ThreadName    string  `json:"tname"`
+	CpuTimeUser   float64 `json:"cputime_user"`
+	CpuTimeSystem float64 `json:"cputime_system"`
+	State         string  `json:"state"`
+	CtxSwitches   int64   `json:"ctx_switches"`
+	PageFaults    int64   `json:"page_faults"`
+}
+
+type procStat struct {
+	utime      uint64
+	stime      uint64
+	started    uint64
+	pageFaults int64
+	err        error
 }
 
 func (q *cpuUtilQuery) match(p *procInfo) bool {
@@ -224,7 +311,7 @@ func (p *Plugin) Collect() (err error) {
 	}
 	p.Tracef("%s() queries:%d", log.Caller(), len(p.queries))
 
-	stats := make(map[int64]*cpuUtil)
+	stats := make(map[int64]*procStat)
 	// find processes matching prepared queries
 	for _, p := range processes {
 		var monitored bool
@@ -235,7 +322,7 @@ func (p *Plugin) Collect() (err error) {
 			}
 		}
 		if monitored {
-			stats[p.pid] = &cpuUtil{}
+			stats[p.pid] = &procStat{}
 		}
 	}
 
@@ -247,7 +334,7 @@ func (p *Plugin) Collect() (err error) {
 
 	now := time.Now()
 	for pid, stat := range stats {
-		p.getProcCpuUtil(pid, stat)
+		getProcessStats(fmt.Sprintf("%d", pid), stat)
 		if stat.err != nil {
 			p.Debugf("cannot get process %d CPU utilization statistics: %s", pid, stat.err)
 		}
@@ -256,7 +343,7 @@ func (p *Plugin) Collect() (err error) {
 	// gather process utilization for queries
 	for _, q := range queries {
 		for _, pid := range q.pids {
-			var stat, last *cpuUtil
+			var stat, last *procStat
 			var ok bool
 			if stat, ok = stats[pid]; !ok || stat.err != nil {
 				continue
@@ -413,10 +500,11 @@ func (p *Plugin) Export(key string, params []string, ctx plugin.ContextProvider)
 		p.queries[query] = stats
 		p.Debugf("registered new CPU utilization query: %s, %s, %s", name, user, cmdline)
 	} else {
-		p.Debugf("cannot register CPU utilization query: %s", err)
+		err = fmt.Errorf("cannot register CPU utilization query: %s", err)
 	}
 	return
 }
+
 func (p *PluginExport) prepareQuery(q *procQuery) (query *cpuUtilQuery, flags int, err error) {
 	regxp, err := regexp.Compile(q.cmdline)
 	if err != nil {
@@ -450,6 +538,8 @@ func (p *PluginExport) Export(key string, params []string, ctx plugin.ContextPro
 		return p.exportProcMem(params)
 	case "proc.num":
 		return p.exportProcNum(params)
+	case "proc.get":
+		return p.exportProcGet(params)
 	}
 
 	return nil, plugin.UnsupportedMetricError
@@ -521,8 +611,7 @@ func (p *PluginExport) exportProcMem(params []string) (result interface{}, err e
 	if cmdline != "" {
 		cmdRgx, err = regexp.Compile(cmdline)
 		if err != nil {
-			p.Debugf("Failed to compile provided regex expression '%s': %s", cmdline, err.Error())
-			return 0, nil
+			return nil, fmt.Errorf("Failed to compile regular expression '%s': %s", cmdline, err.Error())
 		}
 	}
 
@@ -530,9 +619,7 @@ func (p *PluginExport) exportProcMem(params []string) (result interface{}, err e
 	if usr != nil {
 		userID, err = strconv.ParseInt(usr.Uid, 10, 64)
 		if err != nil {
-			p.Logger.Tracef(
-				"failed to convert user id '%s' to uint64 for user '%s'", usr.Uid, usr.Username)
-			return 0, nil
+			return nil, fmt.Errorf("Failed to parse userid '%s' for user '%s", usr.Uid, usr.Username)
 		}
 	}
 
@@ -671,8 +758,7 @@ func (p *PluginExport) exportProcNum(params []string) (interface{}, error) {
 
 	query, flags, err := p.prepareQuery(&procQuery{name, userName, cmdline, state})
 	if err != nil {
-		p.Debugf("Failed to prepare query: %s", err.Error())
-		return count, nil
+		return nil, fmt.Errorf("Failed to prepare query: %s", err.Error())
 	}
 
 	procs, err := getProcesses(flags)
@@ -695,6 +781,197 @@ func (p *PluginExport) exportProcNum(params []string) (interface{}, error) {
 	return count, nil
 }
 
+func (p *PluginExport) exportProcGet(params []string) (interface{}, error) {
+	var name, userName, cmdline, mode string
+	switch len(params) {
+	case 4:
+		mode = params[3]
+		switch mode {
+		case "process", "", "thread", "summary":
+		default:
+			return nil, errors.New("Invalid fourth parameter")
+		}
+		fallthrough
+	case 3:
+		cmdline = params[2]
+		if cmdline != "" && mode == "summary" {
+			return nil, errors.New("Invalid fourth parameter")
+		}
+		fallthrough
+	case 2:
+		userName = params[1]
+		if userName != "" {
+			if _, err := user.Lookup(userName); err != nil {
+				return "[]", nil
+			}
+		}
+		fallthrough
+	case 1:
+		name = params[0]
+	case 0:
+	default:
+		return nil, errors.New("Too many parameters.")
+	}
+
+	array := make([]procStatus, 0)
+	threadArray := make([]thread, 0)
+	summaryArray := make([]procSummary, 0)
+
+	var pids []string
+	var err error
+	if pids, err = getPids(); err != nil {
+		return nil, fmt.Errorf("Cannot open /proc: %s", err)
+	}
+
+	query, _, err := p.prepareQuery(&procQuery{name, userName, cmdline, ""})
+	if err != nil {
+		return nil, fmt.Errorf("Failed to prepare query: %s", err.Error())
+	}
+
+	if mode != "thread" {
+		for _, pid := range pids {
+			data := procStatus{}
+			if err := parseProcessStatus(pid, &data); err != nil {
+				continue
+			}
+			getProcessNames(pid, &data)
+			getProcessCalculatedMetrics(pid, &data)
+			getProcessCpuTimes(pid, &data)
+
+			pi := procInfo{int64(data.Pid), data.Name, 0, "", data.Name, ""}
+			setProcessUserInfo(pid, &data)
+			pi.userid = data.UserID
+			pi.cmdline = data.Cmdline
+
+			if query.match(&pi) {
+				array = append(array, data)
+			}
+		}
+	} else {
+		var threadIds []string
+		for _, pid := range pids {
+			tids, err := getThreadIds(pid)
+			if err == nil {
+				threadIds = append(threadIds, tids...)
+			}
+		}
+		for _, tid := range threadIds {
+			data := procStatus{}
+			if err := parseProcessStatus(tid, &data); err != nil {
+				continue
+			}
+			procPath := fmt.Sprintf("%d", data.Tgid) + "/task/" + tid
+			getProcessNames(tid, &data)
+			getProcessCalculatedMetrics(tid, &data)
+			getProcessCpuTimes(procPath, &data)
+
+			setProcessUserInfo(tid, &data)
+
+			pi := procInfo{int64(data.Pid), data.Name, data.UserID, data.Cmdline, data.Name, ""}
+			if query.match(&pi) {
+				threadArray = append(threadArray, thread{
+					data.Tgid, data.Ppid,
+					data.Name, data.User, data.Group, data.UserID, data.GroupID,
+					data.Pid, data.ThreadName, data.CpuTimeUser, data.CpuTimeSystem,
+					data.State, data.CtxSwitches, data.PageFaults,
+				})
+			}
+		}
+	}
+
+	var jsonArray []byte
+	switch mode {
+	case "summary":
+		var processed []string
+	processes:
+		for i, proc := range array {
+			for _, j := range processed {
+				if j == proc.Name {
+					continue processes
+				}
+			}
+
+			procSum := procSummary{
+				proc.Name, 1, proc.Vsize, proc.Pmem, proc.Rss, proc.Data,
+				proc.Exe, proc.Lck, proc.Lib, proc.Pin, proc.Pte, proc.Size, proc.Stk,
+				proc.Swap, proc.CpuTimeUser, proc.CpuTimeSystem, proc.CtxSwitches, proc.Threads,
+				proc.PageFaults,
+			}
+
+			if len(array) > i+1 {
+				for _, procCmp := range array[i+1:] {
+					if procCmp.Name != proc.Name {
+						continue
+					}
+					procSum.Processes++
+					procSum.Threads += procCmp.Threads
+					addNonNegative(&procSum.Vsize, procCmp.Vsize)
+					addNonNegativeFloat(&procSum.Pmem, procCmp.Pmem)
+					addNonNegative(&procSum.Rss, procCmp.Rss)
+					addNonNegative(&procSum.Data, procCmp.Data)
+					addNonNegative(&procSum.Exe, procCmp.Exe)
+					addNonNegative(&procSum.Lck, procCmp.Lck)
+					addNonNegative(&procSum.Lib, procCmp.Lib)
+					addNonNegative(&procSum.Pin, procCmp.Pin)
+					addNonNegative(&procSum.Pte, procCmp.Pte)
+					addNonNegative(&procSum.Size, procCmp.Size)
+					addNonNegative(&procSum.Stk, procCmp.Stk)
+					addNonNegative(&procSum.Swap, procCmp.Swap)
+					addNonNegativeFloat(&procSum.CpuTimeUser, procCmp.CpuTimeUser)
+					addNonNegativeFloat(&procSum.CpuTimeSystem, procCmp.CpuTimeSystem)
+					addNonNegative(&procSum.CtxSwitches, procCmp.CtxSwitches)
+					addNonNegative(&procSum.PageFaults, procCmp.PageFaults)
+				}
+			}
+			processed = append(processed, proc.Name)
+			summaryArray = append(summaryArray, procSum)
+		}
+		jsonArray, err = json.Marshal(summaryArray)
+	case "thread":
+		jsonArray, err = json.Marshal(threadArray)
+	default:
+		jsonArray, err = json.Marshal(array)
+	}
+
+	if err != nil {
+		return nil, fmt.Errorf("Cannot create JSON array: %s", err)
+	}
+
+	return string(jsonArray), nil
+}
+
+func setProcessUserInfo(pid string, ps *procStatus) {
+	pu, err := getProcessUserInfo(pid)
+	if err != nil {
+		ps.UserID = -1
+		ps.GroupID = -1
+		ps.User = "-1"
+		ps.Group = "-1"
+
+		return
+	}
+
+	ps.UserID = pu.uid
+	ps.GroupID = pu.gid
+
+	uStr := strconv.FormatInt(pu.uid, 10)
+	gStr := strconv.FormatInt(pu.gid, 10)
+
+	u, err := user.LookupId(uStr)
+	if err == nil {
+		ps.User = u.Username
+	} else {
+		ps.User = uStr
+	}
+
+	g, err := user.LookupGroupId(gStr)
+	if err == nil {
+		ps.Group = g.Name
+	} else {
+		ps.Group = gStr
+	}
+}
+
 func getMax(a, b float64) float64 {
 	if a > b {
 		return a
@@ -775,11 +1052,3 @@ func (p *PluginExport) validFile(proc *procInfo, name string, uid int64, cmdRgx
 
 	return true
 }
-
-func init() {
-	plugin.RegisterMetrics(&impl, "Proc", "proc.cpu.util", "Process CPU utilization percentage.")
-	plugin.RegisterMetrics(&implExport, "ProcExporter",
-		"proc.mem", "Process memory utilization values.",
-		"proc.num", "The number of processes.",
-	)
-}
