@@ -9,21 +9,34 @@ import (
 	"fmt"
 	"time"
 
-	"git.zabbix.com/ap/plugin-support/plugin"
-	"zabbix.com/pkg/pdh"
-	"zabbix.com/pkg/win32"
+	"golang.zabbix.com/agent2/pkg/pdh"
+	"golang.zabbix.com/agent2/pkg/win32"
+	"golang.zabbix.com/sdk/errs"
+	"golang.zabbix.com/sdk/plugin"
 )
 
-//Plugin -
+var impl Plugin
+
+// Plugin -
 type Plugin struct {
 	plugin.Base
 	nextObjectRefresh  time.Time
 	nextEngNameRefresh time.Time
 }
 
-var impl Plugin
+func init() {
+	err := plugin.RegisterMetrics(&impl, "WindowsPerfInstance",
+		"perf_instance.discovery", "Get Windows performance instance object list.",
+		"perf_instance_en.discovery", "Get Windows performance instance object English list.",
+	)
+	if err != nil {
+		panic(errs.Wrap(err, "failed to register metrics"))
+	}
 
-//Export -
+	impl.SetCapacity(1)
+}
+
+// Export -
 func (p *Plugin) Export(key string, params []string, ctx plugin.ContextProvider) (response interface{}, err error) {
 	if len(params) > 1 {
 		return nil, errors.New("Too many parameters.")
@@ -71,14 +84,6 @@ func (p *Plugin) Export(key string, params []string, ctx plugin.ContextProvider)
 	return string(respJSON), nil
 }
 
-func init() {
-	plugin.RegisterMetrics(&impl, "WindowsPerfInstance",
-		"perf_instance.discovery", "Get Windows performance instance object list.",
-		"perf_instance_en.discovery", "Get Windows performance instance object English list.",
-	)
-	impl.SetCapacity(1)
-}
-
 func (p *Plugin) refreshObjects() (err error) {
 	if time.Now().After(p.nextObjectRefresh) {
 		_, err = win32.PdhEnumObject()
