@@ -1,61 +1,48 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
-#include "common.h"
 #include "zbxalgo.h"
 
 /******************************************************************************
  *                                                                            *
  * Purpose: create singly linked list (with custom memory functions)          *
  *                                                                            *
- * Parameters: queue           - [IN] the list                                *
+ * Parameters: list            - [IN/OUT]                                     *
  *             mem_malloc_func - [IN] callback for malloc                     *
  *             mem_free_func   - [IN] callback for free                       *
  *                                                                            *
  ******************************************************************************/
-void	zbx_list_create_ext(zbx_list_t *queue, zbx_mem_malloc_func_t mem_malloc_func, zbx_mem_free_func_t mem_free_func)
+void	zbx_list_create_ext(zbx_list_t *list, zbx_mem_malloc_func_t mem_malloc_func, zbx_mem_free_func_t mem_free_func)
 {
-	memset(queue, 0, sizeof(*queue));
+	memset(list, 0, sizeof(*list));
 
-	queue->mem_malloc_func = mem_malloc_func;
-	queue->mem_free_func = mem_free_func;
+	list->mem_malloc_func = mem_malloc_func;
+	list->mem_free_func = mem_free_func;
 }
 
 /******************************************************************************
  *                                                                            *
  * Purpose: create singly linked list                                         *
  *                                                                            *
- * Parameters: queue - [IN] the list                                          *
+ * Parameters: list - [IN]                                                    *
  *                                                                            *
  ******************************************************************************/
-void	zbx_list_create(zbx_list_t *queue)
+void	zbx_list_create(zbx_list_t *list)
 {
-	zbx_list_create_ext(queue, ZBX_DEFAULT_MEM_MALLOC_FUNC, ZBX_DEFAULT_MEM_FREE_FUNC);
+	zbx_list_create_ext(list, ZBX_DEFAULT_MEM_MALLOC_FUNC, ZBX_DEFAULT_MEM_FREE_FUNC);
 }
 
-/******************************************************************************
- *                                                                            *
- * Purpose: destroy list                                                      *
- *                                                                            *
- * Parameters: list - [IN] the list                                           *
- *                                                                            *
- ******************************************************************************/
 void	zbx_list_destroy(zbx_list_t *list)
 {
 	while (FAIL != zbx_list_pop(list, NULL))
@@ -66,20 +53,21 @@ void	zbx_list_destroy(zbx_list_t *list)
  *                                                                            *
  * Purpose: allocate memory and initialize a new list item                    *
  *                                                                            *
- * Parameters: list     - [IN] the list                                       *
- *             value    - [IN] the data to be stored                          *
- *             created  - [OUT] pointer to the created list item              *
+ * Parameters: list    - [IN]                                                 *
+ *             value   - [IN] Data to be stored. Ownership of 'value' goes    *
+ *                             to list item.                                  *
+ *             created - [OUT] pointer to the created list item               *
  *                                                                            *
  ******************************************************************************/
 static void	list_create_item(zbx_list_t *list, void *value, zbx_list_item_t **created)
 {
 	zbx_list_item_t *item;
 
-	ZBX_UNUSED(list);
-
-	item = (zbx_list_item_t *)list->mem_malloc_func(NULL, sizeof(zbx_list_item_t));
-	item->next = NULL;
-	item->data = value;
+	if (NULL != (item = (zbx_list_item_t *)list->mem_malloc_func(NULL, sizeof(zbx_list_item_t))))
+	{
+		item->next = NULL;
+		item->data = value;
+	}
 
 	*created = item;
 }
@@ -88,19 +76,26 @@ static void	list_create_item(zbx_list_t *list, void *value, zbx_list_item_t **cr
  *                                                                            *
  * Purpose: insert value after specified position in the list                 *
  *                                                                            *
- * Parameters: list     - [IN] the list                                       *
+ * Parameters: list     - [IN]                                                *
  *             after    - [IN] specified position (can be NULL to insert at   *
  *                             the end of the list)                           *
- *             value    - [IN] the value to be inserted                       *
+ *             value    - [IN] Value to be inserted. Ownership of 'value'     *
+ *                             goes to list item.                             *
  *             inserted - [OUT] pointer to the inserted list item             *
  *                                                                            *
+ * Return value: SUCCEED - the item was prepended successfully                *
+ *               FAIL    - memory allocation error                            *
+ *                                                                            *
  ******************************************************************************/
-void	zbx_list_insert_after(zbx_list_t *list, zbx_list_item_t *after, void *value, zbx_list_item_t **inserted)
+int	zbx_list_insert_after(zbx_list_t *list, zbx_list_item_t *after, void *value, zbx_list_item_t **inserted)
 {
 	zbx_list_item_t *item;
 
 	list_create_item(list, value, &item);
 
+	if (NULL == item)
+		return FAIL;
+
 	if (NULL == after)
 		after = list->tail;
 
@@ -119,36 +114,50 @@ void	zbx_list_insert_after(zbx_list_t *list, zbx_list_item_t *after, void *value
 
 	if (NULL != inserted)
 		*inserted = item;
+
+	return SUCCEED;
 }
 
 /******************************************************************************
  *                                                                            *
  * Purpose: append value to the end of the list                               *
  *                                                                            *
- * Parameters: list     - [IN] the list                                       *
- *             value    - [IN] the value to append                            *
+ * Parameters: list     - [IN]                                                *
+ *             value    - [IN] Value to append. Ownership of 'value' goes to  *
+ *                             list item.                                     *
  *             inserted - [OUT] pointer to the inserted list item             *
  *                                                                            *
+ * Return value: SUCCEED - the item was prepended successfully                *
+ *               FAIL    - memory allocation error                            *
+ *                                                                            *
  ******************************************************************************/
-void	zbx_list_append(zbx_list_t *list, void *value, zbx_list_item_t **inserted)
+int	zbx_list_append(zbx_list_t *list, void *value, zbx_list_item_t **inserted)
 {
-	zbx_list_insert_after(list, NULL, value, inserted);
+	return zbx_list_insert_after(list, NULL, value, inserted);
 }
 
 /******************************************************************************
  *                                                                            *
  * Purpose: prepend value to the beginning of the list                        *
  *                                                                            *
- * Parameters: list     - [IN] the list                                       *
- *             value    - [IN] the value to prepend                           *
+ * Parameters: list     - [IN]                                                *
+ *             value    - [IN] Value to prepend. Ownership of 'value' goes to *
+ *                             list item.                                     *
  *             inserted - [OUT] pointer to the inserted list item             *
  *                                                                            *
+ * Return value: SUCCEED - the item was prepended successfully                *
+ *               FAIL    - memory allocation error                            *
+ *                                                                            *
  ******************************************************************************/
-void	zbx_list_prepend(zbx_list_t *list, void *value, zbx_list_item_t **inserted)
+int	zbx_list_prepend(zbx_list_t *list, void *value, zbx_list_item_t **inserted)
 {
 	zbx_list_item_t *item;
 
 	list_create_item(list, value, &item);
+
+	if (NULL == item)
+		return FAIL;
+
 	item->next = list->head;
 	list->head = item;
 
@@ -157,14 +166,17 @@ void	zbx_list_prepend(zbx_list_t *list, void *value, zbx_list_item_t **inserted)
 
 	if (NULL != inserted)
 		*inserted = item;
+
+	return SUCCEED;
 }
 
 /******************************************************************************
  *                                                                            *
  * Purpose: removes a value from the beginning of the list                    *
  *                                                                            *
- * Parameters: list  - [IN]  the list                                         *
- *             value - [OUT] the value                                        *
+ * Parameters: list  - [IN]                                                   *
+ *             value - [IN/OUT] pointer to data removed from list. Ownership  *
+ *                              goes to caller.                               *
  *                                                                            *
  * Return value: SUCCEED is returned if list is not empty, otherwise, FAIL is *
  *               returned.                                                    *
@@ -193,10 +205,11 @@ int	zbx_list_pop(zbx_list_t *list, void **value)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: get value from the queue without dequeuing                        *
+ * Purpose: get value from the list without removing                          *
  *                                                                            *
- * Parameters: list  - [IN]  the list                                         *
- *             value - [OUT] the value                                        *
+ * Parameters: list  - [IN]                                                   *
+ *             value - [OUT] non-owning pointer to data stored in first       *
+ *                           element of list                                  *
  *                                                                            *
  * Return value: SUCCEED is returned if list is not empty, otherwise, FAIL is *
  *               returned.                                                    *
@@ -217,8 +230,8 @@ int	zbx_list_peek(const zbx_list_t *list, void **value)
  *                                                                            *
  * Purpose: initialize list iterator                                          *
  *                                                                            *
- * Parameters: list     - [IN]  the list                                      *
- *             iterator - [OUT] the iterator to be initialized                *
+ * Parameters: list     - [IN]                                                *
+ *             iterator - [OUT] iterator to be initialized                    *
  *                                                                            *
  ******************************************************************************/
 void	zbx_list_iterator_init(zbx_list_t *list, zbx_list_iterator_t *iterator)
@@ -228,11 +241,39 @@ void	zbx_list_iterator_init(zbx_list_t *list, zbx_list_iterator_t *iterator)
 	iterator->current = NULL;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Purpose: initialize list iterator starting with the specified item         *
+ *                                                                            *
+ * Parameters: list     - [IN]                                                *
+ *             next     - [IN] item to start with, if NULL the iterator       *
+ *                             will be start with first item                  *
+ *             iterator - [OUT] iterator to be initialized                    *
+ *                                                                            *
+ *  Return value: SUCCEED - iterator was initialized successfully             *
+ *                FAIL    - list is empty                                     *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_list_iterator_init_with(zbx_list_t *list, zbx_list_item_t *next, zbx_list_iterator_t *iterator)
+{
+	if (NULL == next)
+	{
+		zbx_list_iterator_init(list, iterator);
+		return zbx_list_iterator_next(iterator);
+	}
+
+	iterator->list = list;
+	iterator->next = next->next;
+	iterator->current = next;
+
+	return SUCCEED;
+}
+
 /******************************************************************************
  *                                                                            *
  * Purpose: advance list iterator                                             *
  *                                                                            *
- * Parameters: iterator - [IN] the iterator to be advanced                    *
+ * Parameters: iterator - [IN] iterator to be advanced                        *
  *                                                                            *
  * Return value: SUCCEED is returned if next list item exists, otherwise,     *
  *               FAIL is returned.                                            *
@@ -248,6 +289,8 @@ int	zbx_list_iterator_next(zbx_list_iterator_t *iterator)
 		return SUCCEED;
 	}
 
+	iterator->current = NULL;
+
 	return FAIL;
 }
 
@@ -256,7 +299,8 @@ int	zbx_list_iterator_next(zbx_list_iterator_t *iterator)
  * Purpose: get value without removing it from list                           *
  *                                                                            *
  * Parameters: iterator - [IN]  initialized list iterator                     *
- *             value    - [OUT] the value                                     *
+ *             value    - [OUT]  non-owning pointer to data stored in element *
+ *                               pointed to by iterator                       *
  *                                                                            *
  * Return value: SUCCEED is returned if item exists, otherwise, FAIL is       *
  *               returned.                                                    *
@@ -334,7 +378,6 @@ void	zbx_list_iterator_update(zbx_list_iterator_t *iterator)
 		iterator->next = iterator->current->next;
 }
 
-
 /******************************************************************************
  *                                                                            *
  * Purpose: removes next iterator value from list                             *
