@@ -1,39 +1,45 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 #include "audit.h"
+#include "audit/zbxaudit.h"
 
-#include "log.h"
+#include "zbxalgo.h"
 #include "zbxjson.h"
-#include "dbcache.h"
+#include "zbxcacheconfig.h"
+#include "zbxnum.h"
+#include "zbx_scripts_constants.h"
 
 #define AUDIT_USERID		__UINT64_C(0)
 #define AUDIT_USERID_SQL	"null"
 #define AUDIT_USERNAME		"System"
 #define AUDIT_IP		""
 
-static int		audit_mode;
 static zbx_hashset_t	zbx_audit;
 
-int	zbx_get_audit_mode(void)
+static int		auditlog_mode;
+
+int	zbx_get_auditlog_mode(void)
 {
-	return audit_mode;
+	return auditlog_mode;
+}
+
+static int		auditlog_enabled;
+
+int	zbx_get_auditlog_enabled(void)
+{
+	return auditlog_enabled;
 }
 
 zbx_hashset_t	*zbx_get_audit_hashset(void)
@@ -59,8 +65,8 @@ zbx_audit_entry_t	*zbx_audit_entry_init(zbx_uint64_t id, const int id_table, con
 	return audit_entry;
 }
 
-zbx_audit_entry_t	*zbx_audit_entry_init_cuid(const char *cuid, const int id_table, const char *name, int audit_action,
-		int resource_type)
+zbx_audit_entry_t	*zbx_audit_entry_init_cuid(const char *cuid, const int id_table, const char *name,
+		int audit_action, int resource_type)
 {
 	zbx_audit_entry_t	*audit_entry;
 
@@ -169,15 +175,15 @@ static void	delete_json(struct zbx_json *json, const char *audit_op, const char
  ******************************************************************************/
 int	zbx_auditlog_global_script(unsigned char script_type, unsigned char script_execute_on,
 		const char *script_command_orig, zbx_uint64_t hostid, const char *hostname, zbx_uint64_t eventid,
-		zbx_uint64_t proxy_hostid, zbx_uint64_t userid, const char *username, const char *clientip,
+		zbx_uint64_t proxyid, zbx_uint64_t userid, const char *username, const char *clientip,
 		const char *output, const char *error)
 {
 	int		ret = SUCCEED;
 	char		auditid_cuid[CUID_LEN], execute_on_s[MAX_ID_LEN + 1], hostid_s[MAX_ID_LEN + 1],
-			eventid_s[MAX_ID_LEN + 1], proxy_hostid_s[MAX_ID_LEN + 1];
-	char		*details_esc;
+			eventid_s[MAX_ID_LEN + 1], proxyid_s[MAX_ID_LEN + 1];
 	struct zbx_json	details_json;
 	zbx_config_t	cfg;
+	zbx_db_insert_t	db_insert;
 
 	zabbix_log(LOG_LEVEL_TRACE, "In %s()", __func__);
 
@@ -203,10 +209,10 @@ int	zbx_auditlog_global_script(unsigned char script_type, unsigned char script_e
 	zbx_snprintf(hostid_s, sizeof(hostid_s), ZBX_FS_UI64, hostid);
 	append_str_json(&details_json, AUDIT_DETAILS_ACTION_ADD, "script.hostid", hostid_s);
 
-	if (0 != proxy_hostid)
+	if (0 != proxyid)
 	{
-		zbx_snprintf(proxy_hostid_s, sizeof(proxy_hostid_s), ZBX_FS_UI64, proxy_hostid);
-		append_str_json(&details_json, AUDIT_DETAILS_ACTION_ADD, "script.proxy_hostid", proxy_hostid_s);
+		zbx_snprintf(proxyid_s, sizeof(proxyid_s), ZBX_FS_UI64, proxyid);
+		append_str_json(&details_json, AUDIT_DETAILS_ACTION_ADD, "script.proxyid", proxyid_s);
 	}
 
 	if (ZBX_SCRIPT_TYPE_WEBHOOK != script_type)
@@ -218,19 +224,15 @@ int	zbx_auditlog_global_script(unsigned char script_type, unsigned char script_e
 	if (NULL != error)
 		append_str_json(&details_json, AUDIT_DETAILS_ACTION_ADD, "script.error", error);
 
-	details_esc = DBdyn_escape_string(details_json.buffer);
+	zbx_db_insert_prepare(&db_insert, "auditlog", "auditid", "userid", "username", "clock", "action", "ip",
+			"resourceid", "resourcename", "resourcetype", "recordsetid", "details", NULL);
 
-	if (ZBX_DB_OK > DBexecute("insert into auditlog (auditid,userid,username,clock,action,ip,resourceid,"
-			"resourcename,resourcetype,recordsetid,details) values ('%s'," ZBX_FS_UI64 ",'%s',%d,'%d','%s',"
-			ZBX_FS_UI64 ",'%s',%d,'%s','%s')", auditid_cuid, userid, username, (int)time(NULL),
-			AUDIT_ACTION_EXECUTE, clientip, hostid, hostname, AUDIT_RESOURCE_SCRIPT, auditid_cuid,
-			details_esc))
-	{
-		ret = FAIL;
-	}
+	zbx_db_insert_add_values(&db_insert, auditid_cuid, userid, username, (int)time(NULL), ZBX_AUDIT_ACTION_EXECUTE,
+			clientip, hostid, hostname, ZBX_AUDIT_RESOURCE_SCRIPT, auditid_cuid, details_json.buffer);
 
-	zbx_free(details_esc);
+	ret = zbx_db_insert_execute(&db_insert);
 
+	zbx_db_insert_clean(&db_insert);
 	zbx_json_free(&details_json);
 out:
 	zabbix_log(LOG_LEVEL_TRACE, "End of %s():%s", __func__, zbx_result_string(ret));
@@ -262,12 +264,12 @@ static int	zbx_audit_compare_func(const void *d1, const void *d2)
 	return zbx_strcmp_null((*audit_entry_1)->cuid, (*audit_entry_2)->cuid);
 }
 
-void	zbx_audit_clean(void)
+void	zbx_audit_clean(int audit_context_mode)
 {
 	zbx_hashset_iter_t	iter;
 	zbx_audit_entry_t	**audit_entry;
 
-	RETURN_IF_AUDIT_OFF();
+	RETURN_IF_AUDIT_OFF(audit_context_mode);
 
 	zbx_hashset_iter_reset(&zbx_audit, &iter);
 
@@ -282,23 +284,46 @@ void	zbx_audit_clean(void)
 	zbx_hashset_destroy(&zbx_audit);
 }
 
-void	zbx_audit_init(int audit_mode_set)
+void	zbx_audit_init(int auditlog_enabled_set, int auditlog_mode_set, int audit_context_mode)
 {
-	audit_mode = audit_mode_set;
-	RETURN_IF_AUDIT_OFF();
+	auditlog_enabled = auditlog_enabled_set;
+	auditlog_mode = auditlog_mode_set;
+	RETURN_IF_AUDIT_OFF(audit_context_mode);
 #define AUDIT_HASHSET_DEF_SIZE	100
 	zbx_hashset_create(&zbx_audit, AUDIT_HASHSET_DEF_SIZE, zbx_audit_hash_func, zbx_audit_compare_func);
 #undef AUDIT_HASHSET_DEF_SIZE
 }
 
-void	zbx_audit_flush(void)
+void	zbx_audit_prepare(int audit_context_mode)
+{
+	zbx_config_t	cfg;
+
+	zbx_config_get(&cfg, ZBX_CONFIG_FLAGS_AUDITLOG_ENABLED | ZBX_CONFIG_FLAGS_AUDITLOG_MODE);
+	zbx_audit_init(cfg.auditlog_enabled, cfg.auditlog_mode, audit_context_mode);
+}
+
+static int	zbx_audit_validate_entry(const zbx_audit_entry_t *entry)
+{
+	switch (entry->audit_action)
+	{
+		case ZBX_AUDIT_ACTION_ADD:
+		case ZBX_AUDIT_ACTION_UPDATE:
+			if (0 == strcmp(entry->details_json.buffer, "{}"))
+				return FAIL;
+			return SUCCEED;
+		default:
+			return SUCCEED;
+	}
+}
+
+void	zbx_audit_flush(int audit_context_mode)
 {
 	char			recsetid_cuid[CUID_LEN];
 	zbx_hashset_iter_t	iter;
 	zbx_audit_entry_t	**audit_entry;
 	zbx_db_insert_t		db_insert_audit;
 
-	RETURN_IF_AUDIT_OFF();
+	RETURN_IF_AUDIT_OFF(audit_context_mode);
 
 	zbx_new_cuid(recsetid_cuid);
 	zbx_hashset_iter_reset(&zbx_audit, &iter);
@@ -308,8 +333,7 @@ void	zbx_audit_flush(void)
 
 	while (NULL != (audit_entry = (zbx_audit_entry_t **)zbx_hashset_iter_next(&iter)))
 	{
-		if (AUDIT_ACTION_DELETE == (*audit_entry)->audit_action ||
-				0 != strcmp((*audit_entry)->details_json.buffer, "{}"))
+		if (SUCCEED == zbx_audit_validate_entry(*audit_entry))
 		{
 			zbx_db_insert_add_values(&db_insert_audit, (*audit_entry)->audit_cuid, AUDIT_USERID,
 					AUDIT_USERNAME, (int)time(NULL), (*audit_entry)->audit_action, AUDIT_IP,
@@ -322,17 +346,17 @@ void	zbx_audit_flush(void)
 	zbx_db_insert_execute(&db_insert_audit);
 	zbx_db_insert_clean(&db_insert_audit);
 
-	zbx_audit_clean();
+	zbx_audit_clean(audit_context_mode);
 }
 
-int	zbx_audit_flush_once(void)
+int	zbx_audit_flush_once(int audit_context_mode)
 {
 	char			recsetid_cuid[CUID_LEN];
 	int			ret = ZBX_DB_OK;
 	zbx_hashset_iter_t	iter;
 	zbx_audit_entry_t	**audit_entry;
 
-	if (ZBX_AUDITLOG_ENABLED != zbx_get_audit_mode())
+	if (ZBX_AUDITLOG_ENABLED != zbx_get_auditlog_enabled())
 		return ZBX_DB_OK;
 
 	zbx_new_cuid(recsetid_cuid);
@@ -340,14 +364,11 @@ int	zbx_audit_flush_once(void)
 
 	while (NULL != (audit_entry = (zbx_audit_entry_t **)zbx_hashset_iter_next(&iter)))
 	{
-		char	id[ZBX_MAX_UINT64_LEN + 1], *pvalue, *name_esc, *details_esc;
+		char		id[ZBX_MAX_UINT64_LEN + 1], *pvalue, *name_esc, *details_esc;
 		const char	*pfield;
 
-		if (AUDIT_ACTION_DELETE != (*audit_entry)->audit_action &&
-				0 == strcmp((*audit_entry)->details_json.buffer, "{}"))
-		{
+		if (SUCCEED != zbx_audit_validate_entry(*audit_entry))
 			continue;
-		}
 
 		if (0 != (*audit_entry)->id)
 		{
@@ -361,10 +382,10 @@ int	zbx_audit_flush_once(void)
 			pvalue = (*audit_entry)->cuid;
 		}
 
-		name_esc = DBdyn_escape_string((*audit_entry)->name);
-		details_esc = DBdyn_escape_string((*audit_entry)->details_json.buffer);
+		name_esc = zbx_db_dyn_escape_string((*audit_entry)->name);
+		details_esc = zbx_db_dyn_escape_string((*audit_entry)->details_json.buffer);
 
-		ret = DBexecute_once("insert into auditlog (auditid,userid,username,"
+		ret = zbx_db_execute_once("insert into auditlog (auditid,userid,username,"
 				"clock,action,ip,%s,resourcename,resourcetype,recordsetid,details) values"
 				" ('%s'," AUDIT_USERID_SQL ",'%s','%d','%d','%s','%s','%s',%d,'%s','%s')",
 				pfield, (*audit_entry)->audit_cuid, AUDIT_USERNAME, (int)time(NULL),
@@ -378,16 +399,16 @@ int	zbx_audit_flush_once(void)
 			break;
 	}
 
-	zbx_audit_clean();
+	zbx_audit_clean(audit_context_mode);
 
 	return ret;
 }
 
 static int	audit_field_default(const char *table_name, const char *field_name, const char *value, uint64_t id)
 {
-	static ZBX_THREAD_LOCAL char		cached_table_name[ZBX_TABLENAME_LEN_MAX];
-	static ZBX_THREAD_LOCAL const ZBX_TABLE	*table = NULL;
-	const ZBX_FIELD				*field;
+	static ZBX_THREAD_LOCAL char			cached_table_name[ZBX_TABLENAME_LEN_MAX];
+	static ZBX_THREAD_LOCAL const zbx_db_table_t	*table = NULL;
+	const zbx_db_field_t				*field;
 
 	if (NULL == table_name)
 		return FAIL;
@@ -398,7 +419,7 @@ static int	audit_field_default(const char *table_name, const char *field_name, c
 
 	if ('\0' == cached_table_name[0] || 0 != strcmp(cached_table_name, table_name))
 	{
-		if (NULL == (table = DBget_table(table_name)))
+		if (NULL == (table = zbx_db_get_table(table_name)))
 		{
 			zabbix_log(LOG_LEVEL_CRIT, "%s(): cannot find table '%s'", __func__, table_name);
 			THIS_SHOULD_NEVER_HAPPEN;
@@ -408,7 +429,7 @@ static int	audit_field_default(const char *table_name, const char *field_name, c
 		zbx_strlcpy(cached_table_name, table_name, sizeof(cached_table_name));
 	}
 
-	if (NULL == (field = DBget_field(table, field_name)))
+	if (NULL == (field = zbx_db_get_field(table, field_name)))
 	{
 		zabbix_log(LOG_LEVEL_CRIT, "%s(): table '%s', cannot find field '%s'", __func__, table_name,
 				field_name);
@@ -535,7 +556,7 @@ void	zbx_audit_update_json_append_uint64(const zbx_uint64_t id, const int id_tab
 void	zbx_audit_update_json_append_no_value(const zbx_uint64_t id, const int id_table, const char *audit_op,
 		const char *key)
 {
-	PREPARE_UPDATE_JSON_APPEND_OP();
+	PREPARE_UPDATE_JSON_APPEND_OP()
 	append_json_no_value(&((*found_audit_entry)->details_json), audit_op, key);
 }
 
@@ -552,7 +573,7 @@ void	zbx_audit_update_json_append_int(const zbx_uint64_t id, const int id_table,
 	}
 	else
 	{
-		PREPARE_UPDATE_JSON_APPEND_OP();
+		PREPARE_UPDATE_JSON_APPEND_OP()
 		append_int_json(&((*found_audit_entry)->details_json), audit_op, key, value);
 	}
 }
@@ -570,7 +591,7 @@ void	zbx_audit_update_json_append_double(const zbx_uint64_t id, const int id_tab
 	}
 	else
 	{
-		PREPARE_UPDATE_JSON_APPEND_OP();
+		PREPARE_UPDATE_JSON_APPEND_OP()
 		append_double_json(&((*found_audit_entry)->details_json), audit_op, key, value);
 	}
 }
@@ -578,34 +599,34 @@ void	zbx_audit_update_json_append_double(const zbx_uint64_t id, const int id_tab
 void	zbx_audit_update_json_update_string(const zbx_uint64_t id, const int id_table, const char *key,
 		const char *value_old, const char *value_new)
 {
-	PREPARE_UPDATE_JSON_APPEND_OP();
+	PREPARE_UPDATE_JSON_APPEND_OP()
 	update_str_json(&((*found_audit_entry)->details_json), key, value_old, value_new);
 }
 
 void	zbx_audit_update_json_update_uint64(const zbx_uint64_t id, const int id_table, const char *key,
 		uint64_t value_old, uint64_t value_new)
 {
-	PREPARE_UPDATE_JSON_APPEND_OP();
+	PREPARE_UPDATE_JSON_APPEND_OP()
 	update_uint64_json(&((*found_audit_entry)->details_json), key, value_old, value_new);
 }
 
 void	zbx_audit_update_json_update_int(const zbx_uint64_t id, const int id_table, const char *key, int value_old,
 		int value_new)
 {
-	PREPARE_UPDATE_JSON_APPEND_OP();
+	PREPARE_UPDATE_JSON_APPEND_OP()
 	update_int_json(&((*found_audit_entry)->details_json), key, value_old, value_new);
 }
 
 void	zbx_audit_update_json_update_double(const zbx_uint64_t id, const int id_table, const char *key,
 		double value_old, double value_new)
 {
-	PREPARE_UPDATE_JSON_APPEND_OP();
+	PREPARE_UPDATE_JSON_APPEND_OP()
 	update_double_json(&((*found_audit_entry)->details_json), key, value_old, value_new);
 }
 
 void	zbx_audit_update_json_delete(const zbx_uint64_t id, const int id_table, const char *audit_op, const char *key)
 {
-	PREPARE_UPDATE_JSON_APPEND_OP();
+	PREPARE_UPDATE_JSON_APPEND_OP()
 	delete_json(&((*found_audit_entry)->details_json), audit_op, key);
 }
 
@@ -636,10 +657,10 @@ void	zbx_audit_entry_append_int(zbx_audit_entry_t *entry, int audit_op, const ch
 
 	switch (audit_op)
 	{
-		case AUDIT_ACTION_ADD:
+		case ZBX_AUDIT_ACTION_ADD:
 			append_int_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, value1);
 			break;
-		case AUDIT_ACTION_UPDATE:
+		case ZBX_AUDIT_ACTION_UPDATE:
 			value2 = va_arg(args, int);
 			update_int_json(&entry->details_json, key, value1, value2);
 			break;
@@ -661,10 +682,10 @@ void	zbx_audit_entry_append_string(zbx_audit_entry_t *entry, int audit_op, const
 
 	switch (audit_op)
 	{
-		case AUDIT_ACTION_ADD:
+		case ZBX_AUDIT_ACTION_ADD:
 			append_str_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, value1);
 			break;
-		case AUDIT_ACTION_UPDATE:
+		case ZBX_AUDIT_ACTION_UPDATE:
 			value2 = va_arg(args, const char *);
 			update_str_json(&entry->details_json, key, value1, value2);
 			break;
@@ -675,3 +696,53 @@ void	zbx_audit_entry_append_string(zbx_audit_entry_t *entry, int audit_op, const
 
 	va_end(args);
 }
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: record history push request results into audit log                *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_auditlog_history_push(zbx_uint64_t userid, const char *username, const char *clientip, int processed_num,
+		int failed_num, double time_spent)
+{
+	int		ret = SUCCEED;
+	char		auditid_cuid[CUID_LEN];
+	struct zbx_json	details_json;
+	zbx_config_t	cfg;
+	zbx_db_insert_t	db_insert;
+
+	zabbix_log(LOG_LEVEL_TRACE, "In %s()", __func__);
+
+	zbx_config_get(&cfg, ZBX_CONFIG_FLAGS_AUDITLOG_ENABLED);
+
+	if (ZBX_AUDITLOG_ENABLED != cfg.auditlog_enabled)
+		goto out;
+
+	zbx_new_cuid(auditid_cuid);
+
+	zbx_json_init(&details_json, ZBX_JSON_STAT_BUF_LEN);
+
+	append_int_json(&details_json, AUDIT_DETAILS_ACTION_ADD, "history.processed", processed_num);
+	append_int_json(&details_json, AUDIT_DETAILS_ACTION_ADD, "history.failed", failed_num);
+	append_double_json(&details_json, AUDIT_DETAILS_ACTION_ADD, "history.time", time_spent);
+
+	zbx_db_insert_prepare(&db_insert, "auditlog", "auditid", "userid", "username", "clock", "action", "ip",
+			"resourcetype", "recordsetid", "details", NULL);
+
+	zbx_db_begin();
+
+	zbx_db_insert_add_values(&db_insert,  auditid_cuid, userid, username, (int)time(NULL), ZBX_AUDIT_ACTION_PUSH,
+			clientip, ZBX_AUDIT_RESOURCE_HISTORY, auditid_cuid,
+			details_json.buffer);
+
+	ret = zbx_db_insert_execute(&db_insert);
+
+	zbx_db_commit();
+
+	zbx_db_insert_clean(&db_insert);
+	zbx_json_free(&details_json);
+out:
+	zabbix_log(LOG_LEVEL_TRACE, "End of %s():%s", __func__, zbx_result_string(ret));
+
+	return ret;
+}
