@@ -1,31 +1,26 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the envied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
-#include "log.h"
 #include "zbxembed.h"
+#include "embed_xml.h"
+#include "embed.h"
 
 #include "httprequest.h"
 #include "zabbix.h"
 #include "global.h"
 #include "console.h"
-#include "xml.h"
-#include "embed.h"
+#include "zbxstr.h"
 
 #define ZBX_ES_MEMORY_LIMIT	(1024 * 1024 * 512)
 #define ZBX_ES_STACK_LIMIT	1000
@@ -278,7 +273,7 @@ void	zbx_es_destroy(zbx_es_t *es)
 {
 	char	*error = NULL;
 
-	if (SUCCEED != zbx_es_destroy_env(es, &error))
+	if (SUCCEED == zbx_es_is_env_initialized(es) && SUCCEED != zbx_es_destroy_env(es, &error))
 	{
 		zabbix_log(LOG_LEVEL_WARNING, "Cannot destroy embedded scripting engine environment: %s", error);
 	}
@@ -288,14 +283,15 @@ void	zbx_es_destroy(zbx_es_t *es)
  *                                                                            *
  * Purpose: initializes embedded scripting engine environment                 *
  *                                                                            *
- * Parameters: es    - [IN] the embedded scripting engine                     *
- *             error - [OUT] the error message                                *
+ * Parameters: es               - [IN] embedded scripting engine              *
+ *             config_source_ip - [IN]                                        *
+ *             error            - [OUT] error message                         *
  *                                                                            *
  * Return value: SUCCEED                                                      *
  *               FAIL                                                         *
  *                                                                            *
  ******************************************************************************/
-int	zbx_es_init_env(zbx_es_t *es, char **error)
+int	zbx_es_init_env(zbx_es_t *es, const char *config_source_ip, char **error)
 {
 	volatile int	ret = FAIL;
 
@@ -305,6 +301,8 @@ int	zbx_es_init_env(zbx_es_t *es, char **error)
 	memset(es->env, 0, sizeof(zbx_es_env_t));
 	es->env->max_total_alloc = 0;
 
+	es->env->config_source_ip = config_source_ip;
+
 	if (0 != setjmp(es->env->loc))
 	{
 		*error = zbx_strdup(*error, es->env->error);
@@ -338,7 +336,7 @@ int	zbx_es_init_env(zbx_es_t *es, char **error)
 	duk_def_prop(es->env->ctx, -3, DUK_DEFPROP_HAVE_VALUE | DUK_DEFPROP_CLEAR_WRITABLE | DUK_DEFPROP_HAVE_ENUMERABLE |
 			DUK_DEFPROP_HAVE_CONFIGURABLE);
 
-	/* initialize HttpRequest and CurlHttpRequest prototypes */
+	/* initialize HttpRequest prototype */
 	if (FAIL == zbx_es_init_httprequest(es, error))
 		goto out;
 
@@ -387,6 +385,7 @@ int	zbx_es_destroy_env(zbx_es_t *es, char **error)
 
 	duk_destroy_heap(es->env->ctx);
 	zbx_es_debug_disable(es);
+	zbx_free(es->env->browser_endpoint);
 	zbx_free(es->env->error);
 	zbx_free(es->env);
 
@@ -624,7 +623,17 @@ int	zbx_es_execute(zbx_es_t *es, const char *script, const char *code, int size,
 					*error = zbx_strdup(*error, "could not convert return value to utf8");
 				}
 				else
-					zabbix_log(LOG_LEVEL_DEBUG, "%s() output:'%s'", __func__, output);
+				{
+					if (SUCCEED == ZBX_CHECK_LOG_LEVEL(LOG_LEVEL_TRACE))
+					{
+						zabbix_log(LOG_LEVEL_DEBUG, "%s() output:'%s'", __func__, output);
+					}
+					else
+					{
+						zabbix_log(LOG_LEVEL_DEBUG, "%s() output:'%.*s'", __func__, 512,
+								output);
+					}
+				}
 
 				if (SUCCEED == ret && NULL != script_ret)
 					*script_ret = output;
@@ -655,8 +664,13 @@ out:
 		zbx_json_adduint64(es->env->json, "ms", zbx_get_duration_ms(&es->env->start_time));
 	}
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s %s allocated memory: " ZBX_FS_SIZE_T " max allocated or requested "
-			"memory: " ZBX_FS_SIZE_T " max allowed memory: %d", __func__, zbx_result_string(ret),
+	/* Duktape documentation recommends calling duk_gc() twice, see https://duktape.org/api#duk_gc */
+	duk_gc(es->env->ctx, 0);
+	duk_gc(es->env->ctx, 0);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s %s allocated memory: " ZBX_FS_SIZE_T
+			" max allocated or requested memory: " ZBX_FS_SIZE_T " max allowed memory: %d",
+			__func__, zbx_result_string(ret),
 			ZBX_NULL2EMPTY_STR(*error), (zbx_fs_size_t)es->env->total_alloc,
 			(zbx_fs_size_t)es->env->max_total_alloc, ZBX_ES_MEMORY_LIMIT);
 	es->env->max_total_alloc = 0;
@@ -707,20 +721,21 @@ void	zbx_es_debug_disable(zbx_es_t *es)
  *                                                                            *
  * Purpose: executes command (script in form of a text)                       *
  *                                                                            *
- * Parameters: command       - [IN] the command in form of a text             *
- *             param         - [IN] the script parameters                     *
- *             timeout       - [IN] the timeout for the execution (seconds)   *
- *             result        - [OUT] the result of an execution               *
- *             error         - [OUT] the error message                        *
- *             max_error_len - [IN] the maximum length of an error            *
- *             debug         - [OUT] the debug data (optional)                *
+ * Parameters: command          - [IN] command in form of a text              *
+ *             param            - [IN] script parameters                      *
+ *             timeout          - [IN] timeout for the execution (seconds)    *
+ *             config_source_ip - [IN]                                        *
+ *             result           - [OUT] result of an execution                *
+ *             error            - [OUT] error message                         *
+ *             max_error_len    - [IN] maximum length of an error             *
+ *             debug            - [OUT] debug data (optional)                 *
  *                                                                            *
  * Return value: SUCCEED                                                      *
  *               FAIL                                                         *
  *                                                                            *
  ******************************************************************************/
-int	zbx_es_execute_command(const char *command, const char *param, int timeout, char **result,
-		char *error, size_t max_error_len, char **debug)
+int	zbx_es_execute_command(const char *command, const char *param, int timeout, const char *config_source_ip,
+		char **result, char *error, size_t max_error_len, char **debug)
 {
 	int		size, ret = SUCCEED;
 	char		*code = NULL, *errmsg = NULL;
@@ -729,7 +744,7 @@ int	zbx_es_execute_command(const char *command, const char *param, int timeout,
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	zbx_es_init(&es);
-	if (FAIL == zbx_es_init_env(&es, &errmsg))
+	if (FAIL == zbx_es_init_env(&es, config_source_ip, &errmsg))
 	{
 		zbx_snprintf(error, max_error_len, "cannot initialize scripting environment: %s", errmsg);
 		zbx_free(errmsg);
@@ -786,7 +801,62 @@ zbx_es_env_t	*zbx_es_get_env(duk_context *ctx)
 		return NULL;
 
 	env = (zbx_es_env_t *)duk_to_pointer(ctx, -1);
-	duk_pop(ctx);
+	duk_pop_2(ctx);
 
 	return env;
 }
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: call base class prototype with arguments                          *
+ *                                                                            *
+ * Parameters: ctx  - [IN] duktape context                                    *
+ *             base - [IN] base class name                                    *
+ *             args - [IN] number of prototype arguments on stack             *
+ *                                                                            *
+ * Return value: 0                                                            *
+ *                                                                            *
+ ******************************************************************************/
+duk_ret_t	es_super(duk_context *ctx, const char *base, int args)
+{
+	zbx_es_env_t	*env;
+
+	if (NULL == (env = zbx_es_get_env(ctx)))
+		return duk_error(ctx, DUK_RET_TYPE_ERROR, "cannot access internal environment");
+
+	duk_get_global_string(ctx, base);
+	duk_push_this(ctx);
+
+	for (int i = 0; i < args; i++)
+		duk_dup(ctx, i);
+
+	env->constructor_chain = 1;
+	duk_call_method(ctx, args);
+
+	return 0;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: check if the method is called from constructor chain              *
+ *                                                                            *
+ * Parameters: ctx  - [IN] duktape context                                    *
+ *                                                                            *
+ * Return value: !0 - method is called from constructor chain                 *
+ *                0 - otherwise                                               *
+ *                                                                            *
+ ******************************************************************************/
+int	es_is_chained_constructor_call(duk_context *ctx)
+{
+	zbx_es_env_t	*env;
+
+	if (NULL == (env = zbx_es_get_env(ctx)))
+		return duk_error(ctx, DUK_RET_TYPE_ERROR, "cannot access internal environment");
+
+	int	constructor_chain = env->constructor_chain;
+
+	env->constructor_chain = 0;
+
+	return constructor_chain || duk_is_constructor_call(env->ctx);
+}
+
