@@ -1,45 +1,35 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the envied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 #include "httprequest.h"
+#include "embed.h"
 
-#include "common.h"
+#include "zbxstr.h"
 #include "zbxjson.h"
 #include "zbxhttp.h"
-#include "embed.h"
 #include "duktape.h"
 #include "zbxalgo.h"
+#include "zbxcurl.h"
 #include "global.h"
 
 #ifdef HAVE_LIBCURL
 
-#define ZBX_HTTPAUTH_NONE		CURLAUTH_NONE
-#define ZBX_HTTPAUTH_BASIC		CURLAUTH_BASIC
-#define ZBX_HTTPAUTH_DIGEST		CURLAUTH_DIGEST
-#if LIBCURL_VERSION_NUM >= 0x072600
-#	define ZBX_HTTPAUTH_NEGOTIATE	CURLAUTH_NEGOTIATE
-#else
-#	define ZBX_HTTPAUTH_NEGOTIATE	CURLAUTH_GSSNEGOTIATE
-#endif
-#define ZBX_HTTPAUTH_NTLM		CURLAUTH_NTLM
-
-extern char	*CONFIG_SOURCE_IP;
+#define ZBX_HTTPAUTH_NONE	CURLAUTH_NONE
+#define ZBX_HTTPAUTH_BASIC	CURLAUTH_BASIC
+#define ZBX_HTTPAUTH_DIGEST	CURLAUTH_DIGEST
+#define ZBX_HTTPAUTH_NEGOTIATE	CURLAUTH_NEGOTIATE
+#define ZBX_HTTPAUTH_NTLM	CURLAUTH_NTLM
 
 typedef struct
 {
@@ -60,13 +50,17 @@ zbx_es_httprequest_t;
 /* in case of error. Be careful with using ZBX_CURL_SETOPT(), duk_push_error_object() and duk_error()  */
 /* in functions - it is easy to get memory leaks because duk_error() causes longjmp().                 */
 /* Note that the caller of ZBX_CURL_SETOPT() must define variable 'int err_index' and label 'out'.     */
-#define ZBX_CURL_SETOPT(ctx, handle, opt, value, err)							\
-	if (CURLE_OK != (err = curl_easy_setopt(handle, opt, value)))					\
-	{												\
-		err_index = duk_push_error_object(ctx, DUK_RET_EVAL_ERROR,					\
-				"cannot set cURL option " #opt ": %s.", curl_easy_strerror(err));	\
-		goto out;										\
-	}
+#define ZBX_CURL_SETOPT(ctx, handle, opt, value, err)								\
+	do													\
+	{													\
+		if (CURLE_OK != (err = curl_easy_setopt(handle, opt, value)))					\
+		{												\
+			err_index = duk_push_error_object(ctx, DUK_RET_EVAL_ERROR,				\
+					"cannot set cURL option " #opt ": %s.", curl_easy_strerror(err));	\
+			goto out;										\
+		}												\
+	}													\
+	while(0)
 
 static size_t	curl_write_cb(void *ptr, size_t size, size_t nmemb, void *userdata)
 {
@@ -90,7 +84,7 @@ static size_t	curl_header_cb(void *ptr, size_t size, size_t nmemb, void *userdat
 
 /******************************************************************************
  *                                                                            *
- * Purpose: return backing C structure embedded in CurlHttpRequest object     *
+ * Purpose: return backing C structure embedded in HttpRequest object         *
  *                                                                            *
  ******************************************************************************/
 static zbx_es_httprequest_t *es_httprequest(duk_context *ctx)
@@ -100,14 +94,14 @@ static zbx_es_httprequest_t *es_httprequest(duk_context *ctx)
 	duk_push_this(ctx);
 	duk_get_prop_string(ctx, -1, "\xff""\xff""d");
 	request = (zbx_es_httprequest_t *)duk_to_pointer(ctx, -1);
-	duk_pop(ctx);
+	duk_pop_2(ctx);
 
 	return request;
 }
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest destructor                                        *
+ * Purpose: HttpRequest destructor                                            *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_dtor(duk_context *ctx)
@@ -132,9 +126,6 @@ static duk_ret_t	es_httprequest_dtor(duk_context *ctx)
 		zbx_free(request->data);
 		zbx_free(request->headers_in);
 		zbx_free(request);
-
-		duk_push_pointer(ctx, NULL);
-		duk_put_prop_string(ctx, 0, "\xff""\xff""d");
 	}
 
 	return 0;
@@ -142,7 +133,7 @@ static duk_ret_t	es_httprequest_dtor(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest constructor                                       *
+ * Purpose: HttpRequest constructor                                           *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_ctor(duk_context *ctx)
@@ -182,7 +173,7 @@ static duk_ret_t	es_httprequest_ctor(duk_context *ctx)
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_SSL_VERIFYHOST, 0L, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_HEADERFUNCTION, curl_header_cb, err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_HEADERDATA, request, err);
-	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_INTERFACE, CONFIG_SOURCE_IP, err);
+	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_INTERFACE, env->config_source_ip, err);
 
 	duk_push_string(ctx, "\xff""\xff""d");
 	duk_push_pointer(ctx, request);
@@ -209,7 +200,7 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest.SetHeader method                                  *
+ * Purpose: HttpRequest.SetHeader method                                      *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_add_header(duk_context *ctx)
@@ -256,7 +247,7 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest.ClearHeader method                                *
+ * Purpose: HttpRequest.ClearHeader method                                    *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_clear_header(duk_context *ctx)
@@ -276,7 +267,7 @@ static duk_ret_t	es_httprequest_clear_header(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest HTTP request implementation                       *
+ * Purpose: HttpRequest HTTP request implementation                           *
  *                                                                            *
  * Parameters: ctx          - [IN] the scripting engine context               *
  *             http_request - [IN] the HTTP request (GET, PUT, POST, DELETE)  *
@@ -285,7 +276,7 @@ static duk_ret_t	es_httprequest_clear_header(duk_context *ctx)
 static duk_ret_t	es_httprequest_query(duk_context *ctx, const char *http_request)
 {
 	zbx_es_httprequest_t	*request;
-	char			*url = NULL, *contents = NULL;
+	char			*url = NULL, *contents = NULL, *error = NULL;
 	CURLcode		err;
 	int			err_index = -1;
 	zbx_es_env_t		*env;
@@ -356,16 +347,13 @@ static duk_ret_t	es_httprequest_query(duk_context *ctx, const char *http_request
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_POSTFIELDS, ZBX_NULL2EMPTY_STR(contents), err);
 	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_POSTFIELDSIZE, (long)contents_len, err);
 
-	ZBX_CURL_SETOPT(ctx, request->handle, ZBX_CURLOPT_ACCEPT_ENCODING, "", err);
-#if LIBCURL_VERSION_NUM >= 0x071304
-	/* CURLOPT_PROTOCOLS is supported starting with version 7.19.4 (0x071304) */
-	/* CURLOPT_PROTOCOLS was deprecated in favor of CURLOPT_PROTOCOLS_STR starting with version 7.85.0 (0x075500) */
-#	if LIBCURL_VERSION_NUM >= 0x075500
-	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_PROTOCOLS_STR, "HTTP,HTTPS", err);
-#	else
-	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS, err);
-#	endif
-#endif
+	ZBX_CURL_SETOPT(ctx, request->handle, CURLOPT_ACCEPT_ENCODING, "", err);
+
+	if (SUCCEED != zbx_curl_setopt_https(request->handle, &error))
+	{
+		err_index = duk_push_error_object(ctx, DUK_RET_EVAL_ERROR, "%s", error);
+		goto out;
+	}
 
 	request->data_offset = 0;
 	request->headers_in_offset = 0;
@@ -382,6 +370,7 @@ static duk_ret_t	es_httprequest_query(duk_context *ctx, const char *http_request
 out:
 	zbx_free(url);
 	zbx_free(contents);
+	zbx_free(error);
 
 	if (-1 != err_index)
 		return duk_throw(ctx);
@@ -393,7 +382,7 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Purpose: HttpRequest.Get / CurlHttpRequest.Get method                      *
+ * Purpose: HttpRequest.Get method                                            *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_get(duk_context *ctx)
@@ -403,7 +392,7 @@ static duk_ret_t	es_httprequest_get(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: HttpRequest.Put / CurlHttpRequest.Put method                      *
+ * Purpose: HttpRequest.Put method                                            *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_put(duk_context *ctx)
@@ -413,7 +402,7 @@ static duk_ret_t	es_httprequest_put(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: HttpRequest.Post / CurlHttpRequest.Post method                    *
+ * Purpose: HttpRequest.Post method                                           *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_post(duk_context *ctx)
@@ -423,7 +412,7 @@ static duk_ret_t	es_httprequest_post(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: HttpRequest.Delete / CurlHttpRequest.Delete method                *
+ * Purpose: HttpRequest.Delete method                                         *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_delete(duk_context *ctx)
@@ -501,7 +490,7 @@ static duk_ret_t	es_httprequest_customrequest(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest.SetProxy method                                   *
+ * Purpose: HttpRequest.SetProxy method                                       *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_set_proxy(duk_context *ctx)
@@ -523,7 +512,7 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest.Status method                                     *
+ * Purpose: HttpRequest.Status method                                         *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_status(duk_context *ctx)
@@ -714,7 +703,7 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Purpose: HttpRequest.getHeaders / CurlHttpRequest.GetHeaders method        *
+ * Purpose: HttpRequest.getHeaders method                                     *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_get_headers(duk_context *ctx)
@@ -739,7 +728,7 @@ static duk_ret_t	es_httprequest_get_headers(duk_context *ctx)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: CurlHttpRequest.SetHttpAuth method                                *
+ * Purpose: HttpRequest.SetHttpAuth method                                    *
  *                                                                            *
  ******************************************************************************/
 static duk_ret_t	es_httprequest_set_httpauth(duk_context *ctx)
@@ -793,20 +782,6 @@ out:
 	return 0;
 }
 
-static const duk_function_list_entry	curlhttprequest_methods[] = {
-	{"AddHeader", es_httprequest_add_header, 1},
-	{"ClearHeader", es_httprequest_clear_header, 0},
-	{"Get", es_httprequest_get, 2},
-	{"Put", es_httprequest_put, 2},
-	{"Post", es_httprequest_post, 2},
-	{"Delete", es_httprequest_delete, 2},
-	{"Status", es_httprequest_status, 0},
-	{"SetProxy", es_httprequest_set_proxy, 1},
-	{"GetHeaders", es_httprequest_get_headers, 1},
-	{"SetHttpAuth", es_httprequest_set_httpauth, 3},
-	{NULL, NULL, 0}
-};
-
 static const duk_function_list_entry	httprequest_methods[] = {
 	{"addHeader", es_httprequest_add_header, 1},
 	{"clearHeader", es_httprequest_clear_header, 0},
@@ -840,8 +815,6 @@ static duk_ret_t	es_httprequest_ctor(duk_context *ctx)
 static const duk_function_list_entry	httprequest_methods[] = {
 	{NULL, NULL, 0}
 };
-
-static const duk_function_list_entry	*curlhttprequest_methods = httprequest_methods;
 #endif
 
 static int	es_httprequest_create_prototype(duk_context *ctx, const char *obj_name,
@@ -869,8 +842,7 @@ int	zbx_es_init_httprequest(zbx_es_t *es, char **error)
 		return FAIL;
 	}
 
-	if (FAIL == es_httprequest_create_prototype(es->env->ctx, "CurlHttpRequest", curlhttprequest_methods) ||
-			FAIL == es_httprequest_create_prototype(es->env->ctx, "HttpRequest", httprequest_methods))
+	if (FAIL == es_httprequest_create_prototype(es->env->ctx, "HttpRequest", httprequest_methods))
 	{
 		*error = zbx_strdup(*error, duk_safe_to_string(es->env->ctx, -1));
 		duk_pop(es->env->ctx);
