@@ -1,15 +1,29 @@
-#include "common.h"
+/*
+** Copyright (C) 2001-2024 Zabbix SIA
+**
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
+**
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
+**
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
+**/
+
+#include "zbxcommon.h"
 
 #ifdef HAVE_IPCSERVICE
 
 #ifdef HAVE_LIBEVENT
 #	include <event.h>
+#	include <event2/thread.h>
 #endif
 
-#include "zbxtypes.h"
-#include "zbxalgo.h"
-#include "log.h"
 #include "zbxipcservice.h"
+#include "zbxalgo.h"
+#include "zbxstr.h"
 
 #define ZBX_IPC_PATH_MAX	sizeof(((struct sockaddr_un *)0)->sun_path)
 
@@ -25,8 +39,6 @@ static size_t	ipc_path_root_len = 0;
 #define ZBX_IPC_ASYNC_SOCKET_STATE_TIMEOUT	1
 #define ZBX_IPC_ASYNC_SOCKET_STATE_ERROR	2
 
-extern unsigned char	program_type;
-
 /* IPC client, providing nonblocking connections through socket */
 struct zbx_ipc_client
 {
@@ -53,6 +65,8 @@ struct zbx_ipc_client
 	zbx_uint32_t		refcount;
 };
 
+ZBX_PTR_VECTOR_IMPL(ipc_client_ptr, zbx_ipc_client_t *)
+
 /*
  * Private API
  */
@@ -103,6 +117,9 @@ static const char	*ipc_get_path(void)
 #define ZBX_IPC_CLASS_PREFIX_PROXY	"proxy_"
 #define ZBX_IPC_CLASS_PREFIX_AGENT	"agent_"
 
+static const char	*ipc_path_prefix = ZBX_IPC_CLASS_PREFIX_NONE;
+static size_t		ipc_path_prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_NONE);
+
 /******************************************************************************
  *                                                                            *
  * Purpose: makes socket path from the service name                           *
@@ -116,51 +133,31 @@ static const char	*ipc_get_path(void)
  ******************************************************************************/
 static const char	*ipc_make_path(const char *service_name, char **error)
 {
-	const char	*prefix;
-	size_t		path_len, offset, prefix_len;
+	size_t				path_len, offset;
+	static ZBX_THREAD_LOCAL char	ipc_path_full[ZBX_IPC_PATH_MAX];
 
 	path_len = strlen(service_name);
 
-	switch (program_type)
-	{
-		case ZBX_PROGRAM_TYPE_SERVER:
-			prefix = ZBX_IPC_CLASS_PREFIX_SERVER;
-			prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_SERVER);
-			break;
-		case ZBX_PROGRAM_TYPE_PROXY_ACTIVE:
-		case ZBX_PROGRAM_TYPE_PROXY_PASSIVE:
-			prefix = ZBX_IPC_CLASS_PREFIX_PROXY;
-			prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_PROXY);
-			break;
-		case ZBX_PROGRAM_TYPE_AGENTD:
-			prefix = ZBX_IPC_CLASS_PREFIX_AGENT;
-			prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_AGENT);
-			break;
-		default:
-			prefix = ZBX_IPC_CLASS_PREFIX_NONE;
-			prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_NONE);
-			break;
-	}
-
 	if (ZBX_IPC_PATH_MAX < ipc_path_root_len + path_len + 1 + ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_PREFIX) +
-			ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_SUFFIX) + prefix_len)
+			ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_SUFFIX) + ipc_path_prefix_len)
 	{
 		*error = zbx_dsprintf(*error,
 				"Socket path \"%s%s%s%s%s\" exceeds maximum length of unix domain socket path.",
-				ipc_path, ZBX_IPC_SOCKET_PREFIX, prefix, service_name, ZBX_IPC_SOCKET_SUFFIX);
+				ipc_path, ZBX_IPC_SOCKET_PREFIX, ipc_path_prefix, service_name, ZBX_IPC_SOCKET_SUFFIX);
 		return NULL;
 	}
 
+	memcpy(ipc_path_full, ipc_path, ipc_path_root_len);
 	offset = ipc_path_root_len;
-	memcpy(ipc_path + offset , ZBX_IPC_SOCKET_PREFIX, ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_PREFIX));
+	memcpy(ipc_path_full + offset, ZBX_IPC_SOCKET_PREFIX, ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_PREFIX));
 	offset += ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_PREFIX);
-	memcpy(ipc_path + offset, prefix, prefix_len);
-	offset += prefix_len;
-	memcpy(ipc_path + offset, service_name, path_len);
+	memcpy(ipc_path_full + offset, ipc_path_prefix, ipc_path_prefix_len);
+	offset += ipc_path_prefix_len;
+	memcpy(ipc_path_full + offset, service_name, path_len);
 	offset += path_len;
-	memcpy(ipc_path + offset, ZBX_IPC_SOCKET_SUFFIX, ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_SUFFIX) + 1);
+	memcpy(ipc_path_full + offset, ZBX_IPC_SOCKET_SUFFIX, ZBX_CONST_STRLEN(ZBX_IPC_SOCKET_SUFFIX) + 1);
 
-	return ipc_path;
+	return ipc_path_full;
 }
 
 /******************************************************************************
@@ -775,7 +772,7 @@ static void	ipc_service_add_client(zbx_ipc_service_t *service, int fd)
 	client->tx_event = event_new(service->ev, fd, EV_WRITE | EV_PERSIST, ipc_client_write_event_cb, (void *)client);
 	event_add(client->rx_event, NULL);
 
-	zbx_vector_ptr_append(&service->clients, client);
+	zbx_vector_ipc_client_ptr_append(&service->clients, client);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() clientid:" ZBX_FS_UI64, __func__, client->id);
 }
@@ -790,12 +787,10 @@ static void	ipc_service_add_client(zbx_ipc_service_t *service, int fd)
  ******************************************************************************/
 static void	ipc_service_remove_client(zbx_ipc_service_t *service, zbx_ipc_client_t *client)
 {
-	int		i;
-
-	for (i = 0; i < service->clients.values_num; i++)
+	for (int i = 0; i < service->clients.values_num; i++)
 	{
 		if (service->clients.values[i] == client)
-			zbx_vector_ptr_remove_noorder(&service->clients, i);
+			zbx_vector_ipc_client_ptr_remove_noorder(&service->clients, i);
 	}
 }
 
@@ -811,12 +806,11 @@ static void	ipc_service_remove_client(zbx_ipc_service_t *service, zbx_ipc_client
  ******************************************************************************/
 zbx_ipc_client_t	*zbx_ipc_client_by_id(const zbx_ipc_service_t *service, zbx_uint64_t id)
 {
-	int			i;
 	zbx_ipc_client_t	*client;
 
-	for (i = 0; i < service->clients.values_num; i++)
+	for (int i = 0; i < service->clients.values_num; i++)
 	{
-		client = (zbx_ipc_client_t *) service->clients.values[i];
+		client = service->clients.values[i];
 
 		if (id == client->id)
 			return client;
@@ -1346,6 +1340,7 @@ void	zbx_ipc_message_format(const zbx_ipc_message_t *message, char **data)
 	(*data)[data_offset] = '\0';
 }
 
+#ifdef HAVE_OPENIPMI
 /******************************************************************************
  *                                                                            *
  * Purpose: copies ipc message                                                *
@@ -1361,6 +1356,14 @@ void	zbx_ipc_message_copy(zbx_ipc_message_t *dst, const zbx_ipc_message_t *src)
 	dst->data = (unsigned char *)zbx_malloc(NULL, src->size);
 	memcpy(dst->data, src->data, src->size);
 }
+#endif /* HAVE_OPENIPMI */
+
+static void	ipc_service_user_cb(evutil_socket_t fd, short what, void *arg)
+{
+	ZBX_UNUSED(fd);
+	ZBX_UNUSED(what);
+	ZBX_UNUSED(arg);
+}
 
 /*
  * Public service API
@@ -1424,6 +1427,12 @@ int	zbx_ipc_service_init_env(const char *path, char **error)
 
 	ipc_service_init_libevent();
 
+	if (0 != evthread_use_pthreads())
+	{
+		*error = zbx_strdup(*error, "Cannot initialize libevent threading support");
+		goto out;
+	}
+
 	ret = SUCCEED;
 out:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
@@ -1507,7 +1516,7 @@ int	zbx_ipc_service_start(zbx_ipc_service_t *service, const char *service_name,
 	}
 
 	service->path = zbx_strdup(NULL, service_name);
-	zbx_vector_ptr_create(&service->clients);
+	zbx_vector_ipc_client_ptr_create(&service->clients);
 	zbx_queue_ptr_create(&service->clients_recv);
 
 	service->ev = event_base_new();
@@ -1516,6 +1525,7 @@ int	zbx_ipc_service_start(zbx_ipc_service_t *service, const char *service_name,
 	event_add(service->ev_listener, NULL);
 
 	service->ev_timer = event_new(service->ev, -1, 0, ipc_service_timer_cb, service);
+	service->ev_alert = event_new(service->ev, -1, 0, ipc_service_user_cb, NULL);
 
 	ret = SUCCEED;
 out:
@@ -1535,20 +1545,20 @@ out:
  ******************************************************************************/
 void	zbx_ipc_service_close(zbx_ipc_service_t *service)
 {
-	int	i;
-
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() path:%s", __func__, service->path);
 
-	close(service->fd);
+	if (0 != close(service->fd))
+		zabbix_log(LOG_LEVEL_DEBUG, "Cannot close path \"%s\": %s", service->path, zbx_strerror(errno));
 
-	for (i = 0; i < service->clients.values_num; i++)
-		ipc_client_free((zbx_ipc_client_t *)service->clients.values[i]);
+	for (int i = 0; i < service->clients.values_num; i++)
+		ipc_client_free(service->clients.values[i]);
 
 	zbx_free(service->path);
 
-	zbx_vector_ptr_destroy(&service->clients);
+	zbx_vector_ipc_client_ptr_destroy(&service->clients);
 	zbx_queue_ptr_destroy(&service->clients_recv);
 
+	event_free(service->ev_alert);
 	event_free(service->ev_timer);
 	event_free(service->ev_listener);
 	event_base_free(service->ev);
@@ -1636,6 +1646,16 @@ int	zbx_ipc_service_recv(zbx_ipc_service_t *service, const zbx_timespec_t *timeo
 	return ret;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Purpose: interrupt IPC service recv loop from another thread               *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_ipc_service_alert(zbx_ipc_service_t *service)
+{
+	event_active(service->ev_alert, 0, 0);
+}
+
 /******************************************************************************
  *                                                                            *
  * Purpose: Sends IPC message to client                                       *
@@ -1707,6 +1727,11 @@ void	zbx_ipc_client_close(zbx_ipc_client_t *client)
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
+int	zbx_ipc_client_get_fd(zbx_ipc_client_t *client)
+{
+	return client->csocket.fd;
+}
+
 void	zbx_ipc_client_addref(zbx_ipc_client_t *client)
 {
 	client->refcount++;
@@ -2083,4 +2108,28 @@ out:
 	return ret;
 }
 
+void	zbx_init_library_ipcservice(unsigned char program_type)
+{
+	switch (program_type)
+	{
+		case ZBX_PROGRAM_TYPE_SERVER:
+			ipc_path_prefix = ZBX_IPC_CLASS_PREFIX_SERVER;
+			ipc_path_prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_SERVER);
+			break;
+		case ZBX_PROGRAM_TYPE_PROXY_ACTIVE:
+		case ZBX_PROGRAM_TYPE_PROXY_PASSIVE:
+			ipc_path_prefix = ZBX_IPC_CLASS_PREFIX_PROXY;
+			ipc_path_prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_PROXY);
+			break;
+		case ZBX_PROGRAM_TYPE_AGENTD:
+			ipc_path_prefix = ZBX_IPC_CLASS_PREFIX_AGENT;
+			ipc_path_prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_AGENT);
+			break;
+		default:
+			ipc_path_prefix = ZBX_IPC_CLASS_PREFIX_NONE;
+			ipc_path_prefix_len = ZBX_CONST_STRLEN(ZBX_IPC_CLASS_PREFIX_NONE);
+			break;
+	}
+}
+
 #endif
