@@ -1,40 +1,31 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 #include "zbxprometheus.h"
 
-#include "common.h"
-#include "zbxregexp.h"
-#include "log.h"
-#include "zbxjson.h"
+#include "zbxalgo.h"
 #include "zbxeval.h"
-
-/* Defines maximum row length to be written in error message in the case of parsing failure */
-#define ZBX_PROMEHTEUS_ERROR_MAX_ROW_LENGTH	50
+#include "zbxexpr.h"
+#include "zbxjson.h"
+#include "zbxnum.h"
+#include "zbxregexp.h"
+#include "zbxstr.h"
+#include "zbxtypes.h"
 
 #define ZBX_PROMETHEUS_HINT_HELP	0
 #define ZBX_PROMETHEUS_HINT_TYPE	1
 
-#define ZBX_PROMETHEUS_TYPE_UNTYPED	"untyped"
-
-#define ZBX_PROMETHEUS_ERROR_ROW_NUM	10
-
 typedef enum
 {
 	ZBX_PROMETHEUS_CONDITION_OP_EQUAL,
@@ -57,36 +48,20 @@ typedef struct
 }
 zbx_prometheus_condition_t;
 
+ZBX_PTR_VECTOR_DECL(prometheus_condition, zbx_prometheus_condition_t *)
+
 /* the prometheus pattern filter */
 typedef struct
 {
 	/* metric filter, optional - can be NULL */
-	zbx_prometheus_condition_t	*metric;
+	zbx_prometheus_condition_t		*metric;
 	/* value filter, optional - can be NULL */
-	zbx_prometheus_condition_t	*value;
+	zbx_prometheus_condition_t		*value;
 	/* label filters */
-	zbx_vector_ptr_t		labels;
+	zbx_vector_prometheus_condition_t	labels;
 }
 zbx_prometheus_filter_t;
 
-/* the prometheus label */
-typedef struct
-{
-	char	*name;
-	char	*value;
-}
-zbx_prometheus_label_t;
-
-/* the prometheus data row */
-typedef struct
-{
-	char			*metric;
-	char			*value;
-	zbx_vector_ptr_t	labels;
-	char			*raw;
-}
-zbx_prometheus_row_t;
-
 /* the prometheus metric HELP, TYPE hints in comments */
 typedef struct
 {
@@ -96,6 +71,15 @@ typedef struct
 }
 zbx_prometheus_hint_t;
 
+/* indexing support */
+
+typedef struct
+{
+	char				*value;
+	zbx_vector_prometheus_row_t	rows;
+}
+zbx_prometheus_index_t;
+
 /* TYPE, HELP hint hashset support */
 
 static zbx_hash_t	prometheus_hint_hash(const void *d)
@@ -113,21 +97,11 @@ static int	prometheus_hint_compare(const void *d1, const void *d2)
 	return strcmp(hint1->metric, hint2->metric);
 }
 
-/* indexing support */
+ZBX_PTR_VECTOR_IMPL(prometheus_label, zbx_prometheus_label_t *)
+ZBX_PTR_VECTOR_IMPL(prometheus_row, zbx_prometheus_row_t *)
+ZBX_PTR_VECTOR_IMPL(prometheus_label_index, zbx_prometheus_label_index_t *)
 
-typedef struct
-{
-	char			*value;
-	zbx_vector_ptr_t	rows;
-}
-zbx_prometheus_index_t;
-
-typedef struct
-{
-	char		*label;
-	zbx_hashset_t	index;
-}
-zbx_prometheus_label_index_t;
+ZBX_PTR_VECTOR_IMPL(prometheus_condition, zbx_prometheus_condition_t *)
 
 /******************************************************************************
  *                                                                            *
@@ -242,27 +216,6 @@ static char	*str_loc_unescape_hint_dyn(const char *src, const zbx_strloc_t *loc)
 	return str;
 }
 
-/******************************************************************************
- *                                                                            *
- * Purpose: compares substring at the specified location with the specified   *
- *          text                                                              *
- *                                                                            *
- * Parameters: src      - [IN] the source string                              *
- *             loc      - [IN] the substring location                         *
- *             text     - [IN] the text to compare with                       *
- *             text_len - [IN] the text length                                *
- *                                                                            *
- * Return value: -1 - the substring is less than the specified text           *
- *                0 - the substring is equal to the specified text            *
- *                1 - the substring is greater than the specified text        *
- *                                                                            *
- ******************************************************************************/
-static int	str_loc_cmp(const char *src, const zbx_strloc_t *loc, const char *text, size_t text_len)
-{
-	ZBX_RETURN_IF_NOT_EQUAL(loc->r - loc->l + 1, text_len);
-	return memcmp(src + loc->l, text, text_len);
-}
-
 /******************************************************************************
  *                                                                            *
  * Purpose: parses condition operation at the specified location              *
@@ -273,18 +226,18 @@ static int	str_loc_cmp(const char *src, const zbx_strloc_t *loc, const char *tex
  * Return value: The condition operation.                                     *
  *                                                                            *
  ******************************************************************************/
-static zbx_prometheus_condition_op_t	str_loc_op(const char *data, const zbx_strloc_t *loc)
+static zbx_prometheus_condition_op_t	str_loc_op(const char *src, const zbx_strloc_t *loc)
 {
-	if ('=' == data[loc->l])
+	if ('=' == src[loc->l])
 	{
-		if ('~' == data[loc->r])
+		if ('~' == src[loc->r])
 			return ZBX_PROMETHEUS_CONDITION_OP_REGEX;
 		else
 			return ZBX_PROMETHEUS_CONDITION_OP_EQUAL;
 	}
-	else if ('!' == data[loc->l])
+	else if ('!' == src[loc->l])
 	{
-		if ('~' == data[loc->r])
+		if ('~' == src[loc->r])
 			return ZBX_PROMETHEUS_CONDITION_OP_REGEX_NOT_MATCHED;
 		else
 			return ZBX_PROMETHEUS_CONDITION_OP_NOT_EQUAL;
@@ -321,14 +274,14 @@ static size_t	skip_spaces(const char *data, size_t pos)
  * Return value: The position of the next row space character.                *
  *                                                                            *
  ******************************************************************************/
-static size_t	skip_row(const char *data, size_t pos)
+static size_t	skip_row(const char *src, size_t pos)
 {
 	const char	*ptr;
 
-	if (NULL == (ptr = strchr(data + pos, '\n')))
-		return strlen(data + pos) + pos;
+	if (NULL == (ptr = strchr(src + pos, '\n')))
+		return strlen(src + pos) + pos;
 
-	return (size_t)(ptr - data + 1);
+	return (size_t)(ptr - src + 1);
 }
 
 /******************************************************************************
@@ -583,7 +536,7 @@ static void	prometheus_condition_free(zbx_prometheus_condition_t *condition)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: allocates and initializes conditionect                            *
+ * Purpose: allocates and initializes condition                               *
  *                                                                            *
  * Parameters: key     - [IN] the key to match                                *
  *             pattern - [IN] the matching pattern                            *
@@ -620,8 +573,8 @@ static void	prometheus_filter_clear(zbx_prometheus_filter_t *filter)
 	if (NULL != filter->value)
 		prometheus_condition_free(filter->value);
 
-	zbx_vector_ptr_clear_ext(&filter->labels, (zbx_clean_func_t)prometheus_condition_free);
-	zbx_vector_ptr_destroy(&filter->labels);
+	zbx_vector_prometheus_condition_clear_ext(&filter->labels, prometheus_condition_free);
+	zbx_vector_prometheus_condition_destroy(&filter->labels);
 }
 
 /******************************************************************************
@@ -687,7 +640,7 @@ static int	prometheus_filter_parse_labels(zbx_prometheus_filter_t *filter, const
 			return FAIL;
 		}
 
-		if (0 == str_loc_cmp(data, &loc_key, "__name__", ZBX_CONST_STRLEN("__name__")))
+		if (0 == zbx_strloc_cmp(data, &loc_key, "__name__", ZBX_CONST_STRLEN("__name__")))
 		{
 			if (NULL != filter->metric)
 			{
@@ -704,7 +657,7 @@ static int	prometheus_filter_parse_labels(zbx_prometheus_filter_t *filter, const
 
 			condition = prometheus_condition_create(str_loc_dup(data, &loc_key),
 					str_loc_unquote_dyn(data, &loc_value), str_loc_op(data, &loc_op));
-			zbx_vector_ptr_append(&filter->labels, condition);
+			zbx_vector_prometheus_condition_append(&filter->labels, condition);
 		}
 
 		pos = skip_spaces(data, loc_value.r + 1);
@@ -747,7 +700,7 @@ static int	prometheus_filter_init(zbx_prometheus_filter_t *filter, const char *d
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	memset(filter, 0, sizeof(zbx_prometheus_filter_t));
-	zbx_vector_ptr_create(&filter->labels);
+	zbx_vector_prometheus_condition_create(&filter->labels);
 
 	if (NULL == data)
 		return SUCCEED;
@@ -829,8 +782,8 @@ static void	prometheus_row_free(zbx_prometheus_row_t *row)
 	zbx_free(row->metric);
 	zbx_free(row->value);
 	zbx_free(row->raw);
-	zbx_vector_ptr_clear_ext(&row->labels, (zbx_clean_func_t)prometheus_label_free);
-	zbx_vector_ptr_destroy(&row->labels);
+	zbx_vector_prometheus_label_clear_ext(&row->labels, prometheus_label_free);
+	zbx_vector_prometheus_label_destroy(&row->labels);
 	zbx_free(row);
 }
 
@@ -896,7 +849,7 @@ static int	condition_match_metric_value(const char *pattern, const char *value)
 	double	pattern_dbl, value_dbl;
 	char	buffer[5];
 
-	if (SUCCEED != is_double(pattern, &pattern_dbl))
+	if (SUCCEED != zbx_is_double(pattern, &pattern_dbl))
 	{
 		if ('+' == *pattern)
 			pattern++;
@@ -909,10 +862,10 @@ static int	condition_match_metric_value(const char *pattern, const char *value)
 		return (0 == strcmp(pattern, buffer) ? SUCCEED : FAIL);
 	}
 
-	if (SUCCEED != is_double(value, &value_dbl))
+	if (SUCCEED != zbx_is_double(value, &value_dbl))
 		return FAIL;
 
-	if (ZBX_DOUBLE_EPSILON <= fabs(pattern_dbl - value_dbl))
+	if (zbx_get_double_epsilon() <= fabs(pattern_dbl - value_dbl))
 		return FAIL;
 
 	return SUCCEED;
@@ -932,7 +885,7 @@ static int	condition_match_metric_value(const char *pattern, const char *value)
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
-static int	prometheus_metric_parse_labels(const char *data, size_t pos, zbx_vector_ptr_t *labels,
+static int	prometheus_metric_parse_labels(const char *data, size_t pos, zbx_vector_prometheus_label_t *labels,
 		zbx_strloc_t *loc, char **error)
 {
 	zbx_strloc_t		loc_key, loc_value, loc_op;
@@ -961,7 +914,7 @@ static int	prometheus_metric_parse_labels(const char *data, size_t pos, zbx_vect
 		label = (zbx_prometheus_label_t *)zbx_malloc(NULL, sizeof(zbx_prometheus_label_t));
 		label->name = str_loc_dup(data, &loc_key);
 		label->value = str_loc_unquote_dyn(data, &loc_value);
-		zbx_vector_ptr_append(labels, label);
+		zbx_vector_prometheus_label_append(labels, label);
 
 		pos = skip_spaces(data, loc_value.r + 1);
 
@@ -1011,7 +964,7 @@ static int	prometheus_parse_row(zbx_prometheus_filter_t *filter, const char *dat
 
 	row = (zbx_prometheus_row_t *)zbx_malloc(NULL, sizeof(zbx_prometheus_row_t));
 	memset(row, 0, sizeof(zbx_prometheus_row_t));
-	zbx_vector_ptr_create(&row->labels);
+	zbx_vector_prometheus_label_create(&row->labels);
 
 	/* parse metric and check against the filter */
 
@@ -1226,9 +1179,9 @@ static int	prometheus_register_hint(zbx_hashset_t *hints, const char *data, char
 
 	if (NULL == (hint = (zbx_prometheus_hint_t *)zbx_hashset_search(hints, &hint_local)))
 	{
+		hint_local.type = NULL;
+		hint_local.help = NULL;
 		hint = zbx_hashset_insert(hints, &hint_local, sizeof(hint_local));
-		hint->type = NULL;
-		hint->help = NULL;
 	}
 	else
 		zbx_free(metric);
@@ -1266,6 +1219,7 @@ static int	prometheus_register_hint(zbx_hashset_t *hints, const char *data, char
  *             data       - [IN] the prometheus data                          *
  *             pos        - [IN] the position of comments in prometheus data  *
  *             hints      - [IN/OUT] the hint registry                        *
+ *             loc        - [OUT] the location of hint
  *             error      - [OUT] the error message                           *
  *                                                                            *
  * Return value: SUCCEED - the hint was registered successfully               *
@@ -1346,8 +1300,8 @@ static int	prometheus_parse_hint(zbx_prometheus_filter_t *filter, const char *da
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
-static int	prometheus_parse_rows(zbx_prometheus_filter_t *filter, const char *data, zbx_vector_ptr_t *rows,
-		zbx_hashset_t *hints, char **error)
+static int	prometheus_parse_rows(zbx_prometheus_filter_t *filter, const char *data,
+		zbx_vector_prometheus_row_t *rows, zbx_hashset_t *hints, char **error)
 {
 	size_t			pos = 0;
 	int			row_num = 1, ret = FAIL;
@@ -1382,7 +1336,7 @@ static int	prometheus_parse_rows(zbx_prometheus_filter_t *filter, const char *da
 		if (NULL != row)
 		{
 			row->raw = str_loc_dup(data, &loc);
-			zbx_vector_ptr_append(rows, row);
+			zbx_vector_prometheus_row_append(rows, row);
 		}
 
 		pos = loc.r + 1;
@@ -1400,6 +1354,9 @@ out:
 		else
 			len = strlen(data + pos);
 
+/* Defines maximum row length to be written in error message in the case of parsing failure */
+#define ZBX_PROMEHTEUS_ERROR_MAX_ROW_LENGTH	50
+
 		if (ZBX_PROMEHTEUS_ERROR_MAX_ROW_LENGTH < len)
 		{
 			len = ZBX_PROMEHTEUS_ERROR_MAX_ROW_LENGTH;
@@ -1408,6 +1365,8 @@ out:
 		*error = zbx_dsprintf(*error, "data parsing error at row %d \"%.*s%s\": %s", row_num, len, data + pos,
 				suffix, errmsg);
 		zbx_free(errmsg);
+
+#undef ZBX_PROMEHTEUS_ERROR_MAX_ROW_LENGTH
 	}
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s rows:%d hints:%d", __func__, zbx_result_string(ret),
@@ -1428,7 +1387,8 @@ out:
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
-static int prometheus_extract_value(const zbx_vector_ptr_t *rows, const char *output, char **value, char **error)
+static int prometheus_extract_value(const zbx_vector_prometheus_row_t *rows, const char *output, char **value,
+		char **error)
 {
 	const zbx_prometheus_row_t	*row;
 
@@ -1440,6 +1400,8 @@ static int prometheus_extract_value(const zbx_vector_ptr_t *rows, const char *ou
 
 	if (1 < rows->values_num)
 	{
+#define ZBX_PROMETHEUS_ERROR_ROW_NUM	10
+
 		int	i, rows_num = ZBX_PROMETHEUS_ERROR_ROW_NUM;
 		size_t	error_alloc, error_offset = 0;
 
@@ -1452,7 +1414,7 @@ static int prometheus_extract_value(const zbx_vector_ptr_t *rows, const char *ou
 
 		for (i = 0; i < rows_num; i++)
 		{
-			row = (zbx_prometheus_row_t *)rows->values[i];
+			row = rows->values[i];
 			zbx_strcpy_alloc(error, &error_alloc, &error_offset, row->raw);
 			zbx_chrcpy_alloc(error, &error_alloc, &error_offset, '\n');
 		}
@@ -1463,9 +1425,11 @@ static int prometheus_extract_value(const zbx_vector_ptr_t *rows, const char *ou
 			(*error)[error_offset - 1] = '\0';
 
 		return FAIL;
+
+#undef ZBX_PROMETHEUS_ERROR_ROW_NUM
 	}
 
-	row = (const zbx_prometheus_row_t *)rows->values[0];
+	row = rows->values[0];
 
 	if ('\0' != *output)
 	{
@@ -1507,7 +1471,8 @@ static int prometheus_extract_value(const zbx_vector_ptr_t *rows, const char *ou
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
-static int	prometheus_aggregate_values(const zbx_vector_ptr_t *rows, const char *function, char **value, char **error)
+static int	prometheus_aggregate_values(const zbx_vector_prometheus_row_t *rows, const char *function,
+		char **value, char **error)
 {
 	zbx_vector_dbl_t		values;
 	int				i, ret;
@@ -1518,7 +1483,7 @@ static int	prometheus_aggregate_values(const zbx_vector_ptr_t *rows, const char
 
 	for (i = 0; i < rows->values_num; i++)
 	{
-		row = (const zbx_prometheus_row_t *)rows->values[i];
+		row = rows->values[i];
 
 		value_dbl = atof(row->value);
 
@@ -1580,7 +1545,7 @@ static int	prometheus_aggregate_values(const zbx_vector_ptr_t *rows, const char
  *               FAIL    - otherwise                                          *
  *                                                                            *
  ******************************************************************************/
-static int	prometheus_query_rows(const zbx_vector_ptr_t *rows, const char *request, const char *output,
+static int	prometheus_query_rows(const zbx_vector_prometheus_row_t *rows, const char *request, const char *output,
 		char **value, char **error)
 {
 	if (0 == strcmp(request, "function"))
@@ -1598,8 +1563,8 @@ static int	prometheus_query_rows(const zbx_vector_ptr_t *rows, const char *reque
  *             rows_out - [OUT] the filtered rows                             *
  *                                                                            *
  ******************************************************************************/
-static void	prometheus_filter_rows(zbx_vector_ptr_t *rows, zbx_prometheus_filter_t *filter,
-		zbx_vector_ptr_t *rows_out)
+static void	prometheus_filter_rows(zbx_vector_prometheus_row_t *rows, zbx_prometheus_filter_t *filter,
+		zbx_vector_prometheus_row_t *rows_out)
 {
 	int			i, j, k;
 	zbx_prometheus_row_t	*row;
@@ -1608,7 +1573,7 @@ static void	prometheus_filter_rows(zbx_vector_ptr_t *rows, zbx_prometheus_filter
 
 	for (i = 0; i < rows->values_num; i++)
 	{
-		row = (zbx_prometheus_row_t *)rows->values[i];
+		row = rows->values[i];
 
 		if (NULL != filter->metric)
 		{
@@ -1644,12 +1609,21 @@ static void	prometheus_filter_rows(zbx_vector_ptr_t *rows, zbx_prometheus_filter
 				continue;
 		}
 
-		zbx_vector_ptr_append(rows_out, row);
+		zbx_vector_prometheus_row_append(rows_out, row);
 	}
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() rows:%d", __func__, rows_out->values_num);
 }
 
+static void	prometheus_hint_clear(void *d)
+{
+	zbx_prometheus_hint_t	*hint = (zbx_prometheus_hint_t *)d;
+
+	zbx_free(hint->metric);
+	zbx_free(hint->help);
+	zbx_free(hint->type);
+}
+
 /******************************************************************************
  *                                                                            *
  * Purpose: parse prometheus input and initialize cache                       *
@@ -1664,16 +1638,25 @@ static void	prometheus_filter_rows(zbx_vector_ptr_t *rows, zbx_prometheus_filter
  ******************************************************************************/
 int	zbx_prometheus_init(zbx_prometheus_t *prom, const char *data, char **error)
 {
-	zbx_prometheus_filter_t	filter;
+	zbx_prometheus_filter_t	filter = {0};
 	int			ret = FAIL;
 
-	zbx_vector_ptr_create(&prom->rows);
-	zbx_vector_ptr_create(&prom->indexes);
+	zbx_vector_prometheus_row_create(&prom->rows);
+	zbx_vector_prometheus_label_index_create(&prom->indexes);
+
+	zbx_hashset_create_ext(&prom->hints, 100, prometheus_hint_hash, prometheus_hint_compare, prometheus_hint_clear,
+			ZBX_DEFAULT_MEM_MALLOC_FUNC, ZBX_DEFAULT_MEM_REALLOC_FUNC, ZBX_DEFAULT_MEM_FREE_FUNC);
+
+	if (0 != pthread_mutex_init(&prom->index_lock, NULL))
+	{
+		*error = zbx_dsprintf(NULL, "Cannot initialize prometheus cache: %s", zbx_strerror(errno));
+		goto out;
+	}
 
 	if (SUCCEED != prometheus_filter_init(&filter, NULL, error))
-		return FAIL;
+		goto out;
 
-	if (FAIL == prometheus_parse_rows(&filter, data, &prom->rows, NULL, error))
+	if (FAIL == prometheus_parse_rows(&filter, data, &prom->rows, &prom->hints, error))
 		goto out;
 
 	ret = SUCCEED;
@@ -1695,7 +1678,7 @@ static void	prometheus_label_index_free(zbx_prometheus_label_index_t *label_inde
 
 	zbx_hashset_iter_reset(&label_index->index, &iter);
 	while (NULL != (index = (zbx_prometheus_index_t *)zbx_hashset_iter_next(&iter)))
-		zbx_vector_ptr_destroy(&index->rows);
+		zbx_vector_prometheus_row_destroy(&index->rows);
 
 	zbx_hashset_destroy(&label_index->index);
 	zbx_free(label_index);
@@ -1710,11 +1693,35 @@ static void	prometheus_label_index_free(zbx_prometheus_label_index_t *label_inde
  ******************************************************************************/
 void	zbx_prometheus_clear(zbx_prometheus_t *prom)
 {
-	zbx_vector_ptr_clear_ext(&prom->indexes, (zbx_clean_func_t)prometheus_label_index_free);
-	zbx_vector_ptr_destroy(&prom->indexes);
+	zbx_hashset_destroy(&prom->hints);
 
-	zbx_vector_ptr_clear_ext(&prom->rows, (zbx_clean_func_t)prometheus_row_free);
-	zbx_vector_ptr_destroy(&prom->rows);
+	zbx_vector_prometheus_label_index_clear_ext(&prom->indexes, prometheus_label_index_free);
+	zbx_vector_prometheus_label_index_destroy(&prom->indexes);
+
+	zbx_vector_prometheus_row_clear_ext(&prom->rows, prometheus_row_free);
+	zbx_vector_prometheus_row_destroy(&prom->rows);
+
+	pthread_mutex_destroy(&prom->index_lock);
+}
+
+static void	prometheus_lock(zbx_prometheus_t *prom)
+{
+	if (0 != pthread_mutex_lock(&prom->index_lock))
+	{
+		zabbix_log(LOG_LEVEL_CRIT, "Cannot lock prometheus cache: %s", zbx_strerror(errno));
+		THIS_SHOULD_NEVER_HAPPEN;
+		exit(EXIT_FAILURE);
+	}
+}
+
+static void	prometheus_unlock(zbx_prometheus_t *prom)
+{
+	if (0 != pthread_mutex_unlock(&prom->index_lock))
+	{
+		zabbix_log(LOG_LEVEL_CRIT, "Cannot unlock prometheus cache: %s", zbx_strerror(errno));
+		THIS_SHOULD_NEVER_HAPPEN;
+		exit(EXIT_FAILURE);
+	}
 }
 
 /******************************************************************************
@@ -1725,22 +1732,30 @@ void	zbx_prometheus_clear(zbx_prometheus_t *prom)
 
 static	zbx_prometheus_label_index_t	*prometheus_get_index(zbx_prometheus_t *prom, const char *label)
 {
-	int	i;
+	int				i;
+	zbx_prometheus_label_index_t	*label_index = NULL;
+
+	prometheus_lock(prom);
 
 	for (i = 0; i < prom->indexes.values_num; i++)
 	{
-		zbx_prometheus_label_index_t	*label_index = (zbx_prometheus_label_index_t *)prom->indexes.values[i];
-
-		if (0 == strcmp(label_index->label, label))
-			return label_index;
+		if (0 == strcmp(prom->indexes.values[i]->label, label))
+		{
+			label_index = prom->indexes.values[i];
+			break;
+		}
 	}
 
-	return NULL;
+	prometheus_unlock(prom);
+
+	return label_index;
 }
 
 static void	prometheus_add_index(zbx_prometheus_t *prom, zbx_prometheus_label_index_t *index)
 {
-	zbx_vector_ptr_append(&prom->indexes, index);
+	prometheus_lock(prom);
+	zbx_vector_prometheus_label_index_append(&prom->indexes, index);
+	prometheus_unlock(prom);
 }
 
 static zbx_hash_t	prometheus_index_hash_func(const void *d)
@@ -1790,7 +1805,7 @@ static zbx_prometheus_label_t	*prometheus_get_row_label(zbx_prometheus_row_t *ro
  *                                                                            *
  * Parameters: prom   - [IN] the prometheus cache                             *
  *             filter - [IN] the filter                                       *
- *             rows   - [IN] the rows matching filter label or NULL if there  *
+ *             rows   - [OUT] the rows matching filter label or NULL if there *
  *                           are now matching rows                            *
  *                                                                            *
  * Return value: SUCCEED - the matched rows were returned successfully        *
@@ -1803,7 +1818,7 @@ static zbx_prometheus_label_t	*prometheus_get_row_label(zbx_prometheus_row_t *ro
  *                                                                            *
  ******************************************************************************/
 static int	prometheus_get_indexed_rows_by_label(zbx_prometheus_t *prom, zbx_prometheus_filter_t *filter,
-		zbx_vector_ptr_t **rows)
+		zbx_vector_prometheus_row_t **rows)
 {
 	int				i;
 	zbx_prometheus_condition_t	*condition;
@@ -1827,11 +1842,10 @@ static int	prometheus_get_indexed_rows_by_label(zbx_prometheus_t *prom, zbx_prom
 
 		label_index->label = zbx_strdup(NULL, condition->key);
 		zbx_hashset_create(&label_index->index, 0, prometheus_index_hash_func, prometheus_index_compare_func);
-		prometheus_add_index(prom, label_index);
 
 		for (i = 0; i < prom->rows.values_num; i++)
 		{
-			zbx_prometheus_row_t	*row = (zbx_prometheus_row_t *)prom->rows.values[i];
+			zbx_prometheus_row_t	*row = prom->rows.values[i];
 			zbx_prometheus_label_t	*label;
 
 			if (NULL == (label = prometheus_get_row_label(row, label_index->label)))
@@ -1844,11 +1858,13 @@ static int	prometheus_get_indexed_rows_by_label(zbx_prometheus_t *prom, zbx_prom
 			{
 				index = (zbx_prometheus_index_t *)zbx_hashset_insert(&label_index->index, &index_local,
 						sizeof(index_local));
-				zbx_vector_ptr_create(&index->rows);
+				zbx_vector_prometheus_row_create(&index->rows);
 			}
 
-			zbx_vector_ptr_append(&index->rows, row);
+			zbx_vector_prometheus_row_append(&index->rows, row);
 		}
+
+		prometheus_add_index(prom, label_index);
 	}
 
 	index_local.value = condition->pattern;
@@ -1865,6 +1881,10 @@ static int	prometheus_get_indexed_rows_by_label(zbx_prometheus_t *prom, zbx_prom
  *                                                                            *
  * Purpose: validate prometheus pattern request and output                    *
  *                                                                            *
+ * Parameters: request - [IN] the prometheus request                          *
+ *             output  - [IN] the prometheus output                           *
+ *             error   - [OUT] the error message                              *
+ *                                                                            *
  * Return value: SUCCEED - valid request and output combination               *
  *               FAIL    - invalid request and output combination             *
  *                                                                            *
@@ -1894,7 +1914,7 @@ static int	prometheus_validate_request(const char *request, const char *output,
  *                                                                            *
  * Purpose: extract value from prometheus cache by the specified filter       *
  *                                                                            *
- * Parameters: data        - [IN] the prometheus cache                        *
+ * Parameters: prom        - [IN] the prometheus cache                        *
  *             filter_data - [IN] the filter in text format                   *
  *             request     - [IN] the data request - value, label, function   *
  *             output      - [IN] the output template/function name           *
@@ -1908,10 +1928,10 @@ static int	prometheus_validate_request(const char *request, const char *output,
 int	zbx_prometheus_pattern_ex(zbx_prometheus_t *prom, const char *filter_data, const char *request,
 		const char *output, char **value, char **error)
 {
-	zbx_prometheus_filter_t	filter;
-	int			ret = FAIL;
-	char			*errmsg = NULL;
-	zbx_vector_ptr_t	rows, *prows;
+	zbx_prometheus_filter_t		filter;
+	int				ret = FAIL;
+	char				*errmsg = NULL;
+	zbx_vector_prometheus_row_t	rows, *prows;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
@@ -1922,7 +1942,7 @@ int	zbx_prometheus_pattern_ex(zbx_prometheus_t *prom, const char *filter_data, c
 		goto out;
 	}
 
-	zbx_vector_ptr_create(&rows);
+	zbx_vector_prometheus_row_create(&rows);
 
 	if (SUCCEED != prometheus_validate_request(request, output, error))
 		return FAIL;
@@ -1939,7 +1959,7 @@ int	zbx_prometheus_pattern_ex(zbx_prometheus_t *prom, const char *filter_data, c
 	}
 
 	prometheus_filter_clear(&filter);
-	zbx_vector_ptr_destroy(&rows);
+	zbx_vector_prometheus_row_destroy(&rows);
 out:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 
@@ -1964,10 +1984,10 @@ out:
 int	zbx_prometheus_pattern(const char *data, const char *filter_data, const char *request, const char *output,
 		char **value, char **error)
 {
-	zbx_prometheus_filter_t	filter;
-	char			*errmsg = NULL;
-	int			ret = FAIL;
-	zbx_vector_ptr_t	rows;
+	zbx_prometheus_filter_t		filter;
+	char				*errmsg = NULL;
+	int				ret = FAIL;
+	zbx_vector_prometheus_row_t	rows;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
@@ -1978,7 +1998,7 @@ int	zbx_prometheus_pattern(const char *data, const char *filter_data, const char
 		goto out;
 	}
 
-	zbx_vector_ptr_create(&rows);
+	zbx_vector_prometheus_row_create(&rows);
 
 	if (SUCCEED != prometheus_validate_request(request, output, error))
 		return FAIL;
@@ -1996,14 +2016,121 @@ int	zbx_prometheus_pattern(const char *data, const char *filter_data, const char
 	zabbix_log(LOG_LEVEL_DEBUG, "%s(): output:%s", __func__, *value);
 	ret = SUCCEED;
 cleanup:
-	zbx_vector_ptr_clear_ext(&rows, (zbx_clean_func_t)prometheus_row_free);
-	zbx_vector_ptr_destroy(&rows);
+	zbx_vector_prometheus_row_clear_ext(&rows, prometheus_row_free);
+	zbx_vector_prometheus_row_destroy(&rows);
 	prometheus_filter_clear(&filter);
 out:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
 	return ret;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Purpose: converts filtered prometheus rows to json to be used with LLD     *
+ *                                                                            *
+ * Parameters: rows  - [IN] filtered prometheus rows                          *
+ *             hints - [IN] prometheus hints                                  *
+ *             value - [OUT] the converted data                               *
+ *                                                                            *
+ * Return value: SUCCEED - the data was converted successfully                *
+ *               FAIL    - otherwise                                          *
+ *                                                                            *
+ ******************************************************************************/
+static void	prometheus_to_json(zbx_vector_prometheus_row_t *rows, zbx_hashset_t *hints, char **value)
+{
+	int			i, j;
+	struct zbx_json		json;
+	zbx_prometheus_hint_t	*hint, hint_local;
+
+	zbx_json_initarray(&json, (size_t)rows->values_num * 100);
+
+	for (i = 0; i < rows->values_num; i++)
+	{
+		zbx_prometheus_row_t	*row =rows->values[i];
+		char			*hint_type;
+
+		zbx_json_addobject(&json, NULL);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_NAME, row->metric, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_VALUE, row->value, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_LINE_RAW, row->raw, ZBX_JSON_TYPE_STRING);
+
+		if (0 != row->labels.values_num)
+		{
+			zbx_json_addobject(&json, ZBX_PROTO_TAG_LABELS);
+
+			for (j = 0; j < row->labels.values_num; j++)
+			{
+				zbx_prometheus_label_t	*label = row->labels.values[j];
+
+				zbx_json_addstring(&json, label->name, label->value, ZBX_JSON_TYPE_STRING);
+			}
+
+			zbx_json_close(&json);
+		}
+
+		hint_local.metric = row->metric;
+		hint = (zbx_prometheus_hint_t *)zbx_hashset_search(hints, &hint_local);
+
+#define ZBX_PROMETHEUS_TYPE_UNTYPED	"untyped"
+
+		hint_type = (NULL != hint && NULL != hint->type ? hint->type : ZBX_PROMETHEUS_TYPE_UNTYPED);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_TYPE, hint_type, ZBX_JSON_TYPE_STRING);
+
+#undef ZBX_PROMETHEUS_TYPE_UNTYPED
+
+		if (NULL != hint && NULL != hint->help)
+			zbx_json_addstring(&json, ZBX_PROTO_TAG_HELP, hint->help, ZBX_JSON_TYPE_STRING);
+
+		zbx_json_close(&json);
+	}
+
+	*value = zbx_strdup(NULL, json.buffer);
+	zbx_json_free(&json);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: converts cached prometheus data to json to be used with LLD       *
+ *                                                                            *
+ * Parameters: prom        - [IN] the prometheus cache                        *
+ *             filter_data - [IN] the filter in text format                   *
+ *             value       - [OUT] the converted data                         *
+ *             error       - [OUT] the error message                          *
+ *                                                                            *
+ * Return value: SUCCEED - the data was converted successfully                *
+ *               FAIL    - otherwise                                          *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_prometheus_to_json_ex(zbx_prometheus_t *prom, const char *filter_data, char **value, char **error)
+{
+	zbx_vector_prometheus_row_t	rows;
+	zbx_prometheus_filter_t		filter;
+	char				*errmsg = NULL;
+	int				ret = FAIL;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
+
+	if (FAIL == prometheus_filter_init(&filter, filter_data, &errmsg))
+	{
+		*error = zbx_dsprintf(*error, "pattern error: %s", errmsg);
+		zbx_free(errmsg);
+		goto out;
+	}
+
+	zbx_vector_prometheus_row_create(&rows);
+
+	prometheus_filter_rows(&prom->rows, &filter, &rows);
+
+	prometheus_to_json(&rows, &prom->hints, value);
+	zbx_vector_prometheus_row_destroy(&rows);
+	prometheus_filter_clear(&filter);
+
+	ret = SUCCEED;
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
+	return ret;
+}
+
 /******************************************************************************
  *                                                                            *
  * Purpose: converts filtered prometheus data to json to be used with LLD     *
@@ -2019,14 +2146,11 @@ out:
  ******************************************************************************/
 int	zbx_prometheus_to_json(const char *data, const char *filter_data, char **value, char **error)
 {
-	zbx_prometheus_filter_t	filter;
-	char			*errmsg = NULL;
-	int			ret = FAIL, i, j;
-	zbx_vector_ptr_t	rows;
-	zbx_hashset_t		hints;
-	zbx_prometheus_hint_t	*hint, hint_local;
-	zbx_hashset_iter_t	iter;
-	struct zbx_json		json;
+	zbx_prometheus_filter_t		filter;
+	char				*errmsg = NULL;
+	int				ret = FAIL;
+	zbx_vector_prometheus_row_t	rows;
+	zbx_hashset_t			hints;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
@@ -2037,68 +2161,21 @@ int	zbx_prometheus_to_json(const char *data, const char *filter_data, char **val
 		goto out;
 	}
 
-	zbx_vector_ptr_create(&rows);
-	zbx_hashset_create(&hints, 100, prometheus_hint_hash, prometheus_hint_compare);
+	zbx_vector_prometheus_row_create(&rows);
+	zbx_hashset_create_ext(&hints, 100, prometheus_hint_hash, prometheus_hint_compare, prometheus_hint_clear,
+			ZBX_DEFAULT_MEM_MALLOC_FUNC, ZBX_DEFAULT_MEM_REALLOC_FUNC, ZBX_DEFAULT_MEM_FREE_FUNC);
 
-	if (FAIL == prometheus_parse_rows(&filter, data, &rows, &hints, error))
-		goto cleanup;
+	if (FAIL != (ret = prometheus_parse_rows(&filter, data, &rows, &hints, error)))
+		prometheus_to_json(&rows, &hints, value);
 
-	zbx_json_initarray(&json, (size_t)rows.values_num * 100);
-
-	for (i = 0; i < rows.values_num; i++)
-	{
-		zbx_prometheus_row_t	*row = (zbx_prometheus_row_t *)rows.values[i];
-		char			*hint_type;
-
-		zbx_json_addobject(&json, NULL);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_NAME, row->metric, ZBX_JSON_TYPE_STRING);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_VALUE, row->value, ZBX_JSON_TYPE_STRING);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_LINE_RAW, row->raw, ZBX_JSON_TYPE_STRING);
-
-		if (0 != row->labels.values_num)
-		{
-			zbx_json_addobject(&json, ZBX_PROTO_TAG_LABELS);
-
-			for (j = 0; j < row->labels.values_num; j++)
-			{
-				zbx_prometheus_label_t	*label = (zbx_prometheus_label_t *)row->labels.values[j];
-				zbx_json_addstring(&json, label->name, label->value, ZBX_JSON_TYPE_STRING);
-			}
-
-			zbx_json_close(&json);
-		}
-
-		hint_local.metric = row->metric;
-		hint = (zbx_prometheus_hint_t *)zbx_hashset_search(&hints, &hint_local);
-
-		hint_type = (NULL != hint && NULL != hint->type ? hint->type : ZBX_PROMETHEUS_TYPE_UNTYPED);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_TYPE, hint_type, ZBX_JSON_TYPE_STRING);
-
-		if (NULL != hint && NULL != hint->help)
-			zbx_json_addstring(&json, ZBX_PROTO_TAG_HELP, hint->help, ZBX_JSON_TYPE_STRING);
-
-		zbx_json_close(&json);
-	}
-
-	*value = zbx_strdup(NULL, json.buffer);
-	zbx_json_free(&json);
-	zabbix_log(LOG_LEVEL_DEBUG, "%s(): output:%s", __func__, *value);
-	ret = SUCCEED;
-cleanup:
-	zbx_hashset_iter_reset(&hints, &iter);
-	while (NULL != (hint = (zbx_prometheus_hint_t *)zbx_hashset_iter_next(&iter)))
-	{
-		zbx_free(hint->metric);
-		zbx_free(hint->help);
-		zbx_free(hint->type);
-	}
 	zbx_hashset_destroy(&hints);
 
-	zbx_vector_ptr_clear_ext(&rows, (zbx_clean_func_t)prometheus_row_free);
-	zbx_vector_ptr_destroy(&rows);
+	zbx_vector_prometheus_row_clear_ext(&rows, prometheus_row_free);
+	zbx_vector_prometheus_row_destroy(&rows);
 	prometheus_filter_clear(&filter);
 out:
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s value:%s", __func__, zbx_result_string(ret),
+			ZBX_NULL2EMPTY_STR(*value));
 	return ret;
 }
 
