@@ -1,30 +1,27 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 #include "http.h"
 
-#include "common.h"
+#include "../sysinfo.h"
+#include "zbxstr.h"
+#include "zbxnum.h"
+#include "zbxtime.h"
 #include "zbxregexp.h"
 #include "zbxhttp.h"
-#include "comms.h"
-
-extern int	CONFIG_TIMEOUT;
+#include "zbxcomms.h"
+#include "zbxcurl.h"
 
 #define HTTP_SCHEME_STR		"http://"
 
@@ -118,7 +115,7 @@ static int	check_common_params(const char *host, const char *path, char **error)
 }
 
 #ifdef HAVE_LIBCURL
-static int	curl_page_get(char *url, char **buffer, char **error)
+static int	curl_page_get(char *url, int timeout, char **buffer, char **error)
 {
 	CURLcode		err;
 	CURL			*easyhandle;
@@ -137,10 +134,12 @@ static int	curl_page_get(char *url, char **buffer, char **error)
 			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_SSL_VERIFYHOST, 0L)) ||
 			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_FOLLOWLOCATION, 0L)) ||
 			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_URL, url)) ||
-			(NULL != CONFIG_SOURCE_IP &&
-			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_INTERFACE, CONFIG_SOURCE_IP))) ||
-			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_TIMEOUT, (long)CONFIG_TIMEOUT)) ||
-			CURLE_OK != (err = curl_easy_setopt(easyhandle, ZBX_CURLOPT_ACCEPT_ENCODING, "")))
+			(NULL != sysinfo_get_config_source_ip() &&
+			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_INTERFACE,
+					sysinfo_get_config_source_ip()))) ||
+			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_TIMEOUT,
+					(long)timeout)) ||
+			CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_ACCEPT_ENCODING, "")))
 	{
 		*error = zbx_dsprintf(*error, "Cannot set cURL option: %s.", curl_easy_strerror(err));
 		goto out;
@@ -163,19 +162,9 @@ static int	curl_page_get(char *url, char **buffer, char **error)
 		}
 	}
 
-#if LIBCURL_VERSION_NUM >= 0x071304
-	/* CURLOPT_PROTOCOLS is supported starting with version 7.19.4 (0x071304) */
-	/* CURLOPT_PROTOCOLS was deprecated in favor of CURLOPT_PROTOCOLS_STR starting with version 7.85.0 (0x075500) */
-#	if LIBCURL_VERSION_NUM >= 0x075500
-	if (CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_PROTOCOLS_STR, "HTTP,HTTPS")))
-#	else
-	if (CURLE_OK != (err = curl_easy_setopt(easyhandle, CURLOPT_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS)))
-#	endif
-	{
-		*error = zbx_dsprintf(*error, "Cannot set allowed protocols: %s", curl_easy_strerror(err));
+	if (SUCCEED != zbx_curl_setopt_https(easyhandle, error))
 		goto out;
-	}
-#endif
+
 	*errbuf = '\0';
 	if (CURLE_OK == (err = curl_easy_perform(easyhandle)))
 	{
@@ -205,7 +194,8 @@ out:
 	return ret;
 }
 
-static int	get_http_page(const char *host, const char *path, const char *port, char **buffer, char **error)
+static int	get_http_page(const char *host, const char *path, const char *port, int timeout, char **buffer,
+		char **error)
 {
 	char	*url = NULL;
 	int	ret;
@@ -227,7 +217,7 @@ static int	get_http_page(const char *host, const char *path, const char *port, c
 
 		if (NULL != port && '\0' != *port)
 		{
-			if (SUCCEED != is_ushort(port, &port_n))
+			if (SUCCEED != zbx_is_ushort(port, &port_n))
 			{
 				*error = zbx_strdup(*error, "Invalid third parameter.");
 				return SYSINFO_RET_FAIL;
@@ -250,7 +240,7 @@ static int	get_http_page(const char *host, const char *path, const char *port, c
 		goto out;
 	}
 
-	ret = curl_page_get(url, buffer, error);
+	ret = curl_page_get(url, timeout, buffer, error);
 out:
 	zbx_free(url);
 
@@ -277,7 +267,8 @@ static char	*find_port_sep(char *host, size_t len)
 	return NULL;
 }
 
-static int	get_http_page(const char *host, const char *path, const char *port, char **buffer, char **error)
+static int	get_http_page(const char *host, const char *path, const char *port, int timeout, char **buffer,
+		char **error)
 {
 	char		*url = NULL, *hostname = NULL, *path_loc = NULL;
 	int		ret = SYSINFO_RET_OK, ipv6_host_found = 0;
@@ -324,7 +315,7 @@ static int	get_http_page(const char *host, const char *path, const char *port, c
 			{
 				port_str = zbx_dsprintf(NULL, "%.*s", port_len, p + 1);
 
-				if (SUCCEED != is_ushort(port_str, &port_num))
+				if (SUCCEED != zbx_is_ushort(port_str, &port_num))
 					ret = SYSINFO_RET_FAIL;
 				else
 					hostname = zbx_dsprintf(hostname, "%.*s", (int)(p - p_host), p_host);
@@ -370,7 +361,7 @@ static int	get_http_page(const char *host, const char *path, const char *port, c
 		{
 			port_num = ZBX_DEFAULT_HTTP_PORT;
 		}
-		else if (FAIL == is_ushort(port, &port_num))
+		else if (FAIL == zbx_is_ushort(port, &port_num))
 		{
 			*error = zbx_strdup(*error, "Invalid third parameter.");
 			ret = SYSINFO_RET_FAIL;
@@ -391,7 +382,7 @@ static int	get_http_page(const char *host, const char *path, const char *port, c
 		goto out;
 	}
 
-	if (SUCCEED == (ret = zbx_tcp_connect(&s, CONFIG_SOURCE_IP, hostname, port_num, CONFIG_TIMEOUT,
+	if (SUCCEED == (ret = zbx_tcp_connect(&s, sysinfo_get_config_source_ip(), hostname, port_num, timeout,
 			ZBX_TCP_SEC_UNENCRYPTED, NULL, NULL)))
 	{
 		char	*request = NULL;
@@ -437,7 +428,7 @@ out:
 }
 #endif
 
-int	WEB_PAGE_GET(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	web_page_get(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char	*hostname, *path_str, *port_str, *buffer = NULL, *error = NULL;
 	int	ret;
@@ -452,7 +443,7 @@ int	WEB_PAGE_GET(AGENT_REQUEST *request, AGENT_RESULT *result)
 	path_str = get_rparam(request, 1);
 	port_str = get_rparam(request, 2);
 
-	if (SYSINFO_RET_OK == (ret = get_http_page(hostname, path_str, port_str, &buffer, &error)))
+	if (SYSINFO_RET_OK == (ret = get_http_page(hostname, path_str, port_str, request->timeout, &buffer, &error)))
 	{
 		zbx_rtrim(buffer, "\r\n");
 		SET_TEXT_RESULT(result, buffer);
@@ -463,7 +454,7 @@ int	WEB_PAGE_GET(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return ret;
 }
 
-int	WEB_PAGE_PERF(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	web_page_perf(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char	*hostname, *path_str, *port_str, *error = NULL;
 	double	start_time;
@@ -481,7 +472,7 @@ int	WEB_PAGE_PERF(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	start_time = zbx_time();
 
-	if (SYSINFO_RET_OK == (ret = get_http_page(hostname, path_str, port_str, NULL, &error)))
+	if (SYSINFO_RET_OK == (ret = get_http_page(hostname, path_str, port_str, request->timeout, NULL, &error)))
 		SET_DBL_RESULT(result, zbx_time() - start_time);
 	else
 		SET_MSG_RESULT(result, error);
@@ -489,7 +480,7 @@ int	WEB_PAGE_PERF(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return ret;
 }
 
-int	WEB_PAGE_REGEXP(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	web_page_regexp(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char		*hostname, *path_str, *port_str, *buffer = NULL, *error = NULL, *regexp, *length_str;
 	const char	*output;
@@ -516,7 +507,7 @@ int	WEB_PAGE_REGEXP(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	if (NULL == length_str || '\0' == *length_str)
 		length = ZBX_MAX_UINT31_1;
-	else if (FAIL == is_uint31_1(length_str, &length))
+	else if (FAIL == zbx_is_uint31_1(length_str, &length))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid fifth parameter."));
 		return SYSINFO_RET_FAIL;
@@ -526,7 +517,7 @@ int	WEB_PAGE_REGEXP(AGENT_REQUEST *request, AGENT_RESULT *result)
 	if (NULL == output || '\0' == *output)
 		output = "\\0";
 
-	if (SYSINFO_RET_OK == (ret = get_http_page(hostname, path_str, port_str, &buffer, &error)))
+	if (SYSINFO_RET_OK == (ret = get_http_page(hostname, path_str, port_str, request->timeout, &buffer, &error)))
 	{
 		char	*ptr = NULL, *str;
 
