@@ -1,27 +1,23 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
-#include "common.h"
-#include "sysinfo.h"
+#include "../sysinfo.h"
+
 #include "zbxjson.h"
-#include "log.h"
-#include "comms.h"
+#include "zbxcomms.h"
+#include "zbxnum.h"
+#include "zbxip.h"
 
 typedef struct
 {
@@ -53,9 +49,6 @@ typedef struct
 }
 net_count_info_t;
 
-#define IPV4_MAX_CIDR_PREFIX	32	/* max number of bits in IPv4 CIDR prefix */
-#define IPV6_MAX_CIDR_PREFIX	128	/* max number of bits in IPv6 CIDR prefix */
-
 #define NET_CONN_TYPE_TCP	0
 #define NET_CONN_TYPE_UDP	1
 
@@ -105,7 +98,7 @@ static int	find_tcp_port_by_state_nl(unsigned short port, int state, int *found)
 	}
 	request;
 
-	int			ret = FAIL, fd, status, i;
+	int			ret = FAIL, fd, status;
 	int			families[] = {AF_INET, AF_INET6, AF_UNSPEC};
 	unsigned int		sequence = 0x58425A;
 	struct timeval		timeout = { 1, 500 * 1000 };
@@ -142,7 +135,7 @@ static int	find_tcp_port_by_state_nl(unsigned short port, int state, int *found)
 
 	nlerr = NLERR_OK;
 
-	for (i = 0; AF_UNSPEC != families[i]; i++)
+	for (int i = 0; AF_UNSPEC != families[i]; i++)
 	{
 		request.r.idiag_family = families[i];
 
@@ -294,12 +287,12 @@ static int	get_net_stat(const char *if_name, net_stat_t *result, char **error)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: reads /proc/net/tcp(6) file by chunks until the last line in      *
- *          in buffer has non-listening socket state                          *
+ * Purpose: Reads /proc/net/tcp(6) file by chunks until the last line in      *
+ *          in buffer has non-listening socket state.                         *
  *                                                                            *
- * Parameters: filename     - [IN] the file to read                           *
- *             buffer       - [IN/OUT] the output buffer                      *
- *             buffer_alloc - [IN/OUT] the output buffer size                 *
+ * Parameters: filename     - [IN] file to read                               *
+ *             buffer       - [IN/OUT] output buffer                          *
+ *             buffer_alloc - [IN/OUT] output buffer size                     *
  *                                                                            *
  * Return value: -1 error occurred during reading                             *
  *                0 empty file (shouldn't happen)                             *
@@ -377,11 +370,11 @@ out:
 
 /******************************************************************************
  *                                                                            *
- * Purpose: reads whole file into a buffer in a single read operation         *
+ * Purpose: reads whole file into buffer in single read operation             *
  *                                                                            *
- * Parameters: filename     - [IN] the file to read                           *
- *             buffer       - [IN/OUT] the output buffer                      *
- *             buffer_alloc - [IN/OUT] the output buffer size                 *
+ * Parameters: filename     - [IN] file to read                               *
+ *             buffer       - [IN/OUT] output buffer                          *
+ *             buffer_alloc - [IN/OUT] output buffer size                     *
  *                                                                            *
  * Return value: -1 error occurred during reading                             *
  *                0 empty file (shouldn't happen)                             *
@@ -416,7 +409,7 @@ out:
 	return ret;
 }
 
-int	NET_IF_IN(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_if_in(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	net_stat_t	ns;
 	char		*if_name, *mode, *error;
@@ -461,7 +454,7 @@ int	NET_IF_IN(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-int	NET_IF_OUT(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_if_out(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	net_stat_t	ns;
 	char		*if_name, *mode, *error;
@@ -506,7 +499,7 @@ int	NET_IF_OUT(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-int	NET_IF_TOTAL(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_if_total(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	net_stat_t	ns;
 	char		*if_name, *mode, *error;
@@ -547,7 +540,7 @@ int	NET_IF_TOTAL(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-int	NET_IF_COLLISIONS(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_if_collisions(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	net_stat_t	ns;
 	char		*if_name, *error;
@@ -571,7 +564,7 @@ int	NET_IF_COLLISIONS(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-int	NET_IF_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_if_discovery(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char		line[MAX_STRING_LEN], *p;
 	FILE		*f;
@@ -614,7 +607,7 @@ int	NET_IF_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-int	NET_TCP_LISTEN(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_tcp_listen(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char		pattern[64], *port_str, *buffer = NULL;
 	unsigned short	port;
@@ -631,7 +624,7 @@ int	NET_TCP_LISTEN(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	port_str = get_rparam(request, 0);
 
-	if (NULL == port_str || SUCCEED != is_ushort(port_str, &port))
+	if (NULL == port_str || SUCCEED != zbx_is_ushort(port_str, &port))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid first parameter."));
 		return SYSINFO_RET_FAIL;
@@ -715,7 +708,7 @@ out:
 	return ret;
 }
 
-int	NET_UDP_LISTEN(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_udp_listen(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char		pattern[64], *port_str, *buffer = NULL;
 	unsigned short	port;
@@ -730,7 +723,7 @@ int	NET_UDP_LISTEN(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	port_str = get_rparam(request, 0);
 
-	if (NULL == port_str || SUCCEED != is_ushort(port_str, &port))
+	if (NULL == port_str || SUCCEED != zbx_is_ushort(port_str, &port))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid first parameter."));
 		return SYSINFO_RET_FAIL;
@@ -822,11 +815,9 @@ static unsigned char	get_connection_state_udp(const char *name)
 #ifdef HAVE_IPV6
 static int	scan_ipv6_addr(const char *addr, struct sockaddr_in6 *sa6)
 {
-	int	i, k;
-
-	for (i = 0; i < 16; i += 4)
+	for (int i = 0; i < 16; i += 4)
 	{
-		for (k = 0; k < 4; k++)
+		for (int k = 0; k < 4; k++)
 		{
 			if (1 != sscanf(addr + i * 2 + k * 2, "%2hhx", &sa6->sin6_addr.s6_addr[i + 3 - k]))
 				return FAIL;
@@ -893,10 +884,10 @@ static int	get_proc_net_count_ipv6(const char *filename, unsigned char state, ne
 				(0 != exp_r->port && exp_r->port != rport) ||
 				(0 != state && state != state_f) ||
 				(NULL != exp_l->ai &&
-				FAIL == zbx_ip_cmp(exp_l->prefix_sz, exp_l->ai, sockaddr_l,
+				FAIL == zbx_ip_cmp(exp_l->prefix_sz, exp_l->ai, &sockaddr_l,
 				1 == exp_l->mapped && 0 != exp_l->prefix_sz ? 0 : 1)) ||
 				(NULL != exp_r->ai &&
-				FAIL == zbx_ip_cmp(exp_r->prefix_sz, exp_r->ai, sockaddr_r,
+				FAIL == zbx_ip_cmp(exp_r->prefix_sz, exp_r->ai, &sockaddr_r,
 				1 == exp_r->mapped && 0 != exp_r->prefix_sz ? 0 : 1)))
 		{
 			continue;
@@ -953,10 +944,10 @@ static int	get_proc_net_count_ipv4(const char *filename, unsigned char state, ne
 				(0 != exp_r->port && exp_r->port != rport) ||
 				(0 != state && state != state_f) ||
 				(NULL != exp_l->ai &&
-				FAIL == zbx_ip_cmp(exp_l->prefix_sz, exp_l->ai, sockaddr_l,
+				FAIL == zbx_ip_cmp(exp_l->prefix_sz, exp_l->ai, &sockaddr_l,
 				1 == exp_l->mapped && 0 != exp_l->prefix_sz ? 0 : 1)) ||
 				(NULL != exp_r->ai &&
-				FAIL == zbx_ip_cmp(exp_r->prefix_sz, exp_r->ai, sockaddr_r,
+				FAIL == zbx_ip_cmp(exp_r->prefix_sz, exp_r->ai, &sockaddr_r,
 				1 == exp_r->mapped && 0 != exp_r->prefix_sz ? 0 : 1)))
 		{
 			continue;
@@ -992,7 +983,7 @@ static int	get_addr_info(const char *addr_in, const char *port_in, struct addrin
 				goto err;
 			}
 		}
-		else if (FAIL == is_supported_ip(addr))
+		else if (FAIL == zbx_is_supported_ip(addr))
 		{
 			*error = zbx_dsprintf(*error, "IP is not supported: \"%s\"", addr_in);
 			goto err;
@@ -1003,7 +994,7 @@ static int	get_addr_info(const char *addr_in, const char *port_in, struct addrin
 
 	if (NULL != port_in && '\0' != *port_in)
 	{
-		if (SUCCEED != is_ushort(port_in, &info->port))
+		if (SUCCEED != zbx_is_ushort(port_in, &info->port))
 		{
 			if (0 != atoi(port_in))
 			{
@@ -1029,7 +1020,6 @@ static int	get_addr_info(const char *addr_in, const char *port_in, struct addrin
 		*error = zbx_dsprintf(*error, "IP is not supported: \"%s\"", addr_in);
 		goto err;
 	}
-
 #ifdef HAVE_IPV6
 	if (info->ai->ai_family == AF_INET6)
 	{
@@ -1038,7 +1028,7 @@ static int	get_addr_info(const char *addr_in, const char *port_in, struct addrin
 		if (NULL != addr)
 		{
 			if (-1 == prefix_sz_local)
-				prefix_sz_local = IPV6_MAX_CIDR_PREFIX;
+				prefix_sz_local = ZBX_IPV6_MAX_CIDR_PREFIX;
 
 			if (0 == memcmp(((struct sockaddr_in6*)info->ai->ai_addr)->sin6_addr.s6_addr, ipv6_mapped, 12))
 				info->mapped = 1;
@@ -1051,7 +1041,7 @@ static int	get_addr_info(const char *addr_in, const char *port_in, struct addrin
 #endif
 	{
 		if (NULL != addr && -1 == prefix_sz_local)
-			prefix_sz_local = IPV4_MAX_CIDR_PREFIX;
+			prefix_sz_local = ZBX_IPV4_MAX_CIDR_PREFIX;
 
 		if (NULL != service)
 			info->port = ntohs(((struct sockaddr_in*)info->ai->ai_addr)->sin_port);
@@ -1162,12 +1152,12 @@ err:
 	return ret;
 }
 
-int	NET_TCP_SOCKET_COUNT(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_tcp_socket_count(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	return net_socket_count(NET_CONN_TYPE_TCP, request, result);
 }
 
-int	NET_UDP_SOCKET_COUNT(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	net_udp_socket_count(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	return net_socket_count(NET_CONN_TYPE_UDP, request, result);
 }
