@@ -1,29 +1,25 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
-#include "common.h"
-#include "sysinfo.h"
-#include "zbxjson.h"
-#include "log.h"
-#include "zbxalgo.h"
+#include "zbxsysinfo.h"
+#include "../sysinfo.h"
+
 #include "inodes.h"
 
+#include "zbxjson.h"
+#include "zbxalgo.h"
+
 static int	get_fs_size_stat(const char *fs, zbx_uint64_t *total, zbx_uint64_t *free,
 		zbx_uint64_t *used, double *pfree, double *pused, char **error)
 {
@@ -78,9 +74,11 @@ static int	get_fs_size_stat(const char *fs, zbx_uint64_t *total, zbx_uint64_t *f
 	}
 
 	return SYSINFO_RET_OK;
+#undef ZBX_BSIZE
+#undef ZBX_STATFS
 }
 
-static int	VFS_FS_USED(const char *fs, AGENT_RESULT *result)
+static int	vfs_fs_used(const char *fs, AGENT_RESULT *result)
 {
 	zbx_uint64_t	value;
 	char		*error;
@@ -96,7 +94,7 @@ static int	VFS_FS_USED(const char *fs, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-static int	VFS_FS_FREE(const char *fs, AGENT_RESULT *result)
+static int	vfs_fs_free(const char *fs, AGENT_RESULT *result)
 {
 	zbx_uint64_t	value;
 	char		*error;
@@ -112,7 +110,7 @@ static int	VFS_FS_FREE(const char *fs, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-static int	VFS_FS_TOTAL(const char *fs, AGENT_RESULT *result)
+static int	vfs_fs_total(const char *fs, AGENT_RESULT *result)
 {
 	zbx_uint64_t	value;
 	char		*error;
@@ -128,7 +126,7 @@ static int	VFS_FS_TOTAL(const char *fs, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-static int	VFS_FS_PFREE(const char *fs, AGENT_RESULT *result)
+static int	vfs_fs_pfree(const char *fs, AGENT_RESULT *result)
 {
 	double	value;
 	char	*error;
@@ -144,7 +142,7 @@ static int	VFS_FS_PFREE(const char *fs, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-static int	VFS_FS_PUSED(const char *fs, AGENT_RESULT *result)
+static int	vfs_fs_pused(const char *fs, AGENT_RESULT *result)
 {
 	double	value;
 	char	*error;
@@ -160,7 +158,7 @@ static int	VFS_FS_PUSED(const char *fs, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-static int	vfs_fs_size(AGENT_REQUEST *request, AGENT_RESULT *result)
+static int	vfs_fs_size_local(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char	*fsname, *mode;
 
@@ -180,27 +178,27 @@ static int	vfs_fs_size(AGENT_REQUEST *request, AGENT_RESULT *result)
 	}
 
 	if (NULL == mode || '\0' == *mode || 0 == strcmp(mode, "total"))	/* default parameter */
-		return VFS_FS_TOTAL(fsname, result);
+		return vfs_fs_total(fsname, result);
 	if (0 == strcmp(mode, "free"))
-		return VFS_FS_FREE(fsname, result);
+		return vfs_fs_free(fsname, result);
 	if (0 == strcmp(mode, "pfree"))
-		return VFS_FS_PFREE(fsname, result);
+		return vfs_fs_pfree(fsname, result);
 	if (0 == strcmp(mode, "used"))
-		return VFS_FS_USED(fsname, result);
+		return vfs_fs_used(fsname, result);
 	if (0 == strcmp(mode, "pused"))
-		return VFS_FS_PUSED(fsname, result);
+		return vfs_fs_pused(fsname, result);
 
 	SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid second parameter."));
 
 	return SYSINFO_RET_FAIL;
 }
 
-int	VFS_FS_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	vfs_fs_size(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
-	return zbx_execute_threaded_metric(vfs_fs_size, request, result);
+	return zbx_execute_threaded_metric(vfs_fs_size_local, request, result);
 }
 
-int	VFS_FS_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	vfs_fs_discovery(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	struct mnttab	mt;
 	FILE		*f;
@@ -221,6 +219,7 @@ int	VFS_FS_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
 		zbx_json_addobject(&j, NULL);
 		zbx_json_addstring(&j, ZBX_LLD_MACRO_FSNAME, mt.mnt_mountp, ZBX_JSON_TYPE_STRING);
 		zbx_json_addstring(&j, ZBX_LLD_MACRO_FSTYPE, mt.mnt_fstype, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&j, ZBX_LLD_MACRO_FSOPTIONS, mt.mnt_mntopts, ZBX_JSON_TYPE_STRING);
 		zbx_json_close(&j);
 	}
 
@@ -235,16 +234,14 @@ int	VFS_FS_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-static int	vfs_fs_get(AGENT_REQUEST *request, AGENT_RESULT *result)
+static int	vfs_fs_get_local(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	struct mnttab		mt;
 	FILE			*f;
 	struct zbx_json		j;
-	zbx_uint64_t		total, not_used, used;
-	zbx_uint64_t		itotal, inot_used, iused;
-	double			pfree, pused;
-	double			ipfree, ipused;
-	char 			*error;
+	zbx_uint64_t		total, not_used, used, itotal, inot_used, iused;
+	double			pfree, pused, ipfree, ipused;
+	char			*error;
 	zbx_vector_ptr_t	mntpoints;
 	zbx_mpoint_t		*mntpoint;
 	int			ret = SYSINFO_RET_FAIL;
@@ -289,6 +286,7 @@ static int	vfs_fs_get(AGENT_REQUEST *request, AGENT_RESULT *result)
 		mntpoint->inodes.not_used = inot_used;
 		mntpoint->inodes.pfree = ipfree;
 		mntpoint->inodes.pused = ipused;
+		mntpoint->options = zbx_strdup(NULL, mt.mnt_mntopts);
 
 		zbx_vector_ptr_append(&mntpoints, mntpoint);
 	}
@@ -330,6 +328,7 @@ static int	vfs_fs_get(AGENT_REQUEST *request, AGENT_RESULT *result)
 			zbx_json_addfloat(&j, ZBX_SYSINFO_TAG_PFREE, mntpoint->inodes.pfree);
 			zbx_json_addfloat(&j, ZBX_SYSINFO_TAG_PUSED, mntpoint->inodes.pused);
 			zbx_json_close(&j);
+			zbx_json_addstring(&j, ZBX_SYSINFO_TAG_FSOPTIONS, mntpoint->options, ZBX_JSON_TYPE_STRING);
 			zbx_json_close(&j);
 		}
 	}
@@ -347,8 +346,8 @@ out:
 
 	return ret;
 }
-int	VFS_FS_GET(AGENT_REQUEST *request, AGENT_RESULT *result)
-{
-	return zbx_execute_threaded_metric(vfs_fs_get, request, result);
 
+int	vfs_fs_get(AGENT_REQUEST *request, AGENT_RESULT *result)
+{
+	return zbx_execute_threaded_metric(vfs_fs_get_local, request, result);
 }
