@@ -1,38 +1,31 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 #include "timer.h"
 
-#include "log.h"
-#include "dbcache.h"
-#include "daemon.h"
+#include "zbxalgo.h"
+#include "zbxdb.h"
+#include "zbxdbhigh.h"
+#include "zbxstr.h"
+#include "zbxlog.h"
+#include "zbxcacheconfig.h"
+#include "zbxnix.h"
 #include "zbxself.h"
-#include "service_protocol.h"
-
-#define ZBX_TIMER_DELAY		SEC_PER_MIN
-
-#define ZBX_EVENT_BATCH_SIZE	1000
-
-extern ZBX_THREAD_LOCAL unsigned char	process_type;
-extern unsigned char			program_type;
-extern ZBX_THREAD_LOCAL int		server_num, process_num;
-extern int				CONFIG_TIMER_FORKS;
+#include "zbxnum.h"
+#include "zbxtime.h"
+#include "zbx_host_constants.h"
+#include "zbxservice.h"
 
 /* addition data for event maintenance calculations to pair with zbx_event_suppress_query_t */
 typedef struct
@@ -44,7 +37,7 @@ zbx_event_suppress_data_t;
 
 /******************************************************************************
  *                                                                            *
- * Purpose: log host maintenance changes                                      *
+ * Purpose: logs host maintenance changes                                     *
  *                                                                            *
  ******************************************************************************/
 static void	log_host_maintenance_update(const zbx_host_maintenance_diff_t* diff)
@@ -88,23 +81,20 @@ static void	log_host_maintenance_update(const zbx_host_maintenance_diff_t* diff)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: update host maintenance properties in database                    *
+ * Purpose: updates host maintenance properties in database                   *
  *                                                                            *
  ******************************************************************************/
-static void	db_update_host_maintenances(const zbx_vector_ptr_t *updates)
+static void	db_update_host_maintenances(const zbx_vector_host_maintenance_diff_ptr_t *updates)
 {
-	int					i;
-	const zbx_host_maintenance_diff_t	*diff;
 	char					*sql = NULL;
 	size_t					sql_alloc = 0, sql_offset = 0;
 
-	DBbegin_multiple_update(&sql, &sql_alloc, &sql_offset);
+	zbx_db_begin_multiple_update(&sql, &sql_alloc, &sql_offset);
 
-	for (i = 0; i < updates->values_num; i++)
+	for (int i = 0; i < updates->values_num; i++)
 	{
-		char	delim = ' ';
-
-		diff = (const zbx_host_maintenance_diff_t *)updates->values[i];
+		char					delim = ' ';
+		const zbx_host_maintenance_diff_t	*diff = updates->values[i];
 
 		zbx_strcpy_alloc(&sql, &sql_alloc, &sql_offset, "update hosts set");
 
@@ -145,17 +135,17 @@ static void	db_update_host_maintenances(const zbx_vector_ptr_t *updates)
 
 		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, " where hostid=" ZBX_FS_UI64 ";\n", diff->hostid);
 
-		if (SUCCEED != DBexecute_overflowed_sql(&sql, &sql_alloc, &sql_offset))
+		if (SUCCEED != zbx_db_execute_overflowed_sql(&sql, &sql_alloc, &sql_offset))
 			break;
 
 		if (SUCCEED == ZBX_CHECK_LOG_LEVEL(LOG_LEVEL_DEBUG))
 			log_host_maintenance_update(diff);
 	}
 
-	DBend_multiple_update(&sql, &sql_alloc, &sql_offset);
+	zbx_db_end_multiple_update(&sql, &sql_alloc, &sql_offset);
 
 	if (16 < sql_offset)
-		DBexecute("%s", sql);
+		zbx_db_execute("%s", sql);
 
 	zbx_free(sql);
 }
@@ -179,34 +169,35 @@ static void     service_send_suppression_data(const zbx_vector_uint64_t *eventid
 	zbx_free(data);
 }
 
-
 /******************************************************************************
  *                                                                            *
- * Purpose: remove expired event_suppress records                             *
+ * Purpose: removes expired event_suppress records                            *
  *                                                                            *
  ******************************************************************************/
-static void	db_remove_expired_event_suppress_data(int now)
+static void	db_remove_expired_event_suppress_data(time_t now)
 {
 	zbx_vector_uint64_t	eventids;
-	DB_ROW			row;
-	DB_RESULT		result;
+	zbx_db_row_t		row;
+	zbx_db_result_t		result;
 
 	zbx_vector_uint64_create(&eventids);
 
-	result = DBselect("select eventid from event_suppress where suppress_until<%d", now);
+	result = zbx_db_select("select eventid from event_suppress where suppress_until<" ZBX_FS_TIME_T
+			" and suppress_until<>0", (zbx_fs_time_t)now);
 
-	while (NULL != (row = DBfetch(result)))
+	while (NULL != (row = zbx_db_fetch(result)))
 	{
 		zbx_uint64_t	eventid;
 		ZBX_STR2UINT64(eventid, row[0]);
 
 		zbx_vector_uint64_append(&eventids, eventid);
 	}
-	DBfree_result(result);
+	zbx_db_free_result(result);
 
-	DBbegin();
-	DBexecute("delete from event_suppress where suppress_until<%d", now);
-	DBcommit();
+	zbx_db_begin();
+	zbx_db_execute("delete from event_suppress where suppress_until<" ZBX_FS_TIME_T " and suppress_until<>0",
+			(zbx_fs_time_t)now);
+	zbx_db_commit();
 
 	service_send_suppression_data(&eventids, 0);
 
@@ -215,7 +206,7 @@ static void	db_remove_expired_event_suppress_data(int now)
 
 /******************************************************************************
  *                                                                            *
- * Purpose: free event suppress data structure                                *
+ * Purpose: frees event suppress data structure                               *
  *                                                                            *
  ******************************************************************************/
 static void	event_suppress_data_free(zbx_event_suppress_data_t *data)
@@ -224,18 +215,30 @@ static void	event_suppress_data_free(zbx_event_suppress_data_t *data)
 	zbx_free(data);
 }
 
+ZBX_PTR_VECTOR_IMPL(event_suppress_query_ptr, zbx_event_suppress_query_t*)
+
+static int	event_suppress_query_eventid_compare(const void *d1, const void *d2)
+{
+	const zbx_event_suppress_query_t	*ds1 = *(const zbx_event_suppress_query_t * const *)d1;
+	const zbx_event_suppress_query_t	*ds2 = *(const zbx_event_suppress_query_t * const *)d2;
+
+	ZBX_RETURN_IF_NOT_EQUAL(ds1->eventid, ds2->eventid);
+
+	return 0;
+}
+
 /******************************************************************************
  *                                                                            *
- * Purpose: fetch events that need to be queried for maintenance              *
+ * Purpose: fetches events that need to be queried for maintenance            *
  *                                                                            *
  ******************************************************************************/
-static void	event_queries_fetch(DB_RESULT result, zbx_vector_ptr_t *event_queries)
+static void	event_queries_fetch(zbx_db_result_t result, zbx_vector_event_suppress_query_ptr_t *event_queries)
 {
-	DB_ROW				row;
+	zbx_db_row_t			row;
 	zbx_uint64_t			eventid;
 	zbx_event_suppress_query_t	*query = NULL;
 
-	while (NULL != (row = DBfetch(result)))
+	while (NULL != (row = zbx_db_fetch(result)))
 	{
 		ZBX_STR2UINT64(eventid, row[0]);
 
@@ -247,34 +250,47 @@ static void	event_queries_fetch(DB_RESULT result, zbx_vector_ptr_t *event_querie
 			ZBX_STR2UINT64(query->triggerid, row[1]);
 			ZBX_DBROW2UINT64(query->r_eventid, row[2]);
 			zbx_vector_uint64_create(&query->functionids);
-			zbx_vector_ptr_create(&query->tags);
+			zbx_vector_tags_ptr_create(&query->tags);
 			zbx_vector_uint64_pair_create(&query->maintenances);
-			zbx_vector_ptr_append(event_queries, query);
+			zbx_vector_event_suppress_query_ptr_append(event_queries, query);
 		}
 
-		if (FAIL == DBis_null(row[3]))
+		if (FAIL == zbx_db_is_null(row[3]))
 		{
-			zbx_tag_t	*tag;
+			zbx_tag_t	*tag = (zbx_tag_t *)zbx_malloc(NULL, sizeof(zbx_tag_t));
 
-			tag = (zbx_tag_t *)zbx_malloc(NULL, sizeof(zbx_tag_t));
 			tag->tag = zbx_strdup(NULL, row[3]);
 			tag->value = zbx_strdup(NULL, row[4]);
-			zbx_vector_ptr_append(&query->tags, tag);
-
+			zbx_vector_tags_ptr_append(&query->tags, tag);
 		}
 	}
 }
 
+ZBX_PTR_VECTOR_DECL(event_suppress_data_ptr, zbx_event_suppress_data_t*)
+ZBX_PTR_VECTOR_IMPL(event_suppress_data_ptr, zbx_event_suppress_data_t*)
+
+static int	event_suppress_data_eventid_compare(const void *d1, const void *d2)
+{
+	const zbx_event_suppress_data_t    *ds1 = *(const zbx_event_suppress_data_t * const *)d1;
+	const zbx_event_suppress_data_t    *ds2 = *(const zbx_event_suppress_data_t * const *)d2;
+
+	ZBX_RETURN_IF_NOT_EQUAL(ds1->eventid, ds2->eventid);
+
+	return 0;
+}
+
 /******************************************************************************
  *                                                                            *
- * Purpose: get open, recently resolved and resolved problems with suppress   *
- *          data from database and prepare event query, event data structures *
+ * Purpose: Gets open, recently resolved and resolved problems with suppress  *
+ *          data from database and prepares event query, event data           *
+ *          structures.                                                       *
  *                                                                            *
  ******************************************************************************/
-static void	db_get_query_events(zbx_vector_ptr_t *event_queries, zbx_vector_ptr_t *event_data)
+static void	db_get_query_events(zbx_vector_event_suppress_query_ptr_t *event_queries,
+		zbx_vector_event_suppress_data_ptr_t *event_data, int process_num, zbx_get_config_forks_f get_forks_cb)
 {
-	DB_ROW				row;
-	DB_RESULT			result;
+	zbx_db_row_t			row;
+	zbx_db_result_t			result;
 	zbx_event_suppress_data_t	*data = NULL;
 	zbx_uint64_t			eventid;
 	zbx_uint64_pair_t		pair;
@@ -294,7 +310,7 @@ static void	db_get_query_events(zbx_vector_ptr_t *event_queries, zbx_vector_ptr_
 	}
 
 	/* get open or recently closed problems */
-	result = DBselect("select p.eventid,p.objectid,p.r_eventid,%s"
+	result = zbx_db_select("select p.eventid,p.objectid,p.r_eventid,%s"
 			" from problem p"
 			"%s"
 			" where p.source=%d"
@@ -302,48 +318,52 @@ static void	db_get_query_events(zbx_vector_ptr_t *event_queries, zbx_vector_ptr_
 				" and " ZBX_SQL_MOD(p.eventid, %d) "=%d"
 			" order by p.eventid",
 			tag_fields, tag_join,
-			EVENT_SOURCE_TRIGGERS, EVENT_OBJECT_TRIGGER, CONFIG_TIMER_FORKS, process_num - 1);
+			EVENT_SOURCE_TRIGGERS, EVENT_OBJECT_TRIGGER, get_forks_cb(ZBX_PROCESS_TYPE_TIMER),
+			process_num - 1);
 
 	event_queries_fetch(result, event_queries);
-	DBfree_result(result);
+	zbx_db_free_result(result);
 
 	/* get event suppress data */
 
 	zbx_vector_uint64_create(&eventids);
 
-	result = DBselect("select eventid,maintenanceid,suppress_until"
+	result = zbx_db_select("select eventid,maintenanceid,suppress_until"
 			" from event_suppress"
-			" where " ZBX_SQL_MOD(eventid, %d) "=%d"
+			" where " ZBX_SQL_MOD(eventid, %d) "=%d and maintenanceid is not null"
 			" order by eventid",
-			CONFIG_TIMER_FORKS, process_num - 1);
+			get_forks_cb(ZBX_PROCESS_TYPE_TIMER), process_num - 1);
 
-	while (NULL != (row = DBfetch(result)))
+	while (NULL != (row = zbx_db_fetch(result)))
 	{
 		ZBX_STR2UINT64(eventid, row[0]);
 
-		if (FAIL == zbx_vector_ptr_bsearch(event_queries, &eventid, ZBX_DEFAULT_UINT64_PTR_COMPARE_FUNC))
+		zbx_event_suppress_query_t	event_query_search = {.eventid = eventid};
+
+		if (FAIL == zbx_vector_event_suppress_query_ptr_bsearch(event_queries, &event_query_search,
+				event_suppress_query_eventid_compare))
+		{
 			zbx_vector_uint64_append(&eventids, eventid);
+		}
 
 		if (NULL == data || data->eventid != eventid)
 		{
 			data = (zbx_event_suppress_data_t *)zbx_malloc(NULL, sizeof(zbx_event_suppress_data_t));
 			data->eventid = eventid;
 			zbx_vector_uint64_pair_create(&data->maintenances);
-			zbx_vector_ptr_append(event_data, data);
+			zbx_vector_event_suppress_data_ptr_append(event_data, data);
 		}
 
 		ZBX_DBROW2UINT64(pair.first, row[1]);
 		pair.second = atoi(row[2]);
 		zbx_vector_uint64_pair_append(&data->maintenances, pair);
 	}
-	DBfree_result(result);
+	zbx_db_free_result(result);
 
 	/* get missing event data */
 
 	if (0 != eventids.values_num)
 	{
-		int	i;
-
 		if (SUCCEED == read_tags)
 		{
 			tag_fields = "t.tag,t.value";
@@ -352,7 +372,8 @@ static void	db_get_query_events(zbx_vector_ptr_t *event_queries, zbx_vector_ptr_
 
 		zbx_vector_uint64_uniq(&eventids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
 
-		for (i = 0; i < eventids.values_num; i += ZBX_EVENT_BATCH_SIZE)
+#define ZBX_EVENT_BATCH_SIZE	1000
+		for (int i = 0; i < eventids.values_num; i += ZBX_EVENT_BATCH_SIZE)
 		{
 			char	*sql = NULL;
 			size_t	sql_alloc = 0, sql_offset = 0;
@@ -365,18 +386,18 @@ static void	db_get_query_events(zbx_vector_ptr_t *event_queries, zbx_vector_ptr_
 					"%s"
 					" where",
 					tag_fields, tag_join);
-			DBadd_condition_alloc(&sql, &sql_alloc, &sql_offset, "e.eventid",
+			zbx_db_add_condition_alloc(&sql, &sql_alloc, &sql_offset, "e.eventid",
 					eventids.values + i, MIN(eventids.values_num - i, ZBX_EVENT_BATCH_SIZE));
 			zbx_strcpy_alloc(&sql, &sql_alloc, &sql_offset, " order by e.eventid");
 
-			result = DBselect("%s", sql);
+			result = zbx_db_select("%s", sql);
 			zbx_free(sql);
 
 			event_queries_fetch(result, event_queries);
-			DBfree_result(result);
+			zbx_db_free_result(result);
 		}
-
-		zbx_vector_ptr_sort(event_queries, ZBX_DEFAULT_UINT64_PTR_COMPARE_FUNC);
+#undef ZBX_EVENT_BATCH_SIZE
+		zbx_vector_event_suppress_query_ptr_sort(event_queries, ZBX_DEFAULT_UINT64_PTR_COMPARE_FUNC);
 	}
 
 	zbx_vector_uint64_destroy(&eventids);
@@ -384,31 +405,34 @@ static void	db_get_query_events(zbx_vector_ptr_t *event_queries, zbx_vector_ptr_
 
 /******************************************************************************
  *                                                                            *
- * Purpose: create/update event suppress data to reflect latest maintenance   *
- *          changes in cache                                                  *
+ * Purpose: Creates/Updates event suppress data to reflect latest maintenance *
+ *          changes in cache.                                                 *
  *                                                                            *
- * Parameters: suppressed_num - [OUT] the number of suppressed events         *
+ * Parameters: suppressed_num - [OUT]                                         *
+ *             process_num    - [IN]                                          *
+ *             get_forks_cb   - [IN]                                          *
  *                                                                            *
  ******************************************************************************/
-static void	db_update_event_suppress_data(int *suppressed_num)
+static void	db_update_event_suppress_data(int *suppressed_num, int process_num, zbx_get_config_forks_f get_forks_cb)
 {
-	zbx_vector_ptr_t	event_queries, event_data;
-	zbx_vector_uint64_t	s_eventids;
+	zbx_vector_event_suppress_query_ptr_t	event_queries;
+	zbx_vector_event_suppress_data_ptr_t	event_data;
+	zbx_vector_uint64_t			s_eventids;
 
 	*suppressed_num = 0;
 
-	zbx_vector_ptr_create(&event_queries);
-	zbx_vector_ptr_create(&event_data);
+	zbx_vector_event_suppress_query_ptr_create(&event_queries);
+	zbx_vector_event_suppress_data_ptr_create(&event_data);
 	zbx_vector_uint64_create(&s_eventids);
 
-	db_get_query_events(&event_queries, &event_data);
+	db_get_query_events(&event_queries, &event_data, process_num, get_forks_cb);
 
 	if (0 != event_queries.values_num)
 	{
 		zbx_db_insert_t			db_insert;
 		char				*sql = NULL;
 		size_t				sql_alloc = 0, sql_offset = 0;
-		int				i, j, k;
+		int				j, k;
 		zbx_event_suppress_query_t	*query;
 		zbx_event_suppress_data_t	*data;
 		zbx_vector_uint64_pair_t	del_event_maintenances;
@@ -420,26 +444,28 @@ static void	db_update_event_suppress_data(int *suppressed_num)
 
 		zbx_dc_get_running_maintenanceids(&maintenanceids);
 
-		DBbegin();
+		zbx_db_begin();
 
 		if (0 != maintenanceids.values_num && SUCCEED == zbx_db_lock_maintenanceids(&maintenanceids))
 			zbx_dc_get_event_maintenances(&event_queries, &maintenanceids);
 
 		zbx_db_insert_prepare(&db_insert, "event_suppress", "event_suppressid", "eventid", "maintenanceid",
 				"suppress_until", (char *)NULL);
-		DBbegin_multiple_update(&sql, &sql_alloc, &sql_offset);
+		zbx_db_begin_multiple_update(&sql, &sql_alloc, &sql_offset);
 
-		for (i = 0; i < event_queries.values_num; i++)
+		for (int i = 0; i < event_queries.values_num; i++)
 		{
-			query = (zbx_event_suppress_query_t *)event_queries.values[i];
+			query = event_queries.values[i];
 			zbx_vector_uint64_pair_sort(&query->maintenances, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
 
 			k = 0;
 
-			if (FAIL != (j = zbx_vector_ptr_bsearch(&event_data, &query->eventid,
-					ZBX_DEFAULT_UINT64_PTR_COMPARE_FUNC)))
+			zbx_event_suppress_data_t event_suppress_data_search = {.eventid = query->eventid};
+
+			if (FAIL != (j = zbx_vector_event_suppress_data_ptr_bsearch(&event_data,
+					&event_suppress_data_search, event_suppress_data_eventid_compare)))
 			{
-				data = (zbx_event_suppress_data_t *)event_data.values[j];
+				data = event_data.values[j];
 				zbx_vector_uint64_pair_sort(&data->maintenances, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
 
 				j = 0;
@@ -483,8 +509,11 @@ static void	db_update_event_suppress_data(int *suppressed_num)
 									query->eventid,
 									query->maintenances.values[k].first);
 
-						if (FAIL == DBexecute_overflowed_sql(&sql, &sql_alloc, &sql_offset))
+						if (FAIL == zbx_db_execute_overflowed_sql(&sql, &sql_alloc,
+								&sql_offset))
+						{
 							goto cleanup;
+						}
 					}
 					j++;
 					k++;
@@ -513,7 +542,7 @@ static void	db_update_event_suppress_data(int *suppressed_num)
 		}
 
 		zbx_vector_uint64_create(&eventids);
-		for (i = 0; i < del_event_maintenances.values_num; i++)
+		for (int i = 0; i < del_event_maintenances.values_num; i++)
 		{
 			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
 					"delete from event_suppress"
@@ -521,27 +550,28 @@ static void	db_update_event_suppress_data(int *suppressed_num)
 						" and maintenanceid=" ZBX_FS_UI64 ";\n",
 						del_event_maintenances.values[i].first,
 						del_event_maintenances.values[i].second);
+
 			zbx_vector_uint64_append(&eventids, del_event_maintenances.values[i].first);
 
-			if (FAIL == DBexecute_overflowed_sql(&sql, &sql_alloc, &sql_offset))
+			if (FAIL == zbx_db_execute_overflowed_sql(&sql, &sql_alloc, &sql_offset))
 				goto cleanup;
 		}
 		service_send_suppression_data(&eventids, 0);
 
 		zbx_vector_uint64_destroy(&eventids);
 
-		DBend_multiple_update(&sql, &sql_alloc, &sql_offset);
+		zbx_db_end_multiple_update(&sql, &sql_alloc, &sql_offset);
 
 		if (16 < sql_offset)
 		{
-			if (ZBX_DB_OK > DBexecute("%s", sql))
+			if (ZBX_DB_OK > zbx_db_execute("%s", sql))
 				goto cleanup;
 		}
 
 		zbx_db_insert_autoincrement(&db_insert, "event_suppressid");
 		zbx_db_insert_execute(&db_insert);
 cleanup:
-		DBcommit();
+		zbx_db_commit();
 
 		zbx_db_insert_clean(&db_insert);
 		zbx_free(sql);
@@ -552,34 +582,33 @@ cleanup:
 		service_send_suppression_data(&s_eventids, 1);
 	}
 
-	zbx_vector_ptr_clear_ext(&event_data, (zbx_clean_func_t)event_suppress_data_free);
-	zbx_vector_ptr_destroy(&event_data);
+	zbx_vector_event_suppress_data_ptr_clear_ext(&event_data, event_suppress_data_free);
+	zbx_vector_event_suppress_data_ptr_destroy(&event_data);
 
-	zbx_vector_ptr_clear_ext(&event_queries, (zbx_clean_func_t)zbx_event_suppress_query_free);
-	zbx_vector_ptr_destroy(&event_queries);
+	zbx_vector_event_suppress_query_ptr_clear_ext(&event_queries, zbx_event_suppress_query_free);
+	zbx_vector_event_suppress_query_ptr_destroy(&event_queries);
 
 	zbx_vector_uint64_destroy(&s_eventids);
 }
 
 /******************************************************************************
  *                                                                            *
- * Purpose: update host maintenance parameters in cache and database          *
+ * Purpose: updates host maintenance parameters in cache and database         *
  *                                                                            *
  ******************************************************************************/
 static int	update_host_maintenances(void)
 {
-	zbx_vector_uint64_t	maintenanceids;
-	zbx_vector_ptr_t	updates;
-	int			hosts_num = 0;
-	int			tnx_error;
+	zbx_vector_uint64_t			maintenanceids;
+	zbx_vector_host_maintenance_diff_ptr_t	updates;
+	int					tnx_error, hosts_num = 0;
 
 	zbx_vector_uint64_create(&maintenanceids);
-	zbx_vector_ptr_create(&updates);
-	zbx_vector_ptr_reserve(&updates, 100);
+	zbx_vector_host_maintenance_diff_ptr_create(&updates);
+	zbx_vector_host_maintenance_diff_ptr_reserve(&updates, 100);
 
 	do
 	{
-		DBbegin();
+		zbx_db_begin();
 
 		if (SUCCEED == zbx_dc_get_running_maintenanceids(&maintenanceids))
 			zbx_db_lock_maintenanceids(&maintenanceids);
@@ -591,15 +620,15 @@ static int	update_host_maintenances(void)
 		if (0 != updates.values_num)
 			db_update_host_maintenances(&updates);
 
-		if (ZBX_DB_OK == (tnx_error = DBcommit()) && 0 != (hosts_num = updates.values_num))
+		if (ZBX_DB_OK == (tnx_error = zbx_db_commit()) && 0 != (hosts_num = updates.values_num))
 			zbx_dc_flush_host_maintenance_updates(&updates);
 
-		zbx_vector_ptr_clear_ext(&updates, (zbx_clean_func_t)zbx_ptr_free);
+		zbx_vector_host_maintenance_diff_ptr_clear_ext(&updates, zbx_host_maintenance_diff_free);
 		zbx_vector_uint64_clear(&maintenanceids);
 	}
 	while (ZBX_DB_DOWN == tnx_error);
 
-	zbx_vector_ptr_destroy(&updates);
+	zbx_vector_host_maintenance_diff_ptr_destroy(&updates);
 	zbx_vector_uint64_destroy(&maintenanceids);
 
 	return hosts_num;
@@ -612,42 +641,55 @@ static int	update_host_maintenances(void)
  ******************************************************************************/
 ZBX_THREAD_ENTRY(timer_thread, args)
 {
-	double		sec;
-	int		maintenance_time = 0, update_time = 0, idle = 1, events_num, hosts_num, update;
-	char		*info = NULL;
-	size_t		info_alloc = 0, info_offset = 0;
+#define ZBX_MAINTENANCE_TIMER_DELAY	SEC_PER_MIN
+	time_t			schedule_time = 0, update_time = 0;
+	char			*info = NULL;
+	size_t			info_alloc = 0, info_offset = 0;
+	const zbx_thread_info_t	*thread_info = &((zbx_thread_args_t *)args)->info;
+	int			events_num, hosts_num, update, idle = 1,
+				server_num = thread_info->server_num,
+				process_num = thread_info->process_num;
+	unsigned char		process_type = thread_info->process_type;
 
-	process_type = ((zbx_thread_args_t *)args)->process_type;
-	server_num = ((zbx_thread_args_t *)args)->server_num;
-	process_num = ((zbx_thread_args_t *)args)->process_num;
+	zbx_thread_timer_args	*args_in = (zbx_thread_timer_args *)(((zbx_thread_args_t *)args)->args);
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_program_type_string(program_type),
-			server_num, get_process_type_string(process_type), process_num);
+	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]",
+			get_program_type_string(thread_info->program_type), server_num,
+			get_process_type_string(process_type), process_num);
 
-	update_selfmon_counter(ZBX_PROCESS_STATE_BUSY);
+	zbx_update_selfmon_counter(thread_info, ZBX_PROCESS_STATE_BUSY);
 
 	zbx_setproctitle("%s #%d [connecting to the database]", get_process_type_string(process_type), process_num);
 	zbx_strcpy_alloc(&info, &info_alloc, &info_offset, "started");
 
-	DBconnect(ZBX_DB_CONNECT_NORMAL);
+	zbx_db_connect(ZBX_DB_CONNECT_NORMAL);
 
 	while (ZBX_IS_RUNNING())
 	{
-		sec = zbx_time();
+		double	sec = zbx_time();
 		zbx_update_env(get_process_type_string(process_type), sec);
 
 		if (1 == process_num)
 		{
+			zbx_maintenance_timer_t	maintenance_timer;
+
+			if (ZBX_MAINTENANCE_TIMER_DELAY <= sec - (double)schedule_time)
+				maintenance_timer = MAINTENANCE_TIMER_PENDING;
+			else
+				maintenance_timer = MAINTENANCE_TIMER_INITIALIZED;
+
 			/* start update process only when all timers have finished their updates */
-			if (sec - maintenance_time >= ZBX_TIMER_DELAY && FAIL == zbx_dc_maintenance_check_update_flags())
+			if ((SUCCEED == zbx_dc_maintenance_check_immediate_update() ||
+					MAINTENANCE_TIMER_PENDING == maintenance_timer) &&
+					FAIL == zbx_dc_maintenance_check_update_flags())
 			{
 				zbx_setproctitle("%s #%d [%s, processing maintenances]",
 						get_process_type_string(process_type), process_num, info);
 
-				update = zbx_dc_update_maintenances();
+				update = zbx_dc_update_maintenances(maintenance_timer);
 
 				/* force maintenance updates at server startup */
-				if (0 == maintenance_time)
+				if (0 == schedule_time)
 					update = SUCCEED;
 
 				/* update hosts if there are modified (stopped, started, changed) maintenances */
@@ -656,12 +698,14 @@ ZBX_THREAD_ENTRY(timer_thread, args)
 				else
 					hosts_num = 0;
 
-				db_remove_expired_event_suppress_data((int)sec);
+				if (MAINTENANCE_TIMER_PENDING == maintenance_timer)
+					db_remove_expired_event_suppress_data((time_t)sec);
 
 				if (SUCCEED == update)
 				{
 					zbx_dc_maintenance_set_update_flags();
-					db_update_event_suppress_data(&events_num);
+					db_update_event_suppress_data(&events_num, process_num,
+							args_in->get_process_forks_cb_arg);
 					zbx_dc_maintenance_reset_update_flag(process_num);
 				}
 				else
@@ -672,7 +716,8 @@ ZBX_THREAD_ENTRY(timer_thread, args)
 						"updated %d hosts, suppressed %d events in " ZBX_FS_DBL " sec",
 						hosts_num, events_num, zbx_time() - sec);
 
-				update_time = (int)sec;
+				if (MAINTENANCE_TIMER_PENDING == maintenance_timer)
+					update_time = (time_t)sec;
 			}
 		}
 		else if (SUCCEED == zbx_dc_maintenance_check_update_flag(process_num))
@@ -680,22 +725,22 @@ ZBX_THREAD_ENTRY(timer_thread, args)
 			zbx_setproctitle("%s #%d [%s, processing maintenances]", get_process_type_string(process_type),
 					process_num, info);
 
-			db_update_event_suppress_data(&events_num);
+			db_update_event_suppress_data(&events_num, process_num, args_in->get_process_forks_cb_arg);
 
 			info_offset = 0;
 			zbx_snprintf_alloc(&info, &info_alloc, &info_offset, "suppressed %d events in " ZBX_FS_DBL
 					" sec", events_num, zbx_time() - sec);
 
-			update_time = (int)sec;
+			update_time = (time_t)sec;
 			zbx_dc_maintenance_reset_update_flag(process_num);
 		}
 
-		if (maintenance_time != update_time)
+		if (schedule_time != update_time)
 		{
 			update_time -= update_time % 60;
-			maintenance_time = update_time;
+			schedule_time = update_time;
 
-			if (0 > (idle = ZBX_TIMER_DELAY - (zbx_time() - maintenance_time)))
+			if (0 > (idle = (int)(ZBX_MAINTENANCE_TIMER_DELAY - (zbx_time() - (double)schedule_time))))
 				idle = 0;
 
 			zbx_setproctitle("%s #%d [%s, idle %d sec]",
@@ -703,7 +748,7 @@ ZBX_THREAD_ENTRY(timer_thread, args)
 		}
 
 		if (0 != idle)
-			zbx_sleep_loop(1);
+			zbx_sleep_loop(thread_info, 1);
 
 		idle = 1;
 	}
@@ -712,4 +757,5 @@ ZBX_THREAD_ENTRY(timer_thread, args)
 
 	while (1)
 		zbx_sleep(SEC_PER_MIN);
+#undef ZBX_MAINTENANCE_TIMER_DELAY
 }
