@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -40,6 +35,7 @@ abstract class CControllerLatest extends CController {
 		'name' => '',
 		'evaltype' => TAG_EVAL_TYPE_AND_OR,
 		'tags' => [],
+		'state' => -1,
 		'show_tags' => SHOW_TAGS_3,
 		'tag_name_format' => TAG_NAME_FULL,
 		'tag_priority' => '',
@@ -50,9 +46,13 @@ abstract class CControllerLatest extends CController {
 		'subfilter_hostids' => [],
 		'subfilter_tagnames' => [],
 		'subfilter_tags' => [],
+		'subfilter_state' => [],
 		'subfilter_data' => []
 	];
 
+	// Mandatory filter fields.
+	public const MANDATORY_FILTER_FIELDS = ['groupids', 'hostids', 'name', 'tags', 'state'];
+
 	/**
 	 * Prepare the latest data based on the given filter and sorting options.
 	 *
@@ -65,6 +65,7 @@ abstract class CControllerLatest extends CController {
 	 * @param string $filter['tags'][]['tag']
 	 * @param string $filter['tags'][]['value']
 	 * @param int    $filter['tags'][]['operator']
+	 * @param int    $filter['state']              Filter state.
 	 * @param string $sort_field                   Sorting field.
 	 * @param string $sort_order                   Sorting order.
 	 *
@@ -99,11 +100,10 @@ abstract class CControllerLatest extends CController {
 				'evaltype' => $filter['evaltype'],
 				'tags' => $filter['tags'] ?: null,
 				'filter' => [
-					'status' => [ITEM_STATUS_ACTIVE]
-				],
-				'search' => ($filter['name'] === '') ? null : [
-					'name' => $filter['name']
+					'status' => [ITEM_STATUS_ACTIVE],
+					'state' => $filter['state'] == -1 ? null : $filter['state']
 				],
+				'search' => $filter['name'] === '' ? null : ['name_resolved' => $filter['name']],
 				'preservekeys' => true
 			]);
 
@@ -111,16 +111,30 @@ abstract class CControllerLatest extends CController {
 		}
 
 		if ($select_items) {
-			$items = API::Item()->get([
-				'output' => ['itemid', 'type', 'hostid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
-					'value_type', 'units', 'description', 'state', 'error'
+			$items = CArrayHelper::renameObjectsKeys(API::Item()->get([
+				'output' => ['itemid', 'type', 'hostid', 'name_resolved', 'key_', 'delay', 'history', 'trends',
+					'status', 'value_type', 'units', 'description', 'state', 'error'
 				],
 				'selectTags' => ['tag', 'value'],
 				'selectValueMap' => ['mappings'],
 				'itemids' => array_keys($select_items),
 				'webitems' => true,
 				'preservekeys' => true
-			]);
+			]), ['name_resolved' => 'name']);
+
+			// If user role checkbox 'Invoke "Execute now" on read-only hosts' is ON, read-write items are the same.
+			$items_rw = $items;
+
+			// If user role checkbox 'Invoke "Execute now" on read-only hosts' is OFF, get only read-write items.
+			if (!$this->hasInput('filter_counters') && $this->getUserType() < USER_TYPE_SUPER_ADMIN
+					&& !$this->checkAccess(CRoleHelper::ACTIONS_INVOKE_EXECUTE_NOW)) {
+				$items_rw = API::Item()->get([
+					'output' => [],
+					'itemids' => array_keys($items),
+					'editable' => true,
+					'preservekeys' => true
+				]);
+			}
 
 			if ($sort_field === 'host') {
 				$items = array_map(function ($item) use ($hosts) {
@@ -144,11 +158,13 @@ abstract class CControllerLatest extends CController {
 		else {
 			$hosts = [];
 			$items = [];
+			$items_rw = [];
 		}
 
 		return [
 			'hosts' => $hosts,
-			'items' => $items
+			'items' => $items,
+			'items_rw' => $items_rw
 		];
 	}
 
@@ -253,6 +269,7 @@ abstract class CControllerLatest extends CController {
 	 * @param array  $filter['subfilter_hostids']   Selected host subfilter parameters.
 	 * @param array  $filter['subfilter_tagnames']  Selected tagname subfilter parameters.
 	 * @param array  $filter['subfilter_tags']      Selected tags subfilter parameters.
+	 * @param array  $filter['subfilter_state']     Selected state subfilter parameters.
 	 * @param array  $filter['subfilter_data']      Selected data subfilter parameters.
 	 *
 	 * @return array
@@ -268,6 +285,7 @@ abstract class CControllerLatest extends CController {
 			'hostids' => array_flip($filter['subfilter_hostids']),
 			'tagnames' => array_flip($filter['subfilter_tagnames']),
 			'tags' => $tags,
+			'state' => $filter['state'] == -1 ? array_flip($filter['subfilter_state']) : [],
 			'data' => array_flip($filter['subfilter_data'])
 		];
 	}
@@ -366,6 +384,22 @@ abstract class CControllerLatest extends CController {
 				}
 			}
 
+			// State subfilter.
+			$item_matches = true;
+			foreach ($item['matching_subfilters'] as $filter_name => $match) {
+				if ($filter_name === 'state') {
+					continue;
+				}
+				if (!$match) {
+					$item_matches = false;
+					break;
+				}
+			}
+
+			if ($item_matches) {
+				$subfilter_options['state'][$item['state']]['count']++;
+			}
+
 			// Data subfilter.
 			if ($subfilters['data']) {
 				$subfilter_options['data'][$item['has_data'] ? 1 : 0]['count']++;
@@ -401,6 +435,7 @@ abstract class CControllerLatest extends CController {
 
 	/**
 	 * Collect available options of subfilter from existing items and hosts selected by primary filter.
+	 * All currently selected options will be included as well, regardless their presence in the retrieved data.
 	 *
 	 * @param array $data
 	 * @param array $data['hosts']                         Hosts selected by primary filter.
@@ -413,6 +448,7 @@ abstract class CControllerLatest extends CController {
 	 * @param array $subfilter['hostids']                  Selected subfilter hosts.
 	 * @param array $subfilter['tagnames']                 Selected subfilter names.
 	 * @param array $subfilter['tags']                     Selected subfilter tags.
+	 * @param array $subfilter['state']                    Selected subfilter state.
 	 * @param array $subfilter['data']                     Selected subfilter data options.
 	 *
 	 * @return array
@@ -424,6 +460,48 @@ abstract class CControllerLatest extends CController {
 			'tags' => []
 		];
 
+		// First, add currently selected options, regardless their presence in the retrieved data.
+
+		$missing_hostids = array_diff_key($subfilter['hostids'], $data['hosts']);
+
+		if ($missing_hostids) {
+			$missing_hosts = API::Host()->get([
+				'output' => ['name'],
+				'hostids' => array_keys($missing_hostids),
+				'preservekeys' => true
+			]);
+
+			foreach ($missing_hosts as $hostid => $host) {
+				$subfilter_options['hostids'][$hostid] = [
+					'name' => $host['name'],
+					'selected' => true,
+					'count' => 0
+				];
+			}
+		}
+
+		foreach (array_keys($subfilter['tagnames']) as $tagname) {
+			$subfilter_options['tagnames'][$tagname] = [
+				'name' => $tagname,
+				'selected' => true,
+				'items' => [],
+				'count' => 0
+			];
+		}
+
+		foreach ($subfilter['tags'] as $tag => $values) {
+			foreach (array_keys($values) as $value) {
+				$subfilter_options['tags'][$tag][$value] = [
+					'name' => $value,
+					'selected' => true,
+					'items' => [],
+					'count' => 0
+				];
+			}
+		}
+
+		// Second, add options represented by the selected data.
+
 		foreach ($data['hosts'] as $hostid => $host) {
 			$subfilter_options['hostids'][$hostid] = [
 				'name' => $host['name'],
@@ -441,8 +519,6 @@ abstract class CControllerLatest extends CController {
 						'items' => [],
 						'count' => 0
 					];
-
-					$subfilter_options['tags'][$tag['tag']] = [];
 				}
 
 				$subfilter_options['tags'][$tag['tag']][$tag['value']] = [
@@ -456,6 +532,19 @@ abstract class CControllerLatest extends CController {
 			}
 		}
 
+		$subfilter_options['state'] = [
+			ITEM_STATE_NORMAL => [
+				'name' => _('Normal'),
+				'selected' => array_key_exists(ITEM_STATE_NORMAL, $subfilter['state']),
+				'count' => 0
+			],
+			ITEM_STATE_NOTSUPPORTED => [
+				'name' => _('Not supported'),
+				'selected' => array_key_exists(ITEM_STATE_NOTSUPPORTED, $subfilter['state']),
+				'count' => 0
+			]
+		];
+
 		$subfilter_options['data'] = [
 			1 => [
 				'name' => _('With data'),
@@ -487,6 +576,7 @@ abstract class CControllerLatest extends CController {
 	 * @param array  $subfilter['hostids']                        Selected subfilter hosts.
 	 * @param array  $subfilter['tagnames']                       Selected subfilter tagnames.
 	 * @param array  $subfilter['tags']                           Selected subfilter tags.
+	 * @param array  $subfilter['state']                          Selected subfilter state.
 	 * @param array  $subfilter['data']                           Selected subfilter data options.
 	 *
 	 * @return array
@@ -523,6 +613,13 @@ abstract class CControllerLatest extends CController {
 				'tags' => $match_tags
 			];
 
+			if ($subfilter['state']) {
+				$item['matching_subfilters']['state'] = array_key_exists(ITEM_STATE_NORMAL, $subfilter['state'])
+					&& $item['state'] == ITEM_STATE_NORMAL
+					|| array_key_exists(ITEM_STATE_NOTSUPPORTED, $subfilter['state'])
+					&& $item['state'] == ITEM_STATE_NOTSUPPORTED;
+			}
+
 			if ($subfilter['data']) {
 				$item['has_data'] = array_key_exists($item['itemid'], $with_data);
 				$item['matching_subfilters']['data'] = array_key_exists(0, $subfilter['data']) && !$item['has_data']
@@ -545,6 +642,7 @@ abstract class CControllerLatest extends CController {
 	 * @param array|bool $items[]['matching_subfilters']['tags']      (optional) TRUE if item matches tagname/value
 	 *                                                                subfilter or array of exactly matching
 	 *                                                                tagname/value pairs.
+	 * @param bool       $items[]['matching_subfilters']['state']     (optional) TRUE if item matches state subfilter.
 	 * @param bool       $items[]['matching_subfilters']['data']      (optional) TRUE if item matches data subfilter.
 	 *
 	 * @return array
@@ -552,7 +650,7 @@ abstract class CControllerLatest extends CController {
 	protected static function applySubfilters(array $items): array {
 		return array_filter($items, function ($item) {
 			$matches = array_intersect_key($item['matching_subfilters'],
-				array_flip(['hostids', 'tagnames', 'tags', 'data'])
+				array_flip(['hostids', 'tagnames', 'tags', 'state', 'data'])
 			);
 
 			if (array_key_exists('tagnames', $matches)) {
@@ -663,4 +761,44 @@ abstract class CControllerLatest extends CController {
 
 		return $top_priority_fields;
 	}
+
+	/**
+	 * Check if at least one of mandatory filter fields is set.
+	 *
+	 * @param array $filter
+	 *
+	 * @return bool
+	 */
+	public static function isMandatoryFilterFieldSet(array $filter): bool {
+		$mandatory_field_defaults = array_intersect_key(
+			self::FILTER_FIELDS_DEFAULT, array_flip(self::MANDATORY_FILTER_FIELDS)
+		);
+
+		foreach ($mandatory_field_defaults as $field => $value) {
+			if ($filter[$field] != $value) {
+				return true;
+			}
+		}
+
+		return false;
+	}
+
+	/**
+	 * Check if at least one of subfilters is set.
+	 *
+	 * @param array $filter
+	 *
+	 * @return bool
+	 */
+	public static function isSubfilterSet(array $filter): bool {
+		$subfilters = ['subfilter_hostids','subfilter_tagnames','subfilter_tags','subfilter_state','subfilter_data'];
+
+		foreach ($subfilters as $subfilter) {
+			if ($filter[$subfilter]) {
+				return true;
+			}
+		}
+
+		return false;
+	}
 }
