@@ -1,26 +1,22 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 /**
  * @var CView $this
+ * @var array $data
  */
 
 $this->includeJsFile('administration.user.edit.common.js.php');
@@ -29,32 +25,47 @@ $this->includeJsFile(($data['action'] === 'user.edit')
 	: 'administration.userprofile.edit.js.php'
 );
 
-$widget = new CWidget();
+$html_page = new CHtmlPage();
 
 if ($data['action'] === 'user.edit') {
 	$widget_name = _('Users');
+	$doc_url = CDocHelper::USERS_USER_EDIT;
+	$csrf_token = CCsrfTokenHelper::get('user');
 }
 else {
 	$widget_name = _('User profile').NAME_DELIMITER;
 	$widget_name .= ($data['name'] !== '' || $data['surname'] !== '')
 		? $data['name'].' '.$data['surname']
 		: $data['username'];
-	$widget->setTitleSubmenu(getUserSettingsSubmenu());
+	$html_page->setTitleSubmenu(getUserSettingsSubmenu());
+	$doc_url = CDocHelper::USERS_USERPROFILE_EDIT;
+	$csrf_token = CCsrfTokenHelper::get('userprofile');
 }
 
-$widget->setTitle($widget_name);
+$html_page
+	->setTitle($widget_name)
+	->setDocUrl(CDocHelper::getUrl($doc_url));
+
 $tabs = new CTabView();
 
 if ($data['form_refresh'] == 0) {
 	$tabs->setSelected(0);
 }
 
+if ($data['readonly'] == true) {
+	CMessageHelper::addWarning(
+		_('This user is IdP provisioned. Manual changes for provisioned fields are not allowed.')
+	);
+	show_messages();
+}
+
 // Create form.
 $user_form = (new CForm())
 	->addItem((new CVar('form_refresh', $data['form_refresh'] + 1))->removeId())
+	->addItem((new CVar(CCsrfTokenHelper::CSRF_TOKEN_NAME, $csrf_token))->removeId())
 	->setId('user-form')
 	->setName('user_form')
-	->setAttribute('aria-labelledby', ZBX_STYLE_PAGE_TITLE)
+	->setAttribute('aria-labelledby', CHtmlPage::PAGE_TITLE_ID)
 	->addVar('action', $data['action'])
 	->addVar('userid', $data['userid']);
 
@@ -65,7 +76,7 @@ if ($data['action'] === 'user.edit') {
 	$user_form_list
 		->addRow((new CLabel(_('Username'), 'username'))->setAsteriskMark(),
 			(new CTextBox('username', $data['username']))
-				->setReadonly($data['db_user']['username'] === ZBX_GUEST_USER)
+				->setReadonly($data['db_user']['username'] === ZBX_GUEST_USER || $data['readonly'])
 				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 				->setAriaRequired()
 				->setAttribute('autofocus', 'autofocus')
@@ -75,18 +86,21 @@ if ($data['action'] === 'user.edit') {
 			(new CTextBox('name', $data['name']))
 				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 				->setAttribute('maxlength', DB::getFieldLength('users', 'name'))
+				->setReadonly($data['readonly'])
 		)
 		->addRow(_('Last name'),
 			(new CTextBox('surname', $data['surname']))
 				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 				->setAttribute('maxlength', DB::getFieldLength('users', 'surname'))
+				->setReadonly($data['readonly'])
 		)
 		->addRow(
-			(new CLabel(_('Groups'), 'user_groups__ms'))->setAsteriskMark(),
+			new CLabel(_('Groups'), 'user_groups__ms'),
 			(new CMultiSelect([
 				'name' => 'user_groups[]',
 				'object_name' => 'usersGroups',
 				'data' => $data['groups'],
+				'disabled' => $data['readonly'],
 				'popup' => [
 					'parameters' => [
 						'srctbl' => 'usrgrp',
@@ -104,14 +118,6 @@ if ($data['action'] === 'user.edit') {
 if ($data['change_password']) {
 	$user_form->disablePasswordAutofill();
 
-	$password1 = (new CPassBox('password1', $data['password1']))
-		->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
-		->setAriaRequired();
-
-	if ($data['action'] !== 'user.edit') {
-		$password1->setAttribute('autofocus', 'autofocus');
-	}
-
 	$password_requirements = [];
 
 	if ($data['password_requirements']['min_length'] > 1) {
@@ -154,29 +160,58 @@ if ($data['change_password']) {
 		])
 		: null;
 
+	$current_password = (new CPassBox('current_password'))
+		->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+		->setAriaRequired()
+		->setAttribute('autocomplete', 'off');
+
+	if ($data['action'] !== 'user.edit') {
+		$current_password->setAttribute('autofocus', 'autofocus');
+	}
+
+	if (CWebUser::$data['userid'] == $data['userid']) {
+		$user_form_list
+			->addRow((new CLabel(_('Current password'), 'current_password'))->setAsteriskMark(), $current_password);
+	}
+
 	$user_form_list
 		->addRow((new CLabel([_('Password'), $password_hint_icon], 'password1'))->setAsteriskMark(), [
 			// Hidden dummy login field for protection against chrome error when password autocomplete.
 			(new CInput('text', null, null))
 				->setAttribute('tabindex', '-1')
 				->addStyle('position: absolute; left: -100vw;'),
-			$password1
+			(new CPassBox('password1', $data['password1']))
+				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+				->setAriaRequired()
+				->setAttribute('autocomplete', 'off')
 		])
 		->addRow((new CLabel(_('Password (once again)'), 'password2'))->setAsteriskMark(),
 			(new CPassBox('password2', $data['password2']))
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 				->setAriaRequired()
+				->setAttribute('autocomplete', 'off')
 		)
 		->addRow('', _('Password is not mandatory for non internal authentication type.'));
 }
 else {
-	$user_form_list->addRow(_('Password'),
+	$change_password_enabled = !$data['readonly'] && $data['internal_auth'];
+
+	if ($change_password_enabled && $data['action'] === 'user.edit') {
+		$change_password_enabled = $data['db_user']['username'] !== ZBX_GUEST_USER;
+	}
+
+	$hint = !$change_password_enabled
+		? $hint = makeErrorIcon(_('Password can only be changed for users using the internal Zabbix authentication.'))
+		: null;
+
+	$user_form_list->addRow(_('Password'), [
 		(new CSimpleButton(_('Change password')))
-			->setEnabled($data['action'] === 'userprofile.edit' || $data['db_user']['username'] !== ZBX_GUEST_USER)
+			->setEnabled($change_password_enabled)
 			->setAttribute('autofocus', 'autofocus')
-			->onClick('javascript: submitFormWithParam("'.$user_form->getName().'", "change_password", "1");')
-			->addClass(ZBX_STYLE_BTN_GREY)
-	);
+			->onClick('submitFormWithParam("'.$user_form->getName().'", "change_password", "1");')
+			->addClass(ZBX_STYLE_BTN_GREY),
+		$hint
+	]);
 }
 
 // Append languages, timezones & themes to form list.
@@ -229,15 +264,14 @@ else {
 	setlocale(LC_MONETARY, zbx_locale_variants(CWebUser::$data['lang']));
 
 	if (!function_exists('bindtextdomain')) {
-		$language_error = 'Translations are unavailable because the PHP gettext module is missing.';
+		$language_error = makeErrorIcon('Translations are unavailable because the PHP gettext module is missing.');
+
 		$lang_select->setReadonly();
 	}
 	elseif (!$all_locales_available) {
-		$language_error = _('You are not able to choose some of the languages, because locales for them are not installed on the web server.');
-	}
-
-	if ($language_error) {
-		$language_error = (makeErrorIcon($language_error))->addStyle('margin-left: 5px;');
+		$language_error = makeWarningIcon(
+			_('You are not able to choose some of the languages, because locales for them are not installed on the web server.')
+		);
 	}
 
 	$timezone_select
@@ -277,14 +311,13 @@ $user_form_list
 			->setAriaRequired()
 	)
 	->addRow((new CLabel(_('Rows per page'), 'rows_per_page'))->setAsteriskMark(),
-		(new CNumericBox('rows_per_page', $data['rows_per_page'], 6))
+		(new CNumericBox('rows_per_page', $data['rows_per_page'], 6, false, false, false))
 			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 			->setAriaRequired()
 	)
 	->addRow(_('URL (after login)'),
-		(new CTextBox('url', $data['url']))
+		(new CTextBox('url', $data['url'], false, DB::getFieldLength('users', 'url')))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAttribute('maxlength', DB::getFieldLength('users', 'url'))
 	);
 
 $tabs->addTab('userTab', _('User'), $user_form_list);
@@ -300,17 +333,26 @@ if ($data['action'] === 'user.edit' || CWebUser::$data['type'] > USER_TYPE_ZABBI
 		->setHeader([_('Type'), _('Send to'), _('When active'), _('Use if severity'), _('Status'), _('Action')]);
 
 	foreach ($data['medias'] as $index => $media) {
-		if ($media['active'] == MEDIA_STATUS_ACTIVE) {
-			$status = (new CLink(_('Enabled'), '#'))
-				->onClick('return create_var("'.$user_form->getName().'","disable_media",'.$index.', true);')
-				->addClass(ZBX_STYLE_LINK_ACTION)
-				->addClass(ZBX_STYLE_GREEN);
+		if ($data['mediatypes'][$media['mediatypeid']]['status'] == MEDIA_TYPE_STATUS_ACTIVE) {
+			$media_name = $media['name'];
+
+			if ($media['active'] == MEDIA_STATUS_ACTIVE) {
+				$status = (new CButtonLink(_('Enabled')))
+					->onClick('return create_var("'.$user_form->getName().'","disable_media",'.$index.', true);')
+					->addClass(ZBX_STYLE_GREEN);
+			}
+			else {
+				$status = (new CButtonLink(_('Disabled')))
+					->onClick('return create_var("'.$user_form->getName().'","enable_media",'.$index.', true);')
+					->addClass(ZBX_STYLE_RED);
+			}
 		}
 		else {
-			$status = (new CLink(_('Disabled'), '#'))
-				->onClick('return create_var("'.$user_form->getName().'","enable_media",'.$index.', true);')
-				->addClass(ZBX_STYLE_LINK_ACTION)
-				->addClass(ZBX_STYLE_RED);
+			$media_name = [
+				new CSpan($media['name']),
+				makeWarningIcon(_('Media type disabled by Administration.'))
+			];
+			$status = (new CSpan(_('Disabled')))->addClass(ZBX_STYLE_RED);
 		}
 
 		$parameters = [
@@ -320,10 +362,15 @@ if ($data['action'] === 'user.edit' || CWebUser::$data['type'] > USER_TYPE_ZABBI
 			($media['mediatype'] == MEDIA_TYPE_EMAIL) ? 'sendto_emails' : 'sendto' => $media['sendto'],
 			'period' => $media['period'],
 			'severity' => $media['severity'],
-			'active' => $media['active']
+			'active' => $media['active'],
+			'userdirectory_mediaid' => $media['userdirectory_mediaid']
 		];
 		$media_severity = [];
 
+		if (array_key_exists('mediaid', $media)) {
+			$parameters['mediaid'] = $media['mediaid'];
+		}
+
 		for ($severity = TRIGGER_SEVERITY_NOT_CLASSIFIED; $severity < TRIGGER_SEVERITY_COUNT; $severity++) {
 			$severity_name = CSeverityHelper::getName($severity);
 
@@ -333,7 +380,7 @@ if ($data['action'] === 'user.edit' || CWebUser::$data['type'] > USER_TYPE_ZABBI
 				->setHint($severity_name.' ('.($media_active ? _('on') : _('off')).')', '', false)
 				->addClass($media_active
 					? CSeverityHelper::getStatusStyle($severity)
-					: ZBX_STYLE_STATUS_DISABLED_BG
+					: ZBX_STYLE_STATUS_DISABLED
 				);
 		}
 
@@ -347,7 +394,7 @@ if ($data['action'] === 'user.edit' || CWebUser::$data['type'] > USER_TYPE_ZABBI
 
 		$media_table_info->addRow(
 			(new CRow([
-				$media['name'],
+				$media_name,
 				$media['sendto'],
 				(new CDiv($media['period']))
 					->setAttribute('style', 'max-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
@@ -356,17 +403,14 @@ if ($data['action'] === 'user.edit' || CWebUser::$data['type'] > USER_TYPE_ZABBI
 				$status,
 				(new CCol(
 					new CHorList([
-						(new CButton(null, _('Edit')))
-							->addClass(ZBX_STYLE_BTN_LINK)
-							->onClick(
-								'return PopUp("popup.media", '.
-									json_encode($parameters).
-									', {dialogue_class: "modal-popup-generic"}
-								);'
+						(new CButtonLink(_('Edit')))
+							->setAttribute('data-parameters', json_encode($parameters))
+							->onClick('PopUp("popup.media", JSON.parse(this.dataset.parameters),
+								{dialogue_class: "modal-popup-generic"});'
 							),
-						(new CButton(null, _('Remove')))
-							->addClass(ZBX_STYLE_BTN_LINK)
-							->onClick('javascript: removeMedia('.$index.');')
+						(new CButtonLink(_('Remove')))
+							->setEnabled(!$parameters['userdirectory_mediaid'])
+							->onClick('removeMedia('.$index.');')
 					])
 				))->addClass(ZBX_STYLE_NOWRAP)
 			]))->setId('medias_'.$index)
@@ -376,14 +420,11 @@ if ($data['action'] === 'user.edit' || CWebUser::$data['type'] > USER_TYPE_ZABBI
 	$media_form_list->addRow(_('Media'),
 		(new CDiv([
 			$media_table_info,
-			(new CButton(null, _('Add')))
-				->onClick(
-					'return PopUp("popup.media", '.
-						json_encode(['dstfrm' => $user_form->getName()]).
-						', {dialogue_class: "modal-popup-generic"}
-					);'
+			(new CButtonLink(_('Add')))
+				->onClick('PopUp("popup.media", '.
+					json_encode(['dstfrm' => $user_form->getName()]).
+					', {dialogue_class: "modal-popup-generic"});'
 				)
-				->addClass(ZBX_STYLE_BTN_LINK)
 		]))
 			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
@@ -396,12 +437,15 @@ if ($data['action'] === 'user.edit' || CWebUser::$data['type'] > USER_TYPE_ZABBI
 if ($data['action'] === 'user.edit') {
 	$permissions_form_list = new CFormList('permissionsFormList');
 
+	$role_disabled = $data['userid'] != 0 && bccomp(CWebUser::$data['userid'], $data['userid']) == 0;
+	$role_disabled |= $data['readonly'];
+
 	$role_multiselect = (new CMultiSelect([
 		'name' => 'roleid',
 		'object_name' => 'roles',
 		'data' => $data['role'],
 		'multiple' => false,
-		'disabled' => $data['userid'] != 0 && bccomp(CWebUser::$data['userid'], $data['userid']) == 0,
+		'disabled' => $role_disabled,
 		'popup' => [
 			'parameters' => [
 				'srctbl' => 'roles',
@@ -413,7 +457,7 @@ if ($data['action'] === 'user.edit') {
 	]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH);
 
 	if ($data['userid'] != 0 && bccomp(CWebUser::$data['userid'], $data['userid']) == 0) {
-		$permissions_form_list->addRow((new CLabel(_('Role')))->setAsteriskMark(),
+		$permissions_form_list->addRow(new CLabel(_('Role')),
 			(new CDiv([
 				$role_multiselect,
 				new CDiv(_('User cannot change own role.'))
@@ -423,7 +467,9 @@ if ($data['action'] === 'user.edit') {
 		);
 	}
 	else {
-		$permissions_form_list->addRow((new CLabel(_('Role'), 'roleid_ms'))->setAsteriskMark(), $role_multiselect);
+		$permissions_form_list->addRow((new CLabel(_('Role'), 'roleid_ms'))->setAsteriskMark($data['roleid_required']),
+			$role_multiselect
+		);
 	}
 
 	if ($data['roleid']) {
@@ -433,10 +479,11 @@ if ($data['action'] === 'user.edit') {
 
 		$permissions_table = (new CTable())
 			->setAttribute('style', 'width: 100%;')
-			->setHeader([_('Host group'), _('Permissions')]);
+			->setHeader([_('Group'), _('Type'), _('Permissions')]);
 
 		if ($data['user_type'] == USER_TYPE_SUPER_ADMIN) {
-			$permissions_table->addRow([italic(_('All groups')), permissionText(PERM_READ_WRITE)]);
+			$permissions_table->addRow([italic(_('All groups')), _('Hosts'), permissionText(PERM_READ_WRITE)]);
+			$permissions_table->addRow([italic(_('All groups')), _('Templates'), permissionText(PERM_READ_WRITE)]);
 		}
 		else {
 			foreach ($data['groups_rights'] as $groupid => $group_rights) {
@@ -448,7 +495,21 @@ if ($data['action'] === 'user.edit') {
 				else {
 					$group_name = $group_rights['name'];
 				}
-				$permissions_table->addRow([$group_name, permissionText($group_rights['permission'])]);
+
+				$permissions_table->addRow([$group_name, _('Hosts'), permissionText($group_rights['permission'])]);
+			}
+
+			foreach ($data['templategroups_rights'] as $groupid => $group_rights) {
+				if (array_key_exists('grouped', $group_rights) && $group_rights['grouped']) {
+					$group_name = ($groupid == 0)
+						? italic(_('All groups'))
+						: [$group_rights['name'], NBSP(), italic('('._('including subgroups').')')];
+				}
+				else {
+					$group_name = $group_rights['name'];
+				}
+
+				$permissions_table->addRow([$group_name, _('Templates'), permissionText($group_rights['permission'])]);
 			}
 		}
 
@@ -590,11 +651,12 @@ if ($data['action'] === 'user.edit') {
 		else {
 			$elements = [];
 
-			foreach ($data['modules'] as $moduleid => $module) {
-				$elements[] = (new CSpan($module['id']))->addClass(
-					CRoleHelper::checkAccess('modules.module.'.$moduleid, $data['roleid'])
-						? ZBX_STYLE_STATUS_GREEN
-						: ZBX_STYLE_STATUS_GREY
+			foreach ($data['modules'] as $moduleid => $module_name) {
+				$elements[] = (new CSpan($module_name))->addClass(
+					array_key_exists($moduleid, $data['disabled_moduleids'])
+							|| $data['modules_rules'][$moduleid] == MODULE_STATUS_DISABLED
+						? ZBX_STYLE_STATUS_GREY
+						: ZBX_STYLE_STATUS_GREEN
 				);
 			}
 
@@ -659,10 +721,10 @@ if ($data['action'] === 'user.edit') {
 	$tabs->addTab('permissionsTab', _('Permissions'), $permissions_form_list);
 }
 
-// Messaging tab.
+// Frontend notifications tab.
 if ($data['action'] !== 'user.edit') {
 	$messaging_form_list = (new CFormList())
-		->addRow(_('Frontend messaging'),
+		->addRow(_('Frontend notifications'),
 			(new CCheckBox('messages[enabled]'))
 				->setChecked($data['messages']['enabled'] == 1)
 				->setUncheckedValue(0)
@@ -700,12 +762,12 @@ if ($data['action'] !== 'user.edit') {
 				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 				(new CButton('start', _('Play')))
 					->addClass(ZBX_STYLE_BTN_GREY)
-					->onClick("javascript: testUserSound('messages_sounds.recovery');")
+					->onClick("testUserSound('messages_sounds.recovery');")
 					->removeId(),
 				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 				(new CButton('stop', _('Stop')))
 					->addClass(ZBX_STYLE_BTN_GREY)
-					->onClick('javascript: AudioControl.stop();')
+					->onClick('AudioControl.stop();')
 					->removeId()
 			]
 		]);
@@ -735,12 +797,12 @@ if ($data['action'] !== 'user.edit') {
 				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 				(new CButton('start', _('Play')))
 					->addClass(ZBX_STYLE_BTN_GREY)
-					->onClick("javascript: testUserSound('messages_sounds.".$severity."');")
+					->onClick("testUserSound('messages_sounds.".$severity."');")
 					->removeId(),
 				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 				(new CButton('stop', _('Stop')))
 					->addClass(ZBX_STYLE_BTN_GREY)
-					->onClick('javascript: AudioControl.stop();')
+					->onClick('AudioControl.stop();')
 					->removeId()
 			]
 		]);
@@ -757,7 +819,9 @@ if ($data['action'] !== 'user.edit') {
 				->setUncheckedValue(ZBX_PROBLEM_SUPPRESSED_FALSE)
 		);
 
-	$tabs->addTab('messagingTab', _('Messaging'), $messaging_form_list, TAB_INDICATOR_FRONTEND_MESSAGE);
+	$tabs->addTab('notificationsTab', _('Frontend notifications'), $messaging_form_list,
+		TAB_INDICATOR_FRONTEND_NOTIFICATIONS
+	);
 }
 
 // Append buttons to form.
@@ -771,8 +835,10 @@ if ($data['action'] === 'user.edit') {
 		$tabs->setFooter(makeFormFooter(
 			(new CSubmitButton(_('Update'), 'action', 'user.update'))->setId('update'),
 			[
-				(new CRedirectButton(_('Delete'),
-					'zabbix.php?action=user.delete&sid='.$data['sid'].'&userids[]='.$data['userid'],
+				(new CRedirectButton(_('Delete'), (new CUrl('zabbix.php'))
+					->setArgument('action', 'user.delete')
+					->setArgument('userids', [$data['userid']])
+					->setArgument(CCsrfTokenHelper::CSRF_TOKEN_NAME, $csrf_token),
 					_('Delete selected user?')
 				))
 					->setEnabled(bccomp(CWebUser::$data['userid'], $data['userid']) != 0)
@@ -799,6 +865,19 @@ else {
 
 // Append tab to form.
 $user_form->addItem($tabs);
-$widget
+$html_page
 	->addItem($user_form)
 	->show();
+
+if ($data['action'] === 'user.edit') {
+	(new CScriptTag('view.init('.json_encode([
+		'userid' => $data['userid'] ?: null
+	]).');'))
+		->setOnDocumentReady()
+		->show();
+}
+else {
+	(new CScriptTag('view.init();'))
+		->setOnDocumentReady()
+		->show();
+}
