@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -47,7 +42,7 @@ function getSystemStatusData(array $filter) {
 		? $filter['ext_ack']
 		: EXTACK_OPTION_ALL;
 	$filter_evaltype = array_key_exists('evaltype', $filter) ? $filter['evaltype'] : TAG_EVAL_TYPE_AND_OR;
-	$filter_tags = array_key_exists('tags', $filter) ? $filter['tags'] : [];
+	$filter_tags = array_key_exists('tags', $filter) && $filter['tags'] ? $filter['tags'] : null;
 	$show_opdata = array_key_exists('show_opdata', $filter) && $filter['show_opdata'] != OPERATIONAL_DATA_SHOW_NONE;
 
 	if (array_key_exists('exclude_groupids', $filter) && $filter['exclude_groupids']) {
@@ -56,7 +51,7 @@ function getSystemStatusData(array $filter) {
 			if ($filter_groupids === null) {
 				$filter_groupids = array_keys(API::HostGroup()->get([
 					'output' => [],
-					'real_hosts' => true,
+					'with_hosts' => true,
 					'preservekeys' => true
 				]));
 			}
@@ -85,7 +80,7 @@ function getSystemStatusData(array $filter) {
 			'output' => ['groupid', 'name'],
 			'groupids' => $filter_groupids,
 			'hostids' => $filter_hostids,
-			'monitored_hosts' => true,
+			'with_monitored_hosts' => true,
 			'preservekeys' => true
 		]),
 		'triggers' => [],
@@ -96,7 +91,8 @@ function getSystemStatusData(array $filter) {
 			'add_comments' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS),
 			'change_severity' => CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY),
 			'acknowledge' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS),
-			'close' => CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+			'close' => CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS),
+			'suppress' => CWebUser::checkAccess(CRoleHelper::ACTIONS_SUPPRESS_PROBLEMS)
 		]
 	];
 
@@ -120,7 +116,7 @@ function getSystemStatusData(array $filter) {
 
 	$options = [
 		'output' => ['eventid', 'r_eventid', 'objectid', 'clock', 'ns', 'name', 'acknowledged', 'severity'],
-		'selectAcknowledges' => ['action'],
+		'selectAcknowledges' => ['action', 'clock', 'userid'],
 		'groupids' => array_keys($data['groups']),
 		'hostids' => $filter_hostids,
 		'evaltype' => $filter_evaltype,
@@ -128,23 +124,19 @@ function getSystemStatusData(array $filter) {
 		'source' => EVENT_SOURCE_TRIGGERS,
 		'object' => EVENT_OBJECT_TRIGGER,
 		'suppressed' => false,
+		'symptom' => false,
 		'sortfield' => ['eventid'],
 		'sortorder' => ZBX_SORT_DOWN,
 		'preservekeys' => true
 	];
 
-	if (array_key_exists('severities', $filter)) {
-		$filter_severities = implode(',', $filter['severities']);
-		$all_severities = implode(',', range(TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_COUNT - 1));
-
-		if ($filter_severities !== '' && $filter_severities !== $all_severities) {
-			$options['severities'] = $filter['severities'];
-		}
+	if (array_key_exists('severities', $filter) && $filter['severities']) {
+		$options['severities'] = $filter['severities'];
 	}
 
 	if (array_key_exists('show_suppressed', $filter) && $filter['show_suppressed']) {
 		unset($options['suppressed']);
-		$options['selectSuppressionData'] = ['maintenanceid', 'suppress_until'];
+		$options['selectSuppressionData'] = ['maintenanceid', 'suppress_until', 'userid'];
 	}
 
 	if ($filter_ext_ack == EXTACK_OPTION_UNACK) {
@@ -165,7 +157,7 @@ function getSystemStatusData(array $filter) {
 
 		$options = [
 			'output' => ['priority', 'manual_close'],
-			'selectGroups' => ['groupid'],
+			'selectHostGroups' => ['groupid'],
 			'selectHosts' => ['name'],
 			'triggerids' => array_keys($triggerids),
 			'monitored' => true,
@@ -183,13 +175,13 @@ function getSystemStatusData(array $filter) {
 		$data['triggers'] = API::Trigger()->get($options);
 
 		if ($show_opdata && $data['triggers']) {
-			$items = API::Item()->get([
-				'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type', 'units'],
+			$items = CArrayHelper::renameObjectsKeys(API::Item()->get([
+				'output' => ['itemid', 'hostid', 'name_resolved', 'key_', 'value_type', 'units'],
 				'selectValueMap' => ['mappings'],
 				'triggerids' => array_keys($data['triggers']),
 				'webitems' => true,
 				'preservekeys' => true
-			]);
+			]), ['name_resolved' => 'name']);
 
 			foreach ($data['triggers'] as &$trigger) {
 				foreach ($trigger['functions'] as $function) {
@@ -222,7 +214,7 @@ function getSystemStatusData(array $filter) {
 			}
 
 			// groups
-			foreach ($trigger['groups'] as $trigger_group) {
+			foreach ($trigger['hostgroups'] as $trigger_group) {
 				if (!array_key_exists($trigger_group['groupid'], $data['groups'])) {
 					continue;
 				}
@@ -256,9 +248,11 @@ function getSystemStatusData(array $filter) {
 		// actions & tags
 		$problems_data = API::Problem()->get([
 			'output' => ['eventid', 'r_eventid', 'clock', 'objectid', 'severity'],
-			'eventids' => array_keys($visible_problems),
-			'selectAcknowledges' => ['userid', 'clock', 'message', 'action', 'old_severity', 'new_severity'],
+			'selectAcknowledges' => ['userid', 'clock', 'message', 'action', 'old_severity', 'new_severity',
+				'suppress_until'
+			],
 			'selectTags' => ['tag', 'value'],
+			'eventids' => array_keys($visible_problems),
 			'preservekeys' => true
 		]);
 
@@ -369,6 +363,7 @@ function getSystemStatusTotals(array $data) {
  * @param bool  $data['allowed']['change_severity']
  * @param bool  $data['allowed']['acknowledge']
  * @param bool  $data['allowed']['close']
+ * @param bool  $data['allowed']['suppress']
  * @param bool  $hide_empty_groups
  * @param CUrl  $groupurl
  *
@@ -400,7 +395,9 @@ function makeSeverityTable(array $data, $hide_empty_groups = false, CUrl $groupu
 			$row[] = getSeverityTableCell($severity, $data, $stat);
 		}
 
-		$table->addRow($row);
+		$table->addRow(
+			(new CRow($row))->setAttribute('data-hostgroupid', $group['groupid'])
+		);
 	}
 
 	return $table;
@@ -419,6 +416,7 @@ function makeSeverityTable(array $data, $hide_empty_groups = false, CUrl $groupu
  * @param bool  $data['allowed']['change_severity']
  * @param bool  $data['allowed']['acknowledge']
  * @param bool  $data['allowed']['close']
+ * @param bool  $data['allowed']['suppress']
  *
  * @return CDiv
  */
@@ -453,6 +451,7 @@ function makeSeverityTotals(array $data) {
  * @param bool  $data['allowed']['change_severity']
  * @param bool  $data['allowed']['acknowledge']
  * @param bool  $data['allowed']['close']
+ * @param bool  $data['allowed']['suppress']
  * @param array $stat
  * @param int   $stat['count']
  * @param array $stat['problems']
@@ -545,6 +544,7 @@ function getSeverityTableCell($severity, array $data, array $stat, $is_total = f
  * @param bool   $allowed['change_severity']
  * @param bool   $allowed['acknowledge']
  * @param bool   $allowed['close']
+ * @param bool   $allowed['suppress']
  *
  * @return CTableInfo
  */
@@ -578,13 +578,19 @@ function makeProblemsPopup(array $problems, array $triggers, array $actions, arr
 			_('Problem'),
 			($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? _('Operational data') : null,
 			_('Duration'),
-			_('Ack'),
+			_('Update'),
 			_('Actions'),
 			_('Tags')
 		]));
 
+	$data = [
+		'last_clock' => 0,
+		'sortorder' => ZBX_SORT_DOWN,
+		'show_three_columns' => false,
+		'show_two_columns' => false
+	];
+
 	$today = strtotime('today');
-	$last_clock = 0;
 
 	// Unset triggers, which missing in problems array.
 	if ($problems) {
@@ -603,7 +609,7 @@ function makeProblemsPopup(array $problems, array $triggers, array $actions, arr
 	$tags = makeTags($problems);
 
 	if (array_key_exists('show_suppressed', $filter) && $filter['show_suppressed']) {
-		CScreenProblem::addMaintenanceNames($problems);
+		CScreenProblem::addSuppressionNames($problems);
 	}
 
 	foreach ($problems as $problem) {
@@ -624,10 +630,10 @@ function makeProblemsPopup(array $problems, array $triggers, array $actions, arr
 		}
 
 		if ($show_timeline) {
-			if ($last_clock != 0) {
-				CScreenProblem::addTimelineBreakpoint($table, $last_clock, $problem['clock'], ZBX_SORT_DOWN);
+			if ($data['last_clock'] != 0) {
+				CScreenProblem::addTimelineBreakpoint($table, $data, $problem, false, false);
 			}
-			$last_clock = $problem['clock'];
+			$data['last_clock'] = $problem['clock'];
 
 			$row = [
 				$cell_clock->addClass(ZBX_STYLE_TIMELINE_DATE),
@@ -646,8 +652,32 @@ function makeProblemsPopup(array $problems, array $triggers, array $actions, arr
 		}
 
 		$info_icons = [];
-		if (array_key_exists('suppression_data', $problem) && $problem['suppression_data']) {
-			$info_icons[] = makeSuppressedProblemIcon($problem['suppression_data']);
+		if (array_key_exists('suppression_data', $problem)) {
+			if (count($problem['suppression_data']) == 1
+					&& $problem['suppression_data'][0]['maintenanceid'] == 0
+					&& isEventRecentlyUnsuppressed($problem['acknowledges'], $unsuppression_action)) {
+				// Show blinking button if the last manual suppression was recently revoked.
+				$user_unsuppressed = array_key_exists($unsuppression_action['userid'], $actions['users'])
+					? getUserFullname($actions['users'][$unsuppression_action['userid']])
+					: _('Inaccessible user');
+
+				$info_icons[] = (new CButtonIcon(ZBX_ICON_EYE))
+					->addClass(ZBX_STYLE_COLOR_ICON)
+					->addClass('js-blink')
+					->setHint(_s('Unsuppressed by: %1$s', $user_unsuppressed));
+			}
+			elseif ($problem['suppression_data']) {
+				$info_icons[] = makeSuppressedProblemIcon($problem['suppression_data'], false);
+			}
+			elseif (isEventRecentlySuppressed($problem['acknowledges'], $suppression_action)) {
+				// Show blinking button if suppression was made but is not yet processed by server.
+				$info_icons[] = makeSuppressedProblemIcon([[
+					'suppress_until' => $suppression_action['suppress_until'],
+					'username' => array_key_exists($suppression_action['userid'], $actions['users'])
+						? getUserFullname($actions['users'][$suppression_action['userid']])
+						: _('Inaccessible user')
+				]], true);
+			}
 		}
 
 		// operational data
@@ -699,14 +729,12 @@ function makeProblemsPopup(array $problems, array $triggers, array $actions, arr
 		// Create acknowledge link.
 		$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 		$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
-				|| $can_be_closed)
-			? (new CLink($is_acknowledged ? _('Yes') : _('No')))
-				->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+				|| $can_be_closed || $allowed['suppress'])
+			? (new CLink(_('Update')))
 				->addClass(ZBX_STYLE_LINK_ALT)
-				->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);')
-			: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
-				$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
-			);
+				->setAttribute('data-eventid', $problem['eventid'])
+				->onClick('acknowledgePopUp({eventids: [this.dataset.eventid]}, this);')
+			: new CSpan(_('Update'));
 
 		$table->addRow(array_merge($row, [
 			makeInformationList($info_icons),
@@ -720,7 +748,7 @@ function makeProblemsPopup(array $problems, array $triggers, array $actions, arr
 			($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? $opdata : null,
 			zbx_date2age($problem['clock']),
 			$problem_update_link,
-			makeEventActionsIcons($problem['eventid'], $actions['all_actions'], $actions['users']),
+			makeEventActionsIcons($problem['eventid'], $actions['all_actions'], $actions['users'], $is_acknowledged),
 			$tags[$problem['eventid']]
 		]));
 	}
