@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -143,6 +138,13 @@ class API {
 		return self::getApi('configuration');
 	}
 
+	/**
+	 * @return CConnector
+	 */
+	public static function Connector() {
+		return self::getApi('connector');
+	}
+
 	/**
 	 * @return CCorrelation
 	 */
@@ -325,6 +327,13 @@ class API {
 		return self::getApi('mediatype');
 	}
 
+	/**
+	 * @return CMfa
+	 */
+	public static function Mfa() {
+		return self::getApi('mfa');
+	}
+
 	/**
 	 * @return CProblem
 	 */
@@ -339,6 +348,13 @@ class API {
 		return self::getApi('proxy');
 	}
 
+	/**
+	 * @return CProxyGroup
+	 */
+	public static function ProxyGroup() {
+		return self::getApi('proxygroup');
+	}
+
 	/**
 	 * @return CReport
 	 */
@@ -409,6 +425,13 @@ class API {
 		return self::getApi('templatedashboard');
 	}
 
+	/**
+	 * @return CTemplateGroup
+	 */
+	public static function TemplateGroup() {
+		return self::getApi('templategroup');
+	}
+
 	/**
 	 * @return CToken
 	 */
@@ -444,6 +467,13 @@ class API {
 		return self::getApi('user');
 	}
 
+	/**
+	 * @return CUserDirectory
+	 */
+	public static function UserDirectory() {
+		return self::getApi('userdirectory');
+	}
+
 	/**
 	 * @return CUserGroup
 	 */
