@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -63,14 +58,15 @@ class CConfiguration extends CApiService {
 	private function validateExport(array &$params, bool $with_unlinked_parent_templates = false): void {
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
 			'format' =>			['type' => API_STRING_UTF8, 'flags' => API_REQUIRED, 'in' => implode(',', [CExportWriterFactory::YAML, CExportWriterFactory::XML, CExportWriterFactory::JSON, CExportWriterFactory::RAW])],
-			'prettyprint' => 	['type' => API_BOOLEAN, 'default' => false],
+			'prettyprint' =>	['type' => API_BOOLEAN, 'default' => false],
 			'options' =>		['type' => API_OBJECT, 'flags' => API_REQUIRED, 'fields' => [
-				'groups' =>			['type' => API_IDS],
-				'hosts' =>			['type' => API_IDS],
-				'images' =>			['type' => API_IDS],
-				'maps' =>			['type' => API_IDS],
-				'mediaTypes' =>		['type' => API_IDS],
-				'templates' =>		['type' => API_IDS]
+				'hosts' =>				['type' => API_IDS],
+				'images' =>				['type' => API_IDS],
+				'maps' =>				['type' => API_IDS],
+				'mediaTypes' =>			['type' => API_IDS],
+				'template_groups' =>	['type' => API_IDS],
+				'host_groups' => 		['type' => API_IDS],
+				'templates' =>			['type' => API_IDS]
 			]]
 		]];
 
@@ -128,11 +124,11 @@ class CConfiguration extends CApiService {
 	/**
 	 * Validate input parameters for import() and importcompare() methods.
 	 *
-	 * @param type $params
+	 * @param array $params
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	protected function validateImport($params): void {
+	protected function validateImport(array &$params): void {
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
 			'format' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED, 'in' => implode(',', [CImportReaderFactory::YAML, CImportReaderFactory::XML, CImportReaderFactory::JSON])],
 			'source' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED],
@@ -147,7 +143,11 @@ class CConfiguration extends CApiService {
 					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false],
 					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
-				'groups' =>				['type' => API_OBJECT, 'fields' => [
+				'host_groups' =>		['type' => API_OBJECT, 'fields' => [
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
+				]],
+				'template_groups' =>	['type' => API_OBJECT, 'fields' => [
 					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
 					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
@@ -177,7 +177,7 @@ class CConfiguration extends CApiService {
 					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
 					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
-				'templateLinkage' =>	['type' => API_OBJECT, 'fields' => [
+				'templateLinkage' =>	['type' => API_OBJECT, 'default' => [], 'fields' => [
 					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
 					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
@@ -206,6 +206,10 @@ class CConfiguration extends CApiService {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
+		if (mb_substr($params['source'], 0, 1) === pack('H*', 'EFBBBF')) {
+			$params['source'] = mb_substr($params['source'], 1);
+		}
+
 		if (array_key_exists('maps', $params['rules']) && !self::checkAccess(CRoleHelper::ACTIONS_EDIT_MAPS)
 				&& ($params['rules']['maps']['createMissing'] || $params['rules']['maps']['updateExisting'])) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'rules',
@@ -248,7 +252,7 @@ class CConfiguration extends CApiService {
 			->setStrict(true)
 			->validate($data, '/');
 
-		foreach (['1.0', '2.0', '3.0', '3.2', '3.4', '4.0', '4.2', '4.4', '5.0', '5.2', '5.4'] as $version) {
+		foreach ($import_converter_factory::getSequentialVersions() as $version) {
 			if ($data['zabbix_export']['version'] !== $version) {
 				continue;
 			}
@@ -315,7 +319,7 @@ class CConfiguration extends CApiService {
 			->setPreview(true)
 			->validate($data, '/');
 
-		foreach (['1.0', '2.0', '3.0', '3.2', '3.4', '4.0', '4.2', '4.4', '5.0', '5.2', '5.4'] as $version) {
+		foreach ($import_converter_factory::getSequentialVersions() as $version) {
 			if ($data['zabbix_export']['version'] !== $version) {
 				continue;
 			}
@@ -343,10 +347,12 @@ class CConfiguration extends CApiService {
 		$adapter->load($data);
 
 		$import = $adapter->getData();
+
 		$imported_entities = [];
 
 		$entities = [
-			'groups' => 'name',
+			'host_groups' => 'name',
+			'template_groups' => 'name',
 			'templates' => 'template'
 		];
 
@@ -358,13 +364,12 @@ class CConfiguration extends CApiService {
 		}
 
 		$imported_ids = [];
-		$db_templates = [];
 
 		foreach ($imported_entities as $entity => $data) {
 			switch ($entity) {
-				case 'groups':
-					$imported_ids['groups'] = API::HostGroup()->get([
-						'output' => ['groupid'],
+				case 'host_groups':
+					$imported_ids['host_groups'] = API::HostGroup()->get([
+						'output' => [],
 						'filter' => [
 							'uuid' => $data['uuid'],
 							'name' => $data['name']
@@ -373,13 +378,26 @@ class CConfiguration extends CApiService {
 						'searchByAny' => true
 					]);
 
-					$imported_ids['groups'] = array_keys($imported_ids['groups']);
+					$imported_ids['host_groups'] = array_keys($imported_ids['host_groups']);
+					break;
+
+				case 'template_groups':
+					$imported_ids['template_groups'] = API::TemplateGroup()->get([
+						'output' => [],
+						'filter' => [
+							'uuid' => $data['uuid'],
+							'name' => $data['name']
+						],
+						'preservekeys' => true,
+						'searchByAny' => true
+					]);
+
+					$imported_ids['template_groups'] = array_keys($imported_ids['template_groups']);
 					break;
 
 				case 'templates':
-
 					$options = [
-						'output' => ['templateid', 'uuid'],
+						'output' => ['templateid', 'uuid', 'name'],
 						'filter' => [
 							'uuid' => $data['uuid'],
 							'host' => $data['template']
