@@ -1,21 +1,16 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -78,7 +73,11 @@ final class CHistFunctionData {
 		],
 		'count_foreach' => [
 			['rules' => [['type' => 'query']]],
-			['rules' => [['type' => 'period', 'mode' => self::PERIOD_MODE_SEC_ONLY]]]
+			['rules' => [['type' => 'period', 'mode' => self::PERIOD_MODE_SEC_ONLY]]],
+			['rules' => [['type' => 'regexp', 'pattern' => '/^(eq|ne|gt|ge|lt|le|like|bitand|regexp|iregexp)$/']],
+				'required' => false
+			],
+			['required' => false]
 		],
 		'countunique' => [
 			['rules' => [['type' => 'query']]],
@@ -299,54 +298,112 @@ final class CHistFunctionData {
 	 * @var array
 	 */
 	private const EXPRESSION_RULES = [
-		'avg_foreach' => [[
-			'type' => 'require_math_parent',
-			'in' => ['avg', 'count', 'max', 'min', 'sum'],
-			'parameters' => ['count' => 1],
-			'position' => 0
-		]],
+		'avg_foreach' => [
+			[
+				'type' => 'require_math_parent',
+				'in' => ['avg', 'max', 'min', 'sum'],
+				'parameters' => ['count' => 1],
+				'position' => 0
+			],
+			[
+				'type' => 'require_math_parent',
+				'in' => ['count'],
+				'parameters' => ['min' => 1, 'max' => 3],
+				'position' => 0
+			]
+		],
 		'bucket_rate_foreach' => [[
 			'type' => 'require_math_parent',
 			'in' => ['histogram_quantile'],
 			'parameters' => ['count' => 2],
 			'position' => 1
 		]],
-		'count_foreach' => [[
-			'type' => 'require_math_parent',
-			'in' => ['avg', 'count', 'max', 'min', 'sum'],
-			'parameters' => ['count' => 1],
-			'position' => 0
-		]],
-		'exists_foreach' => [[
-			'type' => 'require_math_parent',
-			'in' => ['avg', 'count', 'max', 'min', 'sum'],
-			'parameters' => ['count' => 1],
-			'position' => 0
-		]],
-		'last_foreach' => [[
-			'type' => 'require_math_parent',
-			'in' => ['avg', 'count', 'max', 'min', 'sum'],
-			'parameters' => ['count' => 1],
-			'position' => 0
-		]],
-		'max_foreach' => [[
-			'type' => 'require_math_parent',
-			'in' => ['avg', 'count', 'max', 'min', 'sum'],
-			'parameters' => ['count' => 1],
-			'position' => 0
-		]],
-		'min_foreach' => [[
-			'type' => 'require_math_parent',
-			'in' => ['avg', 'count', 'max', 'min', 'sum'],
-			'parameters' => ['count' => 1],
-			'position' => 0
-		]],
-		'sum_foreach' => [[
-			'type' => 'require_math_parent',
-			'in' => ['avg', 'count', 'max', 'min', 'sum'],
-			'parameters' => ['count' => 1],
-			'position' => 0
-		]]
+		'count_foreach' => [
+			[
+				'type' => 'require_math_parent',
+				'in' => ['avg', 'max', 'min', 'sum'],
+				'parameters' => ['count' => 1],
+				'position' => 0
+			],
+			[
+				'type' => 'require_math_parent',
+				'in' => ['count'],
+				'parameters' => ['min' => 1, 'max' => 3],
+				'position' => 0
+			]
+		],
+		'exists_foreach' => [
+			[
+				'type' => 'require_math_parent',
+				'in' => ['avg', 'max', 'min', 'sum'],
+				'parameters' => ['count' => 1],
+				'position' => 0
+			],
+			[
+				'type' => 'require_math_parent',
+				'in' => ['count'],
+				'parameters' => ['min' => 1, 'max' => 3],
+				'position' => 0
+			]
+		],
+		'last_foreach' => [
+			[
+				'type' => 'require_math_parent',
+				'in' => ['avg', 'kurtosis', 'mad', 'max', 'min', 'skewness', 'stddevpop', 'stddevsamp', 'sum',
+					'sumofsquares', 'varpop', 'varsamp'
+				],
+				'parameters' => ['count' => 1],
+				'position' => 0
+			],
+			[
+				'type' => 'require_math_parent',
+				'in' => ['count'],
+				'parameters' => ['min' => 1, 'max' => 3],
+				'position' => 0
+			]
+		],
+		'max_foreach' => [
+			[
+				'type' => 'require_math_parent',
+				'in' => ['avg', 'max', 'min', 'sum'],
+				'parameters' => ['count' => 1],
+				'position' => 0
+			],
+			[
+				'type' => 'require_math_parent',
+				'in' => ['count'],
+				'parameters' => ['min' => 1, 'max' => 3],
+				'position' => 0
+			]
+		],
+		'min_foreach' => [
+			[
+				'type' => 'require_math_parent',
+				'in' => ['avg', 'max', 'min', 'sum'],
+				'parameters' => ['count' => 1],
+				'position' => 0
+			],
+			[
+				'type' => 'require_math_parent',
+				'in' => ['count'],
+				'parameters' => ['min' => 1, 'max' => 3],
+				'position' => 0
+			]
+		],
+		'sum_foreach' => [
+			[
+				'type' => 'require_math_parent',
+				'in' => ['avg', 'max', 'min', 'sum'],
+				'parameters' => ['count' => 1],
+				'position' => 0
+			],
+			[
+				'type' => 'require_math_parent',
+				'in' => ['count'],
+				'parameters' => ['min' => 1, 'max' => 3],
+				'position' => 0
+			]
+		]
 	];
 
 	/**
