@@ -1,21 +1,16 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -90,10 +85,11 @@ class CMacroValue extends CInput {
 	 * Allow to revert macro value.
 	 */
 	public function addRevertButton() {
-		$this->revert_button = (new CButton(null))
-			->setAttribute('title', _('Revert changes'))
+		$this->revert_button = (new CSimpleButton())
 			->addClass(ZBX_STYLE_BTN_ALT)
-			->addClass(self::ZBX_STYLE_BTN_UNDO);
+			->addClass(ZBX_ICON_ARROW_BACK)
+			->addClass(self::ZBX_STYLE_BTN_UNDO)
+			->setAttribute('title', _('Revert changes'));
 
 		return $this;
 	}
@@ -120,31 +116,36 @@ class CMacroValue extends CInput {
 		$name = $this->getAttribute('name');
 		$value_type = $this->getAttribute('type');
 		$value = $this->getAttribute('value');
-		$readonly = $this->getAttribute('readonly');
+		$readonly = (bool) $this->getAttribute('readonly');
 		$elements = [];
 
 		if ($value_type == ZBX_MACRO_TYPE_TEXT) {
 			$wrapper_class = self::ZBX_STYLE_MACRO_INPUT_GROUP.' '.self::ZBX_STYLE_MACRO_VALUE_TEXT;
-			$dropdown_btn_class = ZBX_STYLE_ICON_TEXT;
+			$dropdown_btn_class = ZBX_ICON_TEXT;
+
 			$elements[] = (new CTextAreaFlexible($name.'[value]', $value, ['add_post_js' => $this->add_post_js]))
 				->setMaxlength($this->maxlength)
 				->setAttribute('placeholder', _('value'))
+				->disableSpellcheck()
 				->setReadonly($readonly);
 		}
 		elseif ($value_type == ZBX_MACRO_TYPE_VAULT) {
 			$wrapper_class = self::ZBX_STYLE_MACRO_INPUT_GROUP.' '.self::ZBX_STYLE_MACRO_VALUE_VAULT;
-			$dropdown_btn_class = ZBX_STYLE_ICON_SECRET_TEXT;
+			$dropdown_btn_class = ZBX_ICON_LOCK;
+
 			$elements[] = (new CTextAreaFlexible($name.'[value]', $value, ['add_post_js' => $this->add_post_js]))
 				->setMaxlength($this->maxlength)
-				->setAttribute('placeholder', _('path/to/secret:key'))
+				->setAttribute('placeholder', _('value'))
+				->disableSpellcheck()
 				->setReadonly($readonly);
 		}
 		else {
 			$wrapper_class = self::ZBX_STYLE_MACRO_INPUT_GROUP.' '.self::ZBX_STYLE_MACRO_VALUE_SECRET;
-			$dropdown_btn_class = ZBX_STYLE_ICON_INVISIBLE;
+			$dropdown_btn_class = ZBX_ICON_EYE_OFF;
+
 			$elements[] = (new CInputSecret($name.'[value]', $value, $this->add_post_js))
 				->setAttribute('maxlength', $this->maxlength)
-				->setAttribute('disabled', ($readonly !== null) ? 'disabled' : null)
+				->setAttribute('disabled', $readonly ? 'disabled' : null)
 				->setAttribute('placeholder', _('value'));
 		}
 
@@ -153,13 +154,13 @@ class CMacroValue extends CInput {
 		}
 
 		$elements[] = (new CButtonDropdown($name.'[type]',  $value_type, [
-				['label' => _('Text'), 'value' => ZBX_MACRO_TYPE_TEXT, 'class' => ZBX_STYLE_ICON_TEXT],
-				['label' => _('Secret text'), 'value' => ZBX_MACRO_TYPE_SECRET, 'class' => ZBX_STYLE_ICON_INVISIBLE],
-				['label' => _('Vault secret'), 'value' => ZBX_MACRO_TYPE_VAULT, 'class' => ZBX_STYLE_ICON_SECRET_TEXT]
-			]))
-				->addClass($dropdown_btn_class)
-				->setAttribute('disabled', ($readonly !== null) ? 'disabled' : null)
-				->setAttribute('aria-label', _('Change type'));
+			['label' => _('Text'), 'value' => ZBX_MACRO_TYPE_TEXT, 'class' => ZBX_ICON_TEXT],
+			['label' => _('Secret text'), 'value' => ZBX_MACRO_TYPE_SECRET, 'class' => ZBX_ICON_EYE_OFF],
+			['label' => _('Vault secret'), 'value' => ZBX_MACRO_TYPE_VAULT, 'class' => ZBX_ICON_LOCK]
+		]))
+			->addClass($dropdown_btn_class)
+			->setAttribute('disabled', $readonly ? 'disabled' : null)
+			->setAttribute('aria-label', _('Change type'));
 
 		$node = (new CDiv())
 			->addClass($wrapper_class)
