@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2025 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -23,24 +18,11 @@ class CWebUser {
 
 	public static $data = null;
 
-	/**
-	 * Flag used to ignore setting authentication cookie performed checkAuthentication.
-	 */
-	static $set_cookie = true;
-
 	/**
 	 * Flag used to not to extend session lifetime in checkAuthentication.
 	 */
 	static $extend_session = true;
 
-	/**
-	 * Disable automatic cookie setting.
-	 * First checkAuthentication call (performed in initialization phase) will not be sending cookies.
-	 */
-	public static function disableSessionCookie() {
-		self::$set_cookie = false;
-	}
-
 	/**
 	 * Disable automatic session extension.
 	 */
@@ -51,19 +33,17 @@ class CWebUser {
 	/**
 	 * Tries to login a user and populates self::$data on success.
 	 *
-	 * @param string $login			user login
-	 * @param string $password		user password
+	 * @param string $login     user login
+	 * @param string $password  user password
 	 *
 	 * @throws Exception if user cannot be logged in
 	 *
 	 * @return bool
 	 */
-	public static function login($login, $password) {
+	public static function login(string $login, string $password): bool {
 		try {
-			self::setDefault();
-
 			self::$data = API::User()->login([
-				'user' => $login,
+				'username' => $login,
 				'password' => $password,
 				'userData' => true
 			]);
@@ -72,29 +52,27 @@ class CWebUser {
 				throw new Exception();
 			}
 
+			API::getWrapper()->auth = [
+				'type' => CJsonRpc::AUTH_TYPE_FRONTEND,
+				'auth' => self::$data['sessionid']
+			];
+
 			if (self::$data['gui_access'] == GROUP_GUI_ACCESS_DISABLED) {
 				error(_('GUI access disabled.'));
 				throw new Exception();
 			}
 
-			$result = (bool) self::$data;
-
-			if (isset(self::$data['attempt_failed']) && self::$data['attempt_failed']) {
+			if (self::$data['mfaid'] == 0 && isset(self::$data['attempt_failed']) && self::$data['attempt_failed']) {
 				CProfile::init();
 				CProfile::update('web.login.attempt.failed', self::$data['attempt_failed'], PROFILE_TYPE_INT);
 				CProfile::update('web.login.attempt.ip', self::$data['attempt_ip'], PROFILE_TYPE_STR);
 				CProfile::update('web.login.attempt.clock', self::$data['attempt_clock'], PROFILE_TYPE_INT);
-				$result &= CProfile::flush();
+				if (!CProfile::flush()) {
+					return false;
+				}
 			}
 
-			// remove guest session after successful login
-			$result &= DBexecute('DELETE FROM sessions WHERE sessionid='.zbx_dbstr(get_cookie(ZBX_SESSION_NAME)));
-
-			if ($result) {
-				self::setSessionCookie(self::$data['sessionid']);
-			}
-
-			return $result;
+			return true;
 		}
 		catch (Exception $e) {
 			self::setDefault();
@@ -105,87 +83,58 @@ class CWebUser {
 	/**
 	 * Log-out the current user.
 	 */
-	public static function logout() {
-		self::$data['sessionid'] = self::getSessionCookie();
-
+	public static function logout(): void {
 		if (API::User()->logout([])) {
 			self::$data = null;
-			CSession::destroy();
-			zbx_unsetcookie(ZBX_SESSION_NAME);
+			session_destroy();
 		}
 	}
 
-	public static function checkAuthentication($sessionId) {
-		try {
-			if ($sessionId !== null) {
-				self::$data = API::User()->checkAuthentication([
-					'sessionid' => $sessionId,
-					'extend' => self::$extend_session
-				]);
-			}
+	public static function checkAuthentication(string $sessionid): bool {
+		self::$data = API::User()->checkAuthentication([
+			'sessionid' => $sessionid,
+			'extend' => self::$extend_session
+		]);
 
-			if ($sessionId === null || empty(self::$data)) {
-				self::setDefault();
-				self::$data = API::User()->login([
-					'user' => ZBX_GUEST_USER,
-					'password' => '',
-					'userData' => true
-				]);
+		return self::$data && self::$data['gui_access'] != GROUP_GUI_ACCESS_DISABLED;
+	}
 
-				if (empty(self::$data)) {
-					clear_messages(1);
-					throw new Exception();
-				}
-				$sessionId = self::$data['sessionid'];
-			}
-
-			if (self::$data['gui_access'] == GROUP_GUI_ACCESS_DISABLED) {
-				throw new Exception();
-			}
-
-			if (self::$set_cookie) {
-				self::setSessionCookie($sessionId);
-			}
-			else {
-				self::$set_cookie = true;
-			}
-
-			return $sessionId;
-		}
-		catch (Exception $e) {
-			self::setDefault();
+	/**
+	 * Checks access of authenticated user to specific access rule.
+	 *
+	 * @static
+	 *
+	 * @param string $rule_name  Rule name.
+	 *
+	 * @return bool  Returns true if user has access to specified rule, false - otherwise.
+	 *
+	 * @throws Exception
+	 */
+	public static function checkAccess(string $rule_name): bool {
+		if (empty(self::$data) || self::$data['roleid'] == 0) {
 			return false;
 		}
+
+		return CRoleHelper::checkAccess($rule_name, self::$data['roleid']);
 	}
 
 	/**
-	 * Shorthand method for setting current session ID in cookies.
+	 * Sets user data defaults.
 	 *
-	 * @param string $sessionId		Session ID string
+	 * @static
 	 */
-	public static function setSessionCookie($sessionId) {
-		$autoLogin = self::isGuest() ? false : (bool) self::$data['autologin'];
-
-		zbx_setcookie(ZBX_SESSION_NAME, $sessionId,  $autoLogin ? strtotime('+1 month') : 0);
-	}
-
-	/**
-	 * Retrieves current session ID from cookie named as defined in ZBX_SESSION_NAME.
-	 *
-	 * @return string
-	 */
-	public static function getSessionCookie() {
-		return get_cookie(ZBX_SESSION_NAME);
-	}
-
-	public static function setDefault() {
+	public static function setDefault(): void {
 		self::$data = [
-			'alias' => ZBX_GUEST_USER,
+			'sessionid' => CEncryptHelper::generateKey(),
+			'username' => ZBX_GUEST_USER,
 			'userid' => 0,
-			'lang' => 'en_gb',
-			'theme' => THEME_DEFAULT,
+			'lang' => CSettingsHelper::getPublic(CSettingsHelper::DEFAULT_LANG),
+			'theme' => CSettingsHelper::getPublic(CSettingsHelper::DEFAULT_THEME),
 			'type' => 0,
-			'debug_mode' => false
+			'gui_access' => GROUP_GUI_ACCESS_SYSTEM,
+			'debug_mode' => false,
+			'roleid' => 0,
+			'autologin' => 0
 		];
 	}
 
@@ -224,7 +173,7 @@ class CWebUser {
 	 * @return bool
 	 */
 	public static function isGuest() {
-		return (self::$data && self::$data['alias'] == ZBX_GUEST_USER);
+		return (self::$data && self::$data['username'] == ZBX_GUEST_USER);
 	}
 
 	/**
@@ -235,7 +184,7 @@ class CWebUser {
 	public static function isGuestAllowed() {
 		$guest = DB::select('users', [
 			'output' => ['userid'],
-			'filter' => ['alias' => ZBX_GUEST_USER]
+			'filter' => ['username' => ZBX_GUEST_USER]
 		]);
 
 		return check_perm2system($guest[0]['userid'])
@@ -268,4 +217,13 @@ class CWebUser {
 	public static function getIp(): string {
 		return $_SERVER['REMOTE_ADDR'];
 	}
+
+	/**
+	 * Check whether user has enabled autologin.
+	 *
+	 * @return bool
+	 */
+	public static function isAutologinEnabled(): bool {
+		return (CWebUser::$data['autologin'] === '1');
+	}
 }
