@@ -1,38 +1,36 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 /**
  * @var CView $this
+ * @var array $data
  */
 
-$widget = new CWidget();
+$html_page = new CHtmlPage();
 
 if ($data['parent_discoveryid'] === null) {
-	$widget
+	$html_page
 		->setTitle(_('Graphs'))
+		->setDocUrl(CDocHelper::getUrl(CDocHelper::DATA_COLLECTION_GRAPH_EDIT))
 		->setNavigation(getHostNavigation('graphs', $data['hostid']));
 }
 else {
-	$widget
+	$html_page
 		->setTitle(_('Graph prototypes'))
+		->setDocUrl(CDocHelper::getUrl(CDocHelper::DATA_COLLECTION_PROTOTYPE_GRAPH_EDIT))
 		->setNavigation(getHostNavigation('graphs', $data['hostid'], $data['parent_discoveryid']));
 }
 
@@ -43,9 +41,10 @@ $url = (new CUrl('graphs.php'))
 
 // Create form.
 $graphForm = (new CForm('post', $url))
+	->addItem((new CVar(CCsrfTokenHelper::CSRF_TOKEN_NAME, CCsrfTokenHelper::get('graphs.php')))->removeId())
 	->addItem((new CVar('form_refresh', $data['form_refresh'] + 1))->removeId())
 	->setName('graphForm')
-	->setAttribute('aria-labelledby', ZBX_STYLE_PAGE_TITLE)
+	->setAttribute('aria-labelledby', CHtmlPage::PAGE_TITLE_ID)
 	->addVar('form', $data['form'])
 	->addVar('hostid', $data['hostid']);
 
@@ -110,7 +109,7 @@ $graphFormList
 			->setFocusableElementId('label-graphtype')
 			->setValue($data['graphtype'])
 			->addOptions(CSelect::createOptionsFromArray(graphType()))
-			->setDisabled($readonly)
+			->setReadonly($readonly)
 	)
 	->addRow(_('Show legend'),
 		(new CCheckBox('show_legend'))
@@ -137,7 +136,7 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 			->setWidth(ZBX_TEXTAREA_TINY_WIDTH);
 		$percentLeftCheckbox = (new CCheckBox('visible[percent_left]'))
 			->setChecked(true)
-			->onClick('javascript: showHideVisible("percent_left");')
+			->onClick('showHideVisible("percent_left");')
 			->setEnabled(!$readonly);
 
 		if(array_key_exists('visible', $data) && array_key_exists('percent_left', $data['visible'])) {
@@ -155,7 +154,7 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 			->setWidth(ZBX_TEXTAREA_TINY_WIDTH);
 		$percentRightCheckbox = (new CCheckBox('visible[percent_right]'))
 			->setChecked(true)
-			->onClick('javascript: showHideVisible("percent_right");')
+			->onClick('showHideVisible("percent_right");')
 			->setEnabled(!$readonly);
 
 		if(array_key_exists('visible', $data) && array_key_exists('percent_right', $data['visible'])) {
@@ -228,7 +227,6 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 					'dstfld1' => 'ymin_itemid',
 					'hostid' => $data['is_template'] ? $data['hostid'] : 0,
 					'numeric' => '1',
-					'webitems' => '1',
 					'real_hosts' => !$data['is_template']
 				]
 			]
@@ -239,17 +237,18 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 			$yaxisMinData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 			$yaxisMinData[] = (new CButton('yaxis_min_prototype', _('Select prototype')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick(
-					'return PopUp("popup.generic", '.json_encode([
-						'srctbl' => 'item_prototypes',
-						'srcfld1' => 'itemid',
-						'srcfld2' => 'name',
-						'dstfrm' => $graphForm->getName(),
-						'dstfld1' => 'ymin_itemid',
-						'parent_discoveryid' => $data['parent_discoveryid'],
-						'numeric' => '1'
-					]).', {dialogue_class: "modal-popup-generic"});'
-				)
+				->setAttribute('data-parent_discoveryid', $data['parent_discoveryid'])
+				->onClick('
+					PopUp("popup.generic", {
+						srctbl: "item_prototypes",
+						srcfld1: "itemid",
+						srcfld2: "name",
+						dstfrm: "'.$graphForm->getName().'",
+						dstfld1: "ymin_itemid",
+						parent_discoveryid: this.dataset.parent_discoveryid,
+						numeric: 1
+					}, {dialogue_class: "modal-popup-generic"});
+				')
 				->setEnabled(!$readonly);
 		}
 	}
@@ -257,7 +256,7 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 		$graphForm->addVar('yaxismin', $data['yaxismin']);
 	}
 
-	$yaxismin_label = new CLabel(_('Y axis MIN value'), 'ymin_type_label');
+	$yaxismin_label = new CLabel(_('Y axis MIN value'),'ymin_type_label');
 	if ($data['ymin_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
 		$yaxismin_label->setAsteriskMark();
 	}
@@ -322,7 +321,6 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 					'dstfld1' => 'ymax_itemid',
 					'hostid' => $data['is_template'] ? $data['hostid'] : 0,
 					'numeric' => '1',
-					'webitems' => '1',
 					'real_hosts' => !$data['is_template']
 				]
 			]
@@ -333,17 +331,18 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 			$yaxisMaxData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 			$yaxisMaxData[] = (new CButton('yaxis_max_prototype', _('Select prototype')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick(
-					'return PopUp("popup.generic", '.json_encode([
-						'srctbl' => 'item_prototypes',
-						'srcfld1' => 'itemid',
-						'srcfld2' => 'name',
-						'dstfrm' => $graphForm->getName(),
-						'dstfld1' => 'ymax_itemid',
-						'parent_discoveryid' => $data['parent_discoveryid'],
-						'numeric' => '1'
-					]).', {dialogue_class: "modal-popup-generic"});'
-				)
+				->setAttribute('data-parent_discoveryid', $data['parent_discoveryid'])
+				->onClick('
+					PopUp("popup.generic", {
+						srctbl: "item_prototypes",
+						srcfld1: "itemid",
+						srcfld2: "name",
+						dstfrm: "'.$graphForm->getName().'",
+						dstfld1: "ymax_itemid",
+						parent_discoveryid: this.dataset.parent_discoveryid,
+						numeric: 1
+					}, {dialogue_class: "modal-popup-generic"});
+				')
 				->setEnabled(!$readonly);
 		}
 	}
@@ -352,7 +351,7 @@ if ($data['graphtype'] == GRAPH_TYPE_NORMAL || $data['graphtype'] == GRAPH_TYPE_
 	}
 
 	$yaxismax_label = new CLabel(_('Y axis MAX value'), 'ymax_type_label');
-	if ($this->data['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
+	if ($data['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
 		$yaxismax_label->setAsteriskMark();
 	}
 
@@ -369,6 +368,7 @@ else {
 // Append items to form list.
 $items_table = (new CTable())
 	->setId('itemsTable')
+	->addClass(ZBX_STYLE_LIST_NUMBERED)
 	->setColumns([
 		(new CTableColumn())->addClass('table-col-handle'),
 		(new CTableColumn())->addClass('table-col-no'),
@@ -403,8 +403,7 @@ $parameters_add = [
 	'dstfrm' => $graphForm->getName(),
 	'numeric' => '1',
 	'writeonly' => '1',
-	'multiselect' => '1',
-	'with_webitems' => '1'
+	'multiselect' => '1'
 ];
 if ($data['normal_only']) {
 	$parameters_add['normal_only'] = '1';
@@ -437,16 +436,19 @@ $items_table->addRow(
 			: (new CCol(
 				new CHorList([
 					(new CButton('add_item', _('Add')))
+						->setAttribute('data-parameters', json_encode($parameters_add))
 						->onClick(
 							'return PopUp("popup.generic",
-								jQuery.extend('.json_encode($parameters_add).', view.getOnlyHostParam())
+								jQuery.extend(JSON.parse(this.dataset.parameters), view.getOnlyHostParam()),
+								{dialogue_class: "modal-popup-generic", trigger_element: this}
 							);'
 						)
 						->addClass(ZBX_STYLE_BTN_LINK),
 					$data['parent_discoveryid']
 						? (new CButton('add_protoitem', _('Add prototype')))
+							->setAttribute('data-parameters', json_encode($parameters_add_prototype))
 							->onClick(
-								'return PopUp("popup.generic", '.json_encode($parameters_add_prototype).',
+								'return PopUp("popup.generic", JSON.parse(this.dataset.parameters),
 									{dialogue_class: "modal-popup-generic"}
 								);'
 							)
@@ -495,10 +497,11 @@ if ($data['graphid'] != 0) {
 	$updateButton = new CSubmit('update', _('Update'));
 	$deleteButton = new CButtonDelete(
 		($data['parent_discoveryid'] === null) ? _('Delete graph?') : _('Delete graph prototype?'),
-		url_params(['graphid', 'parent_discoveryid', 'hostid', 'context']), 'context'
+		url_params(['graphid', 'parent_discoveryid', 'hostid', 'context']).'&'.CCsrfTokenHelper::CSRF_TOKEN_NAME.'='.
+		CCsrfTokenHelper::get('graphs.php'), 'context'
 	);
 
-	if ($readonly) {
+	if ($readonly && $data['parent_discoveryid'] === null) {
 		$updateButton->setEnabled(false);
 	}
 
@@ -531,10 +534,9 @@ require_once dirname(__FILE__).'/js/configuration.graph.edit.js.php';
 
 $graphForm->addItem($graphTab);
 
-// Append form to widget.
-$widget->addItem($graphForm);
-
-$widget->show();
+$html_page
+	->addItem($graphForm)
+	->show();
 
 (new CScriptTag('
 	view.init('.json_encode([
@@ -548,7 +550,9 @@ $widget->show();
 			'normal_only' => $data['normal_only'],
 			'parent_discoveryid' => $data['parent_discoveryid']
 		],
-		'items' => $data['items']
+		'items' => $data['items'],
+		'context' => $data['context'],
+		'parent_discoveryid' => $data['parent_discoveryid']
 	]).');
 '))
 	->setOnDocumentReady()
