@@ -1,26 +1,22 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 /**
  * @var CView $this
+ * @var array $data
  */
 ?>
 
@@ -45,7 +41,7 @@
 		(new CDiv([
 			(new CDiv(new CVar('preprocessing[#{rowNum}][sortorder]', '#{sortorder}')))->addClass(ZBX_STYLE_DRAG_ICON),
 			(new CDiv($preproc_types_select))
-				->addClass('list-numbered-item')
+				->addClass(ZBX_STYLE_LIST_NUMBERED_ITEM)
 				->addClass('step-name'),
 			(new CDiv())->addClass('step-parameters'),
 			(new CDiv(new CCheckBox('preprocessing[#{rowNum}][on_fail]')))->addClass('step-on-fail'),
@@ -76,7 +72,6 @@
 			->addStyle('display: none;')
 	]))
 		->addClass('preprocessing-list-item')
-		->addClass('sortable')
 		->setAttribute('data-step', '#{rowNum}');
 	?>
 </script>
@@ -96,11 +91,11 @@
 </script>
 
 <script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-custom-width-chkbox-tmpl">
-	<?= (new CTextBox('preprocessing[#{rowNum}][params][0]', ''))
+	<?= (new CTextBox('preprocessing[#{rowNum}][params][0]', '#{value_0}'))
 			->setAttribute('placeholder', '#{placeholder_0}')
 			->setWidth('#{width_0}')
 			->setAttribute('maxlength', 1).
-		(new CTextBox('preprocessing[#{rowNum}][params][1]', ''))
+		(new CTextBox('preprocessing[#{rowNum}][params][1]', '#{value_1}'))
 			->setAttribute('placeholder', '#{placeholder_1}')
 			->setWidth('#{width_1}')
 			->setAttribute('maxlength', 1).
@@ -110,7 +105,7 @@
 	?>
 </script>
 
-<script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-custom-prometheus-pattern">
+<script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-custom-prometheus-pattern-tmpl">
 	<?= (new CTextBox('preprocessing[#{rowNum}][params][0]', ''))
 			->setAttribute('placeholder', '#{placeholder_0}').
 		(new CSelect('preprocessing[#{rowNum}][params][1]'))
@@ -130,17 +125,161 @@
 	?>
 </script>
 
+<script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-snmp-walk-value-tmpl">
+	<?= (new CTextBox('preprocessing[#{rowNum}][params][0]', ''))->setAttribute('placeholder', _('OID')).
+		(new CSelect('preprocessing[#{rowNum}][params][1]'))
+			->setValue(ZBX_PREPROC_SNMP_UNCHANGED)
+			->setAdaptiveWidth(202)
+			->addOptions([
+				new CSelectOption(ZBX_PREPROC_SNMP_UNCHANGED, _('Unchanged')),
+				new CSelectOption(ZBX_PREPROC_SNMP_UTF8_FROM_HEX, _('UTF-8 from Hex-STRING')),
+				new CSelectOption(ZBX_PREPROC_SNMP_MAC_FROM_HEX, _('MAC from Hex-STRING')),
+				new CSelectOption(ZBX_PREPROC_SNMP_INT_FROM_BITS, _('Integer from BITS'))
+			])
+	?>
+</script>
+
+<script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-snmp-walk-to-json-tmpl">
+	<?php
+		echo (new CDiv(
+				(new CTable())
+					->addClass('group-json-mapping')
+					->setHeader(
+						(new CRowHeader([
+							new CColHeader(_('Field name')),
+							new CColHeader(_('OID prefix')),
+							new CColHeader(_('Format')),
+							(new CColHeader(_('Action')))->addClass(ZBX_STYLE_NOWRAP)
+						]))->addClass(ZBX_STYLE_GREY)
+					)
+					->addItem(
+						(new CRow([
+							new CCol(
+								(new CTextBox('preprocessing[#{rowNum}][params][]', ''))
+									->removeId()
+									->setAttribute('placeholder', _('Field name'))
+							),
+							new CCol(
+								(new CTextBox('preprocessing[#{rowNum}][params][]', ''))
+									->removeId()
+									->setAttribute('placeholder', _('OID prefix'))
+							),
+							new CCol(
+								(new CSelect('preprocessing[#{rowNum}][params][]'))
+									->setValue(ZBX_PREPROC_SNMP_UNCHANGED)
+									->setWidth(ZBX_TEXTAREA_PREPROC_TREAT_SELECT)
+									->addOptions([
+										new CSelectOption(ZBX_PREPROC_SNMP_UNCHANGED, _('Unchanged')),
+										new CSelectOption(ZBX_PREPROC_SNMP_UTF8_FROM_HEX, _('UTF-8 from Hex-STRING')),
+										new CSelectOption(ZBX_PREPROC_SNMP_MAC_FROM_HEX, _('MAC from Hex-STRING')),
+										new CSelectOption(ZBX_PREPROC_SNMP_INT_FROM_BITS, _('Integer from BITS'))
+									])
+							),
+							(new CCol(
+								(new CButtonLink(_('Remove')))
+									->addClass('js-group-json-action-delete')
+									->setEnabled(false)
+							))->addClass(ZBX_STYLE_NOWRAP)
+						]))->addClass('group-json-row')
+					)
+					->addItem(
+						(new CTag('tfoot', true))
+							->addItem(
+								(new CCol(
+									(new CButtonLink(_('Add')))->addClass('js-group-json-action-add')
+								))->setColSpan(4)
+							)
+					)
+					->setAttribute('data-index', '#{rowNum}')
+			))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR);
+	?>
+</script>
+
+<script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-snmp-walk-to-json-row-tmpl">
+	<?php
+		echo (new CRow([
+			new CCol(
+				(new CTextBox('preprocessing[#{rowNum}][params][]', ''))
+					->removeId()
+					->setAttribute('placeholder', _('Field name'))
+			),
+			new CCol(
+				(new CTextBox('preprocessing[#{rowNum}][params][]', ''))
+					->removeId()
+					->setAttribute('placeholder', _('OID prefix'))
+			),
+			new CCol(
+				(new CSelect('preprocessing[#{rowNum}][params][]'))
+					->setValue(ZBX_PREPROC_SNMP_UNCHANGED)
+					->setWidth(ZBX_TEXTAREA_PREPROC_TREAT_SELECT)
+					->addOptions([
+						new CSelectOption(ZBX_PREPROC_SNMP_UNCHANGED, _('Unchanged')),
+						new CSelectOption(ZBX_PREPROC_SNMP_UTF8_FROM_HEX, _('UTF-8 from Hex-STRING')),
+						new CSelectOption(ZBX_PREPROC_SNMP_MAC_FROM_HEX, _('MAC from Hex-STRING')),
+						new CSelectOption(ZBX_PREPROC_SNMP_INT_FROM_BITS, _('Integer from BITS'))
+					])
+			),
+			(new CCol(
+				(new CButtonLink(_('Remove')))->addClass('js-group-json-action-delete')
+			))->addClass(ZBX_STYLE_NOWRAP)
+		]))->addClass('group-json-row');
+	?>
+</script>
+
+<script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-check-not-supported-row-tmpl">
+	<?= (new CSelect('preprocessing[#{rowNum}][params][0]'))
+			->addOptions(CSelect::createOptionsFromArray([
+				ZBX_PREPROC_MATCH_ERROR_ANY => _('any error'),
+				ZBX_PREPROC_MATCH_ERROR_REGEX => _('error matches'),
+				ZBX_PREPROC_MATCH_ERROR_NOT_REGEX => _('error does not match')
+			]))
+				->setAttribute('placeholder', _('error-matching'))
+				->addClass('js-preproc-param-error-matching')
+				->setValue(ZBX_PREPROC_MATCH_ERROR_ANY).
+		(new CTextBox('preprocessing[#{rowNum}][params][1]', ''))
+			->removeId()
+			->setAttribute('placeholder', _('pattern'))
+			->addClass(ZBX_STYLE_VISIBILITY_HIDDEN);
+	?>
+</script>
+
+<script type="text/x-jquery-tmpl" id="preprocessing-steps-parameters-snmp-get-value-tmpl">
+	<?= (new CSelect('preprocessing[#{rowNum}][params][0]'))
+			->setValue(ZBX_PREPROC_SNMP_UTF8_FROM_HEX)
+			->setAdaptiveWidth(202)
+			->addOptions([
+				new CSelectOption(ZBX_PREPROC_SNMP_UTF8_FROM_HEX, _('UTF-8 from Hex-STRING')),
+				new CSelectOption(ZBX_PREPROC_SNMP_MAC_FROM_HEX, _('MAC from Hex-STRING')),
+				new CSelectOption(ZBX_PREPROC_SNMP_INT_FROM_BITS, _('Integer from BITS'))
+			])
+	?>
+</script>
+
 <script type="text/javascript">
 	jQuery(function($) {
 		function makeParameterInput(index, type) {
-			var preproc_param_single_tmpl = new Template($('#preprocessing-steps-parameters-single-tmpl').html()),
-				preproc_param_double_tmpl = new Template($('#preprocessing-steps-parameters-double-tmpl').html()),
-				preproc_param_custom_width_chkbox_tmpl =
-					new Template($('#preprocessing-steps-parameters-custom-width-chkbox-tmpl').html()),
-				preproc_param_multiline_tmpl = new Template($('#preprocessing-steps-parameters-multiline-tmpl').html()),
-				preproc_param_prometheus_pattern_tmpl = new Template(
-					$('#preprocessing-steps-parameters-custom-prometheus-pattern').html()
-				);
+			const preproc_param_single_tmpl = new Template($('#preprocessing-steps-parameters-single-tmpl').html());
+			const preproc_param_double_tmpl = new Template($('#preprocessing-steps-parameters-double-tmpl').html());
+			const preproc_param_custom_width_chkbox_tmpl =
+				new Template($('#preprocessing-steps-parameters-custom-width-chkbox-tmpl').html());
+			const preproc_param_multiline_tmpl = new Template(
+				$('#preprocessing-steps-parameters-multiline-tmpl').html()
+			);
+			const preproc_param_prometheus_pattern_tmpl = new Template(
+				$('#preprocessing-steps-parameters-custom-prometheus-pattern-tmpl').html()
+			);
+			const preproc_param_snmp_walk_value_tmpl = new Template(
+				$('#preprocessing-steps-parameters-snmp-walk-value-tmpl').html()
+			);
+			const preproc_param_snmp_walk_to_json_tmpl = new Template(
+				$('#preprocessing-steps-parameters-snmp-walk-to-json-tmpl').html()
+			);
+			const preproc_param_check_not_supported_tmpl = new Template(
+				$('#preprocessing-steps-parameters-check-not-supported-row-tmpl').html()
+			);
+			const preproc_param_snmp_get_value_tmpl = new Template(
+				$('#preprocessing-steps-parameters-snmp-get-value-tmpl').html()
+			);
 
 			switch (type) {
 				case '<?= ZBX_PREPROC_MULTIPLIER ?>':
@@ -233,8 +372,10 @@
 						rowNum: index,
 						width_0: <?= ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH ?>,
 						width_1: <?= ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH ?>,
-						placeholder_0: ',',
-						placeholder_1: '"',
+						placeholder_0: <?= json_encode(_('delimiter')) ?>,
+						value_0: ',',
+						placeholder_1: <?= json_encode(_('qualifier')) ?>,
+						value_1: '"',
 						chkbox_label: <?= json_encode(_('With header row')) ?>,
 						chkbox_value: <?= ZBX_PREPROC_CSV_HEADER ?>,
 						chkbox_default: true
@@ -247,41 +388,35 @@
 						placeholder_1: <?= json_encode(_('replacement')) ?>
 					}));
 
+				case '<?= ZBX_PREPROC_VALIDATE_NOT_SUPPORTED ?>':
+					return $(preproc_param_check_not_supported_tmpl.evaluate({
+						rowNum: index
+					}));
+
+				case '<?= ZBX_PREPROC_SNMP_WALK_VALUE ?>':
+					return $(preproc_param_snmp_walk_value_tmpl.evaluate({
+						rowNum: index
+					}));
+
+				case '<?= ZBX_PREPROC_SNMP_WALK_TO_JSON ?>':
+					return $(preproc_param_snmp_walk_to_json_tmpl.evaluate({
+						rowNum: index
+					}));
+
+				case '<?= ZBX_PREPROC_SNMP_GET_VALUE ?>':
+					return $(preproc_param_snmp_get_value_tmpl.evaluate({
+						rowNum: index
+					}));
+
 				default:
 					return '';
 			}
 		}
 
-		/**
-		 * Allow only one option with value "ZBX_PREPROC_VALIDATE_NOT_SUPPORTED" to be enabled.
-		 */
-		function updateTypeOptionsAvailability() {
-			const $preproc_steps = $('z-select[name^="preprocessing["][name$="[type]"]');
-			const $preproc_steps_ns = $preproc_steps
-				.filter((_index, {value}) => value != <?= ZBX_PREPROC_VALIDATE_NOT_SUPPORTED ?>);
-
-			if ($preproc_steps_ns.length == $preproc_steps.length) {
-				for (let select of $preproc_steps_ns) {
-					for (let option of select.getOptions()) {
-						option.disabled = false;
-					}
-				}
-				return;
-			}
-
-			for (let select of $preproc_steps_ns) {
-				for (let option of select.getOptions()) {
-					if (option.value == <?= ZBX_PREPROC_VALIDATE_NOT_SUPPORTED ?>) {
-						option.disabled = true;
-					}
-				}
-			}
-		}
-
 		var $preprocessing = $('#preprocessing');
 
 		if ($preprocessing.length === 0) {
-			const prep_elem = document.querySelector('#preprocessing_div');
+			const prep_elem = document.querySelector('#preprocessing-field');
 
 			if (!prep_elem) {
 				return false;
@@ -295,37 +430,32 @@
 			$preprocessing = $(obj.querySelector('#preprocessing'));
 		}
 
-		let step_index = $preprocessing.find('li.sortable').length;
-
-		$preprocessing.sortable({
-			disabled: $preprocessing.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').hasClass('<?= ZBX_STYLE_DISABLED ?>'),
-			items: 'li.sortable',
-			axis: 'y',
-			containment: 'parent',
-			cursor: 'grabbing',
-			handle: 'div.<?= ZBX_STYLE_DRAG_ICON ?>',
-			tolerance: 'pointer',
-			opacity: 0.6,
-			update: function() {
-				let i = 0;
-
-				$(this).find('li.sortable').each(function() {
-					$(this).find('[name*="sortorder"]').val(i++);
+		new CSortable($preprocessing[0], {
+			selector_handle: 'div.<?= ZBX_STYLE_DRAG_ICON ?>',
+			enable_sorting: $preprocessing[0].dataset.readonly == 0,
+			freeze_start: 1,
+			freeze_end: 1
+		})
+			.on(CSortable.EVENT_SORT, () => {
+				$preprocessing[0].querySelectorAll('.preprocessing-list-item').forEach((list_item, index) => {
+					list_item.querySelector('[name*="sortorder"]').value = index;
 				});
-			}
-		});
+			});
 
 		const change_event = new CustomEvent('item.preprocessing.change');
 
+		let step_index = $preprocessing.find('.preprocessing-list-item').length;
+
 		$preprocessing
 			.on('click', '.element-table-add', function() {
-				let sortable_count = $preprocessing.find('li.sortable').length;
+				let sortable_count = $preprocessing.find('.preprocessing-list-item').length;
 				const preproc_row_tmpl = new Template($('#preprocessing-steps-tmpl').html());
 				const $row = $(preproc_row_tmpl.evaluate({
 					rowNum: step_index,
 					sortorder: sortable_count++
 				}));
 				const type = $('z-select[name*="type"]', $row).val();
+				const massupdate_form = document.getElementById('massupdate-form');
 
 				$('.step-parameters', $row).html(makeParameterInput(step_index, type));
 				$(this).closest('.preprocessing-list-foot').before($row);
@@ -334,20 +464,20 @@
 
 				if (sortable_count == 1) {
 					$('#preproc_test_all').show();
-					$preprocessing
-						.sortable('disable')
-						.find('div.<?= ZBX_STYLE_DRAG_ICON ?>')
-						.addClass('<?= ZBX_STYLE_DISABLED ?>');
+
+					if (massupdate_form !== null) {
+						$preprocessing.find('button.btn-link.element-table-remove').attr('disabled', 'disabled');
+					}
 				}
 				else if (sortable_count > 1) {
-					$preprocessing
-						.sortable('enable')
-						.find('div.<?= ZBX_STYLE_DRAG_ICON ?>')
-						.removeClass('<?= ZBX_STYLE_DISABLED ?>');
+					if (massupdate_form !== null) {
+						$preprocessing.find('.preprocessing-list-item').each(function() {
+							$(this).find('button.btn-link.element-table-remove').removeAttr('disabled');
+						});
+					}
 				}
 
 				$preprocessing[0].dispatchEvent(change_event);
-				updateTypeOptionsAvailability();
 				step_index++;
 			})
 			.on('click', '#preproc_test_all', function() {
@@ -367,30 +497,29 @@
 				openItemTestDialog([num], false, false, this, num);
 			})
 			.on('click', '.element-table-remove', function() {
-				$(this).closest('li.sortable').remove();
+				$(this).closest('.preprocessing-list-item').remove();
 
-				const sortable_count = $preprocessing.find('li.sortable').length;
+				const sortable_count = $preprocessing.find('.preprocessing-list-item').length;
 
 				if (sortable_count == 0) {
 					$('#preproc_test_all').hide();
 					$('.preprocessing-list-head').hide();
 				}
 				else if (sortable_count == 1) {
-					$preprocessing
-						.sortable('disable')
-						.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').addClass('<?= ZBX_STYLE_DISABLED ?>');
+					if (document.getElementById('massupdate-form') !== null) {
+						$preprocessing.find('button.btn-link.element-table-remove').attr('disabled', 'disabled');
+					}
 				}
 
 				if (sortable_count > 0) {
 					let i = 0;
 
-					$preprocessing.find('li.sortable').each(function() {
+					$preprocessing.find('.preprocessing-list-item').each(function() {
 						$(this).find('[name*="sortorder"]').val(i++);
 					});
 				}
 
 				$preprocessing[0].dispatchEvent(change_event);
-				updateTypeOptionsAvailability();
 			})
 			.on('change', 'z-select[name*="type"]', function() {
 				var $row = $(this).closest('.preprocessing-list-item'),
@@ -427,8 +556,6 @@
 						$row.find('[name*="[test]"]').prop('disabled', false);
 						break;
 				}
-
-				updateTypeOptionsAvailability();
 			})
 			.on('change', 'input[type="text"][name*="params"]', function() {
 				$(this).attr('title', $(this).val());
@@ -469,6 +596,43 @@
 			})
 			.on('change', '.js-preproc-param-prometheus-pattern-function', function() {
 				$(this).next('input').prop('disabled', $(this).val() !== '<?= ZBX_PREPROC_PROMETHEUS_LABEL ?>');
+			})
+			.on('change', '.js-preproc-param-error-matching', function() {
+				$(this).next('input')
+					.toggleClass('<?= ZBX_STYLE_VISIBILITY_HIDDEN ?>', this.value == <?= ZBX_PREPROC_MATCH_ERROR_ANY ?>);
+			})
+			.on('click', '.js-group-json-action-delete', function() {
+				const table = this.closest('.group-json-mapping');
+				const row = this.closest('.group-json-row');
+				const count = table.querySelectorAll('.group-json-row').length;
+
+				if (count == 1) {
+					return;
+				}
+
+				row.remove();
+
+				if (count == 2) {
+					table.querySelector('.js-group-json-action-delete').disabled = true;
+				}
+			})
+			.on('click', '.js-group-json-action-add', function() {
+				const template = new Template(
+					document
+						.getElementById('preprocessing-steps-parameters-snmp-walk-to-json-row-tmpl')
+						.innerHTML
+				);
+				const container = this.closest('.group-json-mapping');
+
+				const row_numb = container.dataset.index;
+
+				[...container.querySelectorAll('.js-group-json-action-delete')].map((btn) => {
+					btn.disabled = false;
+				});
+
+				container
+					.querySelector('tbody')
+					.insertAdjacentHTML('beforeend', template.evaluate({rowNum: row_numb}));
 			});
 	});
 </script>
