@@ -1,20 +1,15 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -32,7 +27,7 @@
 function Overlay(type, dialogueid) {
 	this.type = type;
 	this.dialogueid = dialogueid || overlays_stack.getNextId();
-	this.headerid = 'dashboard-widget-head-title-' + this.dialogueid;
+	this.headerid =  'overlay-dialogue-header-title-' + this.dialogueid;
 	this.$backdrop = jQuery('<div>', {
 		'class': 'overlay-bg',
 		'data-dialogueid': this.dialogueid
@@ -46,31 +41,33 @@ function Overlay(type, dialogueid) {
 		'aria-labelledby': this.headerid
 	});
 
-	var $close_btn = jQuery('<button>', {
-			class: 'overlay-close-btn',
-			title: t('S_CLOSE')
-		})
-		.click(function(e) {
-			overlayDialogueDestroy(this.dialogueid);
-			e.preventDefault();
-		}.bind(this));
-
-	this.$dialogue.append($close_btn);
-
 	this.$dialogue.$header = jQuery('<h4>', {id: this.headerid});
+
+	const $close_btn = jQuery('<button>', {
+		class: 'btn-overlay-close',
+		title: t('S_CLOSE')
+	}).click(function(e) {
+		overlayDialogueDestroy(this.dialogueid);
+		e.preventDefault();
+	}.bind(this));
+
 	this.$dialogue.$controls = jQuery('<div>', {class: 'overlay-dialogue-controls'});
+	this.$dialogue.$head = jQuery('<div>', {class: 'overlay-dialogue-header'});
 	this.$dialogue.$body = jQuery('<div>', {class: 'overlay-dialogue-body'});
 	this.$dialogue.$debug = jQuery('<pre>', {class: 'debug-output'});
 	this.$dialogue.$footer = jQuery('<div>', {class: 'overlay-dialogue-footer'});
 	this.$dialogue.$script = jQuery('<script>');
 
-	this.$dialogue.append(jQuery('<div>', {class: 'dashboard-widget-head'}).append(this.$dialogue.$header));
+	this.$dialogue.$head.append(this.$dialogue.$header, $close_btn);
+
+	this.$dialogue.append(this.$dialogue.$head);
 	this.$dialogue.append(this.$dialogue.$body);
 	this.$dialogue.append(this.$dialogue.$footer);
 
 	this.$dialogue.$body.on('submit', 'form', function(e) {
-		if (this.$btn_submit) {
-			e.preventDefault();
+		e.preventDefault();
+
+		if (this.$btn_submit && this.$btn_submit.prop('disabled') === false) {
 			this.$btn_submit.trigger('click');
 		}
 	}.bind(this));
@@ -150,13 +147,13 @@ Overlay.prototype.recoverFocus = function() {
 	}
 
 	if (jQuery('[autofocus=autofocus]', this.$dialogue).length) {
-		jQuery('[autofocus=autofocus]', this.$dialogue).first().focus();
+		jQuery('[autofocus=autofocus]', this.$dialogue)[0].focus();
 	}
 	else if (jQuery('.overlay-dialogue-body form :focusable', this.$dialogue).length) {
-		jQuery('.overlay-dialogue-body form :focusable', this.$dialogue).first().focus();
+		jQuery('.overlay-dialogue-body form :focusable', this.$dialogue)[0].focus();
 	}
 	else {
-		jQuery(':focusable:first', this.$dialogue).focus();
+		jQuery(':focusable:first', this.$dialogue)[0].focus();
 	}
 };
 
@@ -166,34 +163,33 @@ Overlay.prototype.recoverFocus = function() {
 Overlay.prototype.containFocus = function() {
 	var focusable = jQuery(':focusable', this.$dialogue);
 
+	focusable.off('keydown.containFocus');
+
 	if (focusable.length > 1) {
 		var first_focusable = focusable.filter(':first'),
 			last_focusable = focusable.filter(':last');
 
 		first_focusable
-			.off('keydown')
-			.on('keydown', function(e) {
+			.on('keydown.containFocus', function(e) {
 				// TAB and SHIFT
 				if (e.which == 9 && e.shiftKey) {
-					last_focusable.focus();
+					last_focusable[0].focus();
 					return false;
 				}
 			});
 
 		last_focusable
-			.off('keydown')
-			.on('keydown', function(e) {
+			.on('keydown.containFocus', function(e) {
 				// TAB and not SHIFT
 				if (e.which == 9 && !e.shiftKey) {
-					first_focusable.focus();
+					first_focusable[0].focus();
 					return false;
 				}
 			});
 	}
 	else {
 		focusable
-			.off('keydown')
-			.on('keydown', function(e) {
+			.on('keydown.containFocus', function(e) {
 				if (e.which == 9) {
 					return false;
 				}
@@ -266,6 +262,8 @@ Overlay.prototype.unmount = function() {
 
 	jQuery.unsubscribe('debug.click', this.center_dialog_function);
 
+	this.unsetProperty('prevent_navigation');
+
 	this.$backdrop.remove();
 	this.$dialogue.remove();
 
@@ -275,11 +273,16 @@ Overlay.prototype.unmount = function() {
 		cancelAnimationFrame(this.center_dialog_animation_frame);
 	}
 
-	var $wrapper = jQuery('.wrapper');
+	const $wrapper = jQuery('.wrapper');
 
 	if (!jQuery('[data-dialogueid]').length) {
 		$wrapper.css('overflow', $wrapper.data('overflow'));
 		$wrapper.removeData('overflow');
+
+		$wrapper[0].scrollTo($wrapper[0].dataset.scroll_x, $wrapper[0].dataset.scroll_y);
+
+		delete $wrapper[0].dataset.scroll_x;
+		delete $wrapper[0].dataset.scroll_y;
 	}
 };
 
@@ -287,7 +290,10 @@ Overlay.prototype.unmount = function() {
  * Appends associated nodes to document body.
  */
 Overlay.prototype.mount = function() {
-	var $wrapper = jQuery('.wrapper');
+	const $wrapper = jQuery('.wrapper');
+
+	$wrapper[0].dataset.scroll_x = $wrapper[0].scrollLeft;
+	$wrapper[0].dataset.scroll_y = $wrapper[0].scrollTop;
 
 	if (!jQuery('[data-dialogueid]').length) {
 		$wrapper.data('overflow', $wrapper.css('overflow'));
@@ -396,6 +402,11 @@ Overlay.prototype.makeButtons = function(arr) {
 	return buttons;
 };
 
+Overlay.prototype.preventNavigation = function(event) {
+	event.preventDefault();
+	event.returnValue = '';
+};
+
 /**
  * @param {string} key
  */
@@ -405,6 +416,13 @@ Overlay.prototype.unsetProperty = function(key) {
 			this.$dialogue.$header.text('');
 			break;
 
+		case 'doc_url':
+			const doc_link = this.$dialogue.$head[0].querySelector('.' + ZBX_ICON_HELP_SMALL);
+			if (doc_link !== null) {
+				doc_link.remove();
+			}
+			break;
+
 		case 'buttons':
 			this.$dialogue.$footer.find('button').remove();
 			break;
@@ -427,6 +445,10 @@ Overlay.prototype.unsetProperty = function(key) {
 		case 'script_inline':
 			this.$dialogue.$script.remove();
 			break;
+
+		case 'prevent_navigation':
+			window.removeEventListener('beforeunload', this.preventNavigation);
+			break;
 	}
 };
 
@@ -451,6 +473,13 @@ Overlay.prototype.setProperties = function(obj) {
 				this.$dialogue.$header.text(obj[key]);
 				break;
 
+			case 'doc_url':
+				this.unsetProperty(key);
+				this.$dialogue.$header[0].insertAdjacentHTML('afterend', `
+					<a class="${ZBX_STYLE_BTN_ICON} ${ZBX_ICON_HELP_SMALL}" target="_blank" title="${t('Help')}" href="${obj[key]}"></a>
+				`);
+				break;
+
 			case 'buttons':
 				this.unsetProperty(key);
 				this.$dialogue.$footer.append(this.makeButtons(obj[key]));
@@ -486,6 +515,11 @@ Overlay.prototype.setProperties = function(obj) {
 				this.$dialogue.$footer.prepend(this.$dialogue.$script);
 				break;
 
+			case 'prevent_navigation':
+				this.unsetProperty(key);
+				window.addEventListener('beforeunload', this.preventNavigation, {passive: false});
+				break;
+
 			case 'element':
 				this.element = obj[key];
 				break;
